/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "osAdapt.h"

#include "idt82V3911.h"
#include "Define.h"
#include "ApllApi.h"
#include "Output.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
/**
 * @brief  Specific device profile
 */
const T_idtDrvDeviceConfig idt82V3911Config = {
	{ 0, 5, 6, 3, 4, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0 },	        /* Input port translation (external to internal) */
	{ 1, 2 },											    	/* Input sync translation */
	{ 0, 1, 2, 3, 4, 5, 6, 6, 7, 7, 0, 0 },				        /* Output port translation (external to internal) */
	{ 
	  NULL_APLL_CONFIG,
	  NULL_APLL_CONFIG,                                         /* Output port 1 APLL config */
	  NULL_APLL_CONFIG,                                         /* Output port 2 APLL config */
      NULL_APLL_CONFIG,                                         /* Output port 3 APLL config */
      NULL_APLL_CONFIG,                                         /* Output port 4 APLL config */
      NULL_APLL_CONFIG,                                         /* Output port 5 APLL config */
      {APLL_INST_1, 6, APLL_OUTPUT_QA, 6},		        	    /* Output port 6 APLL config */
      {APLL_INST_1, 6, APLL_OUTPUT_QB, 7},		        	    /* Output port 7 APLL config */
      {APLL_INST_2, 7, APLL_OUTPUT_QA, 8},		        	    /* Output port 8 APLL config */
      {APLL_INST_2, 7, APLL_OUTPUT_QB, 9},					    /* Output port 9 APLL config */
      NULL_APLL_CONFIG,                                         /* Output port 10 APLL config */
      NULL_APLL_CONFIG                                          /* Output port 11 APLL config */
    },
	{ 1, 2 },												    /* Output sync translation */
	2,														    /* Number of Sonet/GigE path */
	2,                                                          /* Number of APLLs */
    { 1, 1 },                                                   /* Both APLLs are populated */   
	{ Dpll_T0, Dpll_T4 },									    /* PLL translation (external to internal) */
    { DPLL_INSTANCE_1, DPLL_INSTANCE_2 },					    /* PLL translation (internal to external) */
	Dpll_T4,                                                    /* T0 mode */
    { 0, 0 },                                                   /* DPLL support DCO mode */
    { 0, 0 },                                                   /* DPLL support phase slope limiting */
    2,                                                          /* Maximum 2 XTALs per APLL */
    { 1, 1, 1, 1 },                                             /* All 4 XTALs are supported */   
    { 
      { 1, 1, 1 },                                              /* XTAL1 supports: 25MHz, 25.78125MHz, 24.8832MHz */
      { 1, 1, 1 },                                              /* XTAL2 supports: 25MHz, 25.78125MHz, 24.8832MHz */
      { 1, 1, 1 },                                              /* XTAL3 supports: 25MHz, 25.78125MHz, 24.8832MHz */
      { 1, 1, 1 }                                               /* XTAL4 supports: 25MHz, 25.78125MHz, 24.8832MHz */
    },
    NULL,                                                       /* No input frequency filtering */
    NULL                                                        /* No output frequency filtering */
};

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * @brief Initialize the device to its power on defaults.
 *
 * This function is optionally called. This function is required only
 * after power on to intialize device registers to know states. For
 * "warm" restarts skip this function.
 *
 * @param hdlr Device driver handle
 */
void IDTGeneral_InitDevice(T_idtDrvHdlr hdlr)
{
    T_osUint8 nIntInd, nOutputIndex;
    T_idtOutputApllConfig outputApll;

    OS_LOGGER(5, ("%s: dev/%s\n", __FUNCTION__, hdlr->name_m));
    OS_ASSERT_PTR_NULL(hdlr);

	/* De-protect all registers */
	IDTGeneral_SetFullyUnprotectMode(hdlr);

    /* Device common init */
    IDTGeneral_InitDeviceCommon(hdlr);

    /* Shut off internal outputs to improve jitter performance */
    IDTOutput_DisableAllIntOutput(hdlr);

    /* Set differential clocks out of DPLL to LVDS */
    nIntInd = IDTHal_Read(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG);
    IDTHAL_ModifyBitfield(nIntInd, BF_OUT6_PECL_LVDS_MASK,
                          BF_OUT6_PECL_LVDS_LSB, 0);
    IDTHAL_ModifyBitfield(nIntInd, BF_OUT7_PECL_LVDS_MASK,
                          BF_OUT7_PECL_LVDS_LSB, 0);
    IDTHal_Write(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG, nIntInd);

    /* Disable auto bandwidth selection */
    IDTHAL_WriteBitfield(hdlr, REG_T0_BW_OVERSHOOT_CNFG,
                         BF_AUTO_BW_SEL_MASK, BF_AUTO_BW_SEL_LSB, 0);

    /* Power saving setting */
    IDTHal_Write(hdlr, 0x03, 0x1C);

    /* Shutoff APLL if there is no XTAL configured for it */
    for (nIntInd=0; nIntInd<hdlr->deviceConfig_m->numberOfApll_m; nIntInd++)
    {
        if (!IDTApll_XtalConfigured(hdlr, nIntInd))
        {
            IDTApll_SetShutoff(hdlr, nIntInd, APLL_SHUTOFF_SHUTOFF);
            /* Traverse APLL config list and turn off internal input(s) to APLL */
            for (nOutputIndex=0;  nOutputIndex < IDT_DRV_MAX_OUTPUT; nOutputIndex++)
            {
                IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutputIndex+1, outputApll);
                if ((APLL_CONFIGURED(outputApll)) && (nIntInd == outputApll.apllInst))
                {
                    IDTOutput_DisableApllInput(hdlr, outputApll);
                }
            }
        }
    }
}

/**
 * @brief Initialize device driver.
 *
 * This function creates a driver handler used as reference to a
 * specific device. It does not perform any device access.
 *
 * The isHighLevelApi determines whether device driver initializes
 * high level APIs. 
 *
 * @param name Name of device (ex. "myDev")
 * @param drvAccess Device access data. \see drvAccess_t structure
 * information
 * @param deviceType What supported device
 * @param xtalList A list of crystal(s) connected to APLL(s),
 *                 set this parameter to NULL if there is no XTAL connected to APLL
 * @param numberOfXtal Number of crystal(s) in the xtalList,
 *                     set this parameter to 0 if xtaList is NULL
 * @param i2cAddr 7-bit unmodified i2c slave address
 * @param useHighLevelApi 0-Don't use high level API, 1-high level API
 * enabled.
 * @return Device driver handler (used in all APIs)
 */
T_idtDrvHdlr IDTGeneral_InitDriver(const char      *name, 
                                   T_idtDrvAccess  drvAccess,
                                   T_idtDeviceType deviceType,
                                   T_idtApllXtal   *xtalList,
                                   T_osUint8       numberOfXtal,
                                   T_osUint8       i2cAddr,
                                   int             useHighLevelApi)
{
	int i;
    T_idtDrvHdlr hdlr;
	static T_idtI2CtransData dpllI2CtransData = { 0, 1 };
	static T_idtI2CtransData apllI2CtransData = { 0, 0 };

    OS_ASSERT_PTR_NULL(name);
    OS_ASSERTS(deviceType == IDT_82V3911,
                ("This driver only supports 82V3911\n"));
    OS_ASSERTS(IDTApll_ValidXtalInput(&idt82V3911Config, xtalList, numberOfXtal),
                ("Invalid XTAL configuration input,"
                 " use IDTApll_GetSupportedXtal to get a list of"
                 " supported XTAL configuration for the device\n"));

	OS_LOGGER(5, ("%s: numberOfXtal/%d i2cAddr/0x%x\n",
                   __FUNCTION__, numberOfXtal, i2cAddr));

    hdlr = IDTGeneral_InitDriverCommon(name,
                                       drvAccess,
                                       IDT_82V3911,
                                       &idt82V3911Config,
                                       useHighLevelApi);

    /* Initialize XTAL list */
	hdlr->numberOfApllXtal_m = numberOfXtal;
	for (i = 0; i < IDT_DRV_MAX_APLL_XTAL; i++)
	{
		hdlr->xtalList[i].apllXtalId = APLL_XTAL_MAX;
		hdlr->xtalList[i].apllXtalFreq = APLL_XTAL_FREQ_MAX;
	}

    /* Save XTAL paramters */
	if (numberOfXtal)
	{
		for (i = 0; i < numberOfXtal; i++)
        {
            OS_LOGGER(5, ("%s: xtal id: %d, xtal frequency: %d\n",
                            __FUNCTION__, 
                            xtalList[i].apllXtalId, 
                            xtalList[i].apllXtalFreq));

            hdlr->xtalList[i].apllXtalFreq = xtalList[i].apllXtalFreq;
            hdlr->xtalList[i].apllXtalId   = xtalList[i].apllXtalId;
        }
	}

	hdlr->i2cAddrTransFunc_m  = IDTGeneral_i2cTranslate;
	hdlr->dpllI2CTransData_mp = (void *)&dpllI2CtransData;
	hdlr->apllI2CTransData_mp = (void *)&apllI2CtransData;
	hdlr->i2cAddr             = i2cAddr;

    return hdlr;
}
