/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <math.h>
#include <strings.h>

/* 82V3911 Driver include */
#include "idt82V3911.h"
#include "osAdapt.h"

#include "sample.h"
#include "inputFreq.h"
#include "outputFreq.h"
#include "profiles.h"
#include "loadstore.h"
#include "access.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
/**
 * @brief Function typedef for DPLL profile provisioning 
 */
typedef void (*T_DpllProfileFunc) (T_idtDrvHdlr      hdlr,
                                   T_idtDpllInstance nDpll);

/**
 * @brief Function typedef for sample configurations
 */
typedef void (*T_SampleConfigFunc) (T_idtDrvHdlr hdlr);

/**
 * @brief Function typedef for sample configuration descriptions
 */
typedef void (*T_SampleConfigDescrFunc) (void);

/**
 * @brief Sample configuration information 
 */
typedef struct S_SampleConfigEntry {
    T_SampleConfigFunc configFunction_m; /**< Function to configure */
    T_SampleConfigDescrFunc configDescription_m; /**< Function to output description */
} T_SampleConfigEntry;

/** 
 * @brief List of possible DPLL profiles 
 */
typedef enum {
    DpllProfile_GR253, /**< GR-253 (SONET) Stratum 3 DPLL Profile */
    DpllProfile_GR1244, /**< GR-1244 Stratum 3 DPLL Profile */
    DpllProfile_SMC, /**< SMC DPLL Profile */
    DpllProfile_G8262Option1, /**< G-8262 Option 1 DPLL Profile */
    DpllProfile_G8262Option2, /**< G-8262 Option 2 DPLL Profile */
    DpllProfile_G813Option1, /**< G-813 Option 1 DPLL Profile */
    DpllProfile_G813Option2, /**< G-813 Option 2 DPLL Profile */
    DpllProfile_1PPS, /**< 1 PPS DPLL Profile */
    DpllProfile_MAX /**< \internal Not used */
} T_DpllProfile;

/** No-op mode */
#define MODE_None   (0)

/** Load from file mode */
#define MODE_Load   (1<<0)

/** Save to file mode */
#define MODE_Save   (1<<1)

/** Provision input port mode */
#define MODE_Input  (1<<2)

/** Provision output port mode */
#define MODE_Output (1<<3)

/** Provision DPLL profile mode */
#define MODE_DpllProfile (1<<4)

/** Sample configuration mode */
#define MODE_Sample (1<<5)

/** Read register mode */
#define MODE_Read (1<<6)

/** Read register mode */
#define MODE_Write (1<<7)

/** Max filename size */
#define SAMPLE_MAX_FILENAME 50
/** 
 * @brief Structure to define program command line arguments
 */
typedef struct {
    T_osUint32        modes_m;                 /**< Operation modes
                                                 * for prgram */  
    char               filename_ma[SAMPLE_MAX_FILENAME];         /**< File name for
                                                 * load/store */
    T_osUint32        inputFreq_m[IDT_DRV_MAX_INPUT]; /**< Input
                                                        * freqency
                                                        * (kHz) */
    outputFrequency_t outFreq_m[IDT_DRV_MAX_OUTPUT];  /**< Output
                                                        * frequency
                                                        * (enum) */
    T_idtDpllType     dpll_m[IDT_DRV_MAX_OUTPUT];     /**< Select
                                                        * DPLL T0 /
                                                        * T4 */  
    T_osUint8         isSim_m;                 /**< Switch to
                                                 * similated mode */
    T_osUint8         dcoMode_m;               /**< Enable/Disable
                                                 * DCO mode */
    long              offset_m;                /**< DCO Offset */
    T_osUint8         numberOfXtal;	/**< Number APLL external Xtals */
    T_idtApllXtal     *xtalList; 	/**< APLL external Xtal list */
    T_DpllProfile     dpllProfile_ma[DPLL_INSTANCE_MAX]; /**< Dpll
                                                           * Profile */
    T_osUint8         sampleConfig_m; /**< Sample configuration */
    T_osUint8         regOffset_m; /**< register offset */
    T_osUint8         regValue_m; /**< register value */
} globalArgs_t;

#define no_argument       0
#define required_argument 1
#define optional_argument 2

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/* 7-bit unshifted i2c slave address for 82V3911 on the IDT eval board */
static T_osUint8 i2cSlaveAddr = 0x50;

/** Global command line argument variable */
globalArgs_t globalArgs;


/** Command line option string */
static const char *optString = "l:s:i:o:df:?hp:s:w:r:x:n:";

/** Detailed command line options paramters */
static const struct option longOpts[] = {
    { "load",   required_argument, NULL, 'l' },
    { "save",   required_argument, NULL, 's' },
    { "input",  required_argument, NULL, 'i' },
    { "output", required_argument, NULL, 'o' },
    { "help",   no_argument,       NULL, 'h' },
    { "i2c",    no_argument,       NULL, 'e' },
    { "sim",    no_argument,       NULL, 'm' },
    { "dco",    no_argument,       NULL, 'd' },
    { "offset", required_argument, NULL, 'f' },
    { "profile", required_argument, NULL, 'p' },
    { "sample", required_argument, NULL, 'a' },
    { "write",  required_argument, NULL, 'w' },
    { "read",   required_argument, NULL, 'r' },
    { "apllXtalList", required_argument, NULL, 'x' },
    { "numXtal", required_argument, NULL, 'n' },
    { NULL,     no_argument,       NULL,  0 }
};

/**
 * @brief Table to list DPLL profile functions 
 */
static const T_DpllProfileFunc ProfileFunc_ca[DpllProfile_MAX] = {
    IDTHlApi_gr253SonetStratum3, /* DpllProfile_GR253 */
    IDTHlApi_gr1244Stratum3, /* DpllProfile_GR1244 */
    IDTHlApi_smc, /* DpllProfile_SMC */
    IDTHlApi_g8262EecOption1, /* DpllProfile_G8262Option1 */
    IDTHlApi_g8262EecOption2, /* DpllProfile_G8262Option2 */
    IDTHlApi_g813Option1, /* DpllProfile_G813Option1 */
    IDTHlApi_g813Option2, /* DpllProfile_G813Option2 */
    IDTHlApi_pps /* DpllProfile_1PPS */
};


/**
 * @brief Table to list sample configurations functions 
 */
static const T_SampleConfigEntry SampleConfigurationTable[] = {
    { IDTSample_ConfigureSampleConfig1, 
      NULL },
    { IDTSample_ConfigureSampleConfig2, 
      NULL },
    { IDTSample_ConfigureSampleConfig3, 
      NULL },
    { IDTSample_ConfigureSampleConfig4, 
      NULL },
    { IDTSample_ConfigureSampleConfig5, 
      NULL },
    { IDTSample_ConfigureSampleConfig6, 
      NULL },
    { IDTSample_ConfigureSampleConfig7,
      NULL },
};

static const int SampleConfigurationTableSize_c = 
    (sizeof(SampleConfigurationTable)/sizeof(T_SampleConfigEntry));

void IDTSample_DumpRegistersRgf (char *filename);

extern int simDebugFlag;
extern const char *IDTHlApi_Frequency2String_c[OutFreq_MAX];

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
void printHelp(void)
{
    printf(
           "\nUsage:  drv82V3911 [OPTION]\n\n"
           "82V3911 Device driver sample application\n"
           "\n"
           "-?                show this page\n"
           "--load FILE\n"
           "--save FILE\n"
           "\n");
    printf("--input [INPORT] [INFREQ] Provision input port.\n"
           "                         Where parameters are:\n"
           "                           [INPORT] = 1..16\n"
           "                           [INFREQ] in kHz\n");

    printf("--profile [DPLL_INSTANCE] [PROFILE] Provision\n"
           "                          Where parameters are:\n"
           "                           [DPLL_INSTANCE] = 1 or 2\n"
           "                           [PROFILE] = enum type:\n"
           "      0- GR-253 (SONET) Stratum 3        4- G-8262 Option 2\n"
           "      1- GR-1244 Stratum 3               5- G-813 Option 1\n"
           "      2- SMC                             6- G-813 Option 2\n"
           "      3- G-8262 Option 1                 7- 1 PPS\n");

    printf("--output [OUTPORT] [OUTFREQ] [DPLL_INSTANCE] Provision\n"
           "                          output port. Where\n"
           "                          parameters are:\n"
           "                           [OUTPORT] = 1..7\n"
           "                           [OUTFREQ] = enum type:\n");

    printf("      0- 64 kHz              1- 8 kHz               2- 2 kHz\n"
           "      3- 400 Hz              4- 1 Hz (1 PPS)        5- 24.576 MHz\n"
           "      6- 12.288 MHz          7- 6.144 MHz           8- 4.096 MHz\n"
           "      9- 2.048 MHz          10- 32.768 MHz         11- 16.384MHz\n");
    printf("     12- 8.192 MHz          13- 24.704 MHz         14- 12.352 MHz\n"
           "     15- 6.176 MHz          16- 3.088 MHz          17- 1.544 MHz\n"
           "     18- 34.368 MHz         19- 44.736 MHz         20- 26 MHz\n"
           "     21- 13 MHz             22- 30.72 MHz          23- 15.36 MHz\n");
    printf("     24- 7.68 MHz           25- 3.84 MHz           26- 37.056 MHz\n"
           "     27- 18.528 MHz         28- 9.264 MHz          29- 4.632 MHz\n"
           "     30- 40 MHz             31- 20 MHz             32- 10 MHz\n"
           "     33- 5 MHz              34- 622.08 MHz         35- 625 MHz\n");
    printf("     36- 625*(66/64) MHz    37- 311.04 MHz         38- 312.5 MHz\n"
           "     39- 312.5*(66/64) MHz  40- 155.52 MHz         41- 156.25 MHz\n"
           "     42- 156.25*(66/64) MHz 43- 77.76 MHz          44- 51.84 MHz\n"
           "     45- 38.88 MHz          46- 25.92 MHz          47- 125 MHz\n");
    printf("     48- 125*(66/64) MHz    49- 19.44 MHz          50- 25 MHz\n"
           "     51- 25*(66/64) MHz     52- 6.48 MHz\n\n"
           "                           [DPLL_INSTANCE] = 1 (T0) or 2 (T4)\n");
    printf("--sample [SAMPLE] Provision device using a sample configuration\n");
    printf("--write [OFFSET] [VAL] Write to register\n");
    printf("--read [OFFSET] Read from register\n");
    printf("--apllXtalList [[XTAL ID],[XTAL FREQ]_..[XTAL ID],[XTAL FREQ]]\n"
           "    Where XTAL ID = 1..4, 1 and 3 connect to APLL 1,\n" 
           "                          2 and 4 connect to APLL 2.\n"
           "	  XTAL_FREQ = 1..3, 1 - 25MHz, 2 - 24.8832MHz, 3 - 25.78125MHz\n");
}

static void processXtalConfigString(T_idtApllXtal **apllXtalList, char *configString)
{
	char *fieldDelimiter = ",";
	char *fieldStringXtalId = NULL;
	char *fieldStringXtalFreq = NULL;
	char *fieldDelLocation = NULL;
	T_osUint8 xtalId;
	T_osUint8 xtalFreq;
	T_idtApllXtalId	apllXtalId;
	T_idtApllXtalType apllXtalFreq;

	if (!configString)
    {
        printf("Empty APLL xtal config string\n");
        exit(0);
    }
	
	fieldDelLocation = strstr(configString, fieldDelimiter);
	fieldStringXtalId = configString;
	fieldStringXtalFreq = fieldDelLocation+1;
	*fieldDelLocation = 0;

	xtalId = (T_osUint8)atoi(fieldStringXtalId);
	switch (xtalId)
    {
        case 1:
			apllXtalId = APLL_XTAL_1_APLL1;
        break;

		case 2:
			apllXtalId = APLL_XTAL_2_APLL2;
        break;

		case 3:
			apllXtalId = APLL_XTAL_3_APLL1;
        break;

		case 4:
			apllXtalId = APLL_XTAL_4_APLL2;
        break;

		default:
			printf("Invalid APLL xtal id: %d\n", xtalId);
			exit(0);
        break;
    }

	xtalFreq = (T_osUint8)atoi(fieldStringXtalFreq);
	switch (xtalFreq)
    {
		case 1:
			apllXtalFreq = APLL_XTAL_FREQ_25000KHz;
        break;

		case 2:
			apllXtalFreq = APLL_XTAL_FREQ_25781_DOT_25KHz;
        break;

		case 3:
			apllXtalFreq = APLL_XTAL_FREQ_24883_DOT_2KHz;
        break;

		default:
			printf("Invalid APLL xtal frequency id: %d\n", xtalFreq);
			exit(0);
        break;
    }
	
	*apllXtalList = realloc(*apllXtalList, sizeof(T_idtApllXtal)*(globalArgs.numberOfXtal+1));
	(*apllXtalList)[globalArgs.numberOfXtal].apllXtalId = apllXtalId;
	(*apllXtalList)[globalArgs.numberOfXtal].apllXtalFreq = apllXtalFreq;
	globalArgs.numberOfXtal++;
}

static T_idtApllXtal* processXtalListInput(char* xtalListInput)
{
	char configDelimiter = '_';
	char *configString = NULL;
	char *cfgDelLocation = NULL;
	T_idtApllXtal *apllXtalList = NULL;
	globalArgs.numberOfXtal = 0;

	
	if (!xtalListInput)
    {
        printf("APLL xtal list is empty\n");
        return NULL;
    }
	
	while ((cfgDelLocation = strchr(xtalListInput, configDelimiter)))
    {
        if (cfgDelLocation)
        {
            /* Found a new set of configuration */
            if (globalArgs.numberOfXtal > 4)
            {
                printf("Maxium of number of APLL xtal is 4, the rest of the configuration is ignored\n");
                return apllXtalList;
            }
			
            configString = xtalListInput;
            xtalListInput = cfgDelLocation+1;
            *cfgDelLocation = 0;
            processXtalConfigString(&apllXtalList, configString);
        }
    }

	/* process the last remaining config string */
	if (xtalListInput)
		processXtalConfigString(&apllXtalList, xtalListInput);

#ifdef DEBUG_DRV_CLI
	{
		int i;
		printf("%d xtals configured\n", globalArgs.numberOfXtal);
		for (i = 0; i < globalArgs.numberOfXtal; i++)
        {
            printf("xtal id: %d\n", apllXtalList[i].apllXtalId);
            printf("xtal freq: %d\n", apllXtalList[i].apllXtalFreq);
        }
	}
#endif

	return apllXtalList;
}

int processCmdLineArguments(int argc, char *argv[], globalArgs_t *gblArgs)
{
    int opt;
    int longIndex;

    /* Initialize global arguments variable */
    gblArgs->modes_m = MODE_None;
    memset(gblArgs->filename_ma,0,SAMPLE_MAX_FILENAME);
    memset(gblArgs->inputFreq_m,0,IDT_DRV_MAX_INPUT);
    memset(gblArgs->outFreq_m,  0,IDT_DRV_MAX_OUTPUT);
    memset(gblArgs->dpll_m,     0,IDT_DRV_MAX_OUTPUT);
    gblArgs->isSim_m      = 0;
    gblArgs->dcoMode_m    = 0;
    gblArgs->offset_m     = 0;
    gblArgs->dpllProfile_ma[DPLL_INSTANCE_1] = DpllProfile_MAX;
    gblArgs->dpllProfile_ma[DPLL_INSTANCE_2] = DpllProfile_MAX;

    /* parse command line arguments */
    opt = getopt_long( argc, argv, optString, longOpts, &longIndex );
    while( opt != -1 ) {
        switch (opt) {

            case 'h':
            case '?':
                printHelp();
                exit(1);
            break;

            /* Load Mode */
            case 'l':
                if (gblArgs->modes_m == MODE_None)
                    gblArgs->modes_m = MODE_Load;
            if (strlen(optarg)>(SAMPLE_MAX_FILENAME-1)) {
                printf("Filename is too large %ld (max %d)\n",
                       (long int)(strlen(optarg)),
                       (SAMPLE_MAX_FILENAME-1));
                exit(0);
            }
            strncpy(gblArgs->filename_ma, optarg, SAMPLE_MAX_FILENAME-1);
            break;

                /* Save mode */
            case 's':
                if (gblArgs->modes_m == MODE_None)
                    gblArgs->modes_m = MODE_Save;
            if (strlen(optarg)>(SAMPLE_MAX_FILENAME-1)) {
                printf("Filename is too large %ld (max %d)\n",
                       (long int)(strlen(optarg)),
                       (SAMPLE_MAX_FILENAME-1));
                exit(0);
            }
            strncpy(gblArgs->filename_ma, optarg, SAMPLE_MAX_FILENAME-1);
            break;

                /* Input provisioning */
            case 'i':
            {
                int inputPort;

                gblArgs->modes_m |= MODE_Input;

                inputPort = (T_osUint8)atoi(optarg);

                if ((inputPort>0) && (inputPort<=IDT_DRV_MAX_INPUT)) {

                    /* Convert port to index */
                    inputPort--;

                    gblArgs->inputFreq_m[inputPort] =
                        (T_osUint32)strtol(
                            optarg+strlen(optarg)+1,
                            NULL,
                            0);
                }
            }
            break;

            /* Output provisioning */
            case 'o':
            {
                char *nextArg;
                int outputPort;

                gblArgs->modes_m |= MODE_Output;

                nextArg = optarg;
                outputPort = (T_osUint8)atoi(nextArg);

                if ((outputPort>0) && (outputPort<=IDT_DRV_MAX_OUTPUT)) {

                    /* Convert port to index */
                    outputPort--;

                    nextArg += strlen(nextArg)+1;
                    gblArgs->outFreq_m[outputPort] =
                        (T_osUint8)atoi(nextArg);

                    nextArg += strlen(nextArg)+1;
                    gblArgs->dpll_m[outputPort] =
                        (T_osUint8)atoi(nextArg);
                }
            }
            break;

            /* I2C Access */
            case 'e':
            {
                gblArgs->isSim_m = 0;
            }
            break;

            /* Simulated Access */
            case 'm':
            {
                gblArgs->isSim_m = 1;
            }
            break;

            /* DCO Mode */
            case 'd':
                gblArgs->dcoMode_m = 1;
            break;

            /* Frequency offset */
            case 'f':
                gblArgs->offset_m = strtol(optarg, NULL, 10);
            break;

            case 'n':
                gblArgs->numberOfXtal = strtol(optarg, NULL, 10);
                break;

            case 'x':
                gblArgs->xtalList = processXtalListInput(optarg);
            break;

            /* DPLL Profiles */
            case 'p':
            {
                T_osUint8 dpllInstance;
                char *nextArg;

                gblArgs->modes_m |= MODE_DpllProfile;

                nextArg = optarg;
                dpllInstance = (T_osUint8)atoi(nextArg);

                if ((dpllInstance>=1) && (dpllInstance<=2)) {

                    dpllInstance--;
                    nextArg += strlen(nextArg)+1;
                    gblArgs->dpllProfile_ma[dpllInstance] =
                        (T_DpllProfile)atoi(nextArg);
                }
            }
            break;

            /* Sample Configuration */
            case 'a':
            {
                gblArgs->modes_m = MODE_Sample;
                gblArgs->sampleConfig_m = (T_osUint8)atoi(optarg);
                if ((gblArgs->sampleConfig_m < 1) ||
                    (gblArgs->sampleConfig_m >  SampleConfigurationTableSize_c)) {
                    printf("Sample configurations [1..%d]\n", SampleConfigurationTableSize_c);
                    return -1;
                }
            }
            break;

            case 'w':
            {
                gblArgs->modes_m = MODE_Write;
                gblArgs->regOffset_m = (T_osUint8)strtol(optarg,NULL,0);
                gblArgs->regValue_m = (T_osUint8)strtol(argv[optind],NULL,0);
            }
            break;

            case 'r':
            {
                gblArgs->modes_m = MODE_Read;
                gblArgs->regOffset_m = (T_osUint8)strtol(optarg,NULL,0);
            }
            break;

            default:
                /* Shouldn't be here. */
            break;
        }

        opt = getopt_long( argc, 
                           argv, 
                           optString, 
                           longOpts, 
                           &longIndex );
    }

    return 0;
}

int main(int argc, char *argv[])
{
    T_idtDrvHdlr hdlr;
    T_idtApllXtalList xtalList;

    /* Process command line options */
    if (processCmdLineArguments(argc,argv,&globalArgs) != 0)
    {
        return -1;
    }

    /* Clear XTAL list */
    IDTSample_ResetXtalList(&xtalList);
    switch (globalArgs.sampleConfig_m) {
        case 1: /* IDTSample_ConfigureSampleConfig1 */
        case 4: /* IDTSample_ConfigureSampleConfig4 */
        case 5: /* IDTSample_ConfigureSampleConfig5 */
            xtalList.xtal1Apll1 = 1;
            xtalList.xtal1Apll1Freq = APLL_XTAL_FREQ_25000KHz;
        break;

        case 2: /* IDTSample_ConfigureSampleConfig2 */
            xtalList.xtal1Apll1 = 1;
            xtalList.xtal1Apll1Freq = APLL_XTAL_FREQ_24883_DOT_2KHz;
            xtalList.xtal2Apll2 = 1;
            xtalList.xtal2Apll2Freq = APLL_XTAL_FREQ_25000KHz;
        break;

        case 3: /* IDTSample_ConfigureSampleConfig3 */
            xtalList.xtal1Apll1 = 1;
            xtalList.xtal1Apll1Freq = APLL_XTAL_FREQ_25000KHz;
            xtalList.xtal2Apll2 = 1;
            xtalList.xtal2Apll2Freq = APLL_XTAL_FREQ_24883_DOT_2KHz;
        break;

        case 6: /* IDTSample_ConfigureSampleConfig6 */
            xtalList.xtal1Apll1 = 1;
            xtalList.xtal1Apll1Freq = APLL_XTAL_FREQ_24883_DOT_2KHz;
        break;

        case 7: /* IDTSample_ConfigureSampleConfig7 */
            xtalList.xtal1Apll1 = 1;
            xtalList.xtal1Apll1Freq = APLL_XTAL_FREQ_25000KHz;
        break;

        default:
            /* Do nothing */
        break;
    }

    /* Initialize driver */
    OS_SETLOGLVL(5);
    if (globalArgs.isSim_m)
        hdlr = IDTSample_InitDriverSimulator(&xtalList,
                                             i2cSlaveAddr);
    else
        hdlr = IDTSample_InitDriverSimulator(&xtalList,
                                       i2cSlaveAddr);

    if (globalArgs.modes_m & MODE_Read)
    {
        T_osUint8 regVal;
        regVal = IDTHal_Read(hdlr,
                            globalArgs.regOffset_m);

        printf("READ  0x%02X = 0x%02X\n",
                globalArgs.regOffset_m,
                regVal);

        /* Deinitialize driver */
        IDTGeneral_DeInitDriver(hdlr);
        return 0;
    }
        
    if (globalArgs.modes_m & MODE_Write)
    {
        T_osUint8 regVal;

        IDTHal_Write(hdlr,
                    globalArgs.regOffset_m,
                    globalArgs.regValue_m);
        regVal = IDTHal_Read(hdlr,
                            globalArgs.regOffset_m);
        if (regVal == globalArgs.regValue_m)
            printf("WRITE 0x%02X = 0x%02X\n",
                    globalArgs.regOffset_m,
                    globalArgs.regValue_m);
        else
            printf("WRITE FAILED 0x%02X = 0x%02X (0x%02X)\n",
                    globalArgs.regOffset_m,
                    globalArgs.regValue_m,
                    regVal);

        /* Deinitialize driver */
        IDTGeneral_DeInitDriver(hdlr);
        return 0;
    }
 
    /* Initialize 82V3911 device */
    IDTGeneral_InitDevice(hdlr);

    if (globalArgs.modes_m & MODE_Input)
    {
        T_osUint8 inputPort;
        T_osUint8 lockStatus;

        for (inputPort=0; inputPort<IDT_DRV_MAX_INPUT; inputPort++)
        {
            if (globalArgs.inputFreq_m[inputPort] != 0)
                IDTHlApi_ConfigureInputFrequency(hdlr, 
                                                 inputPort+1,
                                                 globalArgs.inputFreq_m[inputPort]);

        }
        printf("Waiting 20 seconds\n");
        OS_MSEC_DELAY (20);
        lockStatus = IDTDpll_IsDpllLocked(hdlr, DPLL_INSTANCE_1);

        printf("T0 status = %s(%d) 0x%02X\n",
               lockStatus?"LOCKED":"UNLOCKED",lockStatus,
               IDTHal_Read(hdlr, REG_OPERATING_STS));
    }

    if (globalArgs.modes_m & MODE_DpllProfile)
    {
        T_idtDpllInstance i;

        for (i=DPLL_INSTANCE_1; i<DPLL_INSTANCE_MAX; i++)
        {
          T_idtDpllType dpllType;

          dpllType = ((T_idtDpllType)
                      hdlr->deviceConfig_m->pllExt2IntXlate_ma[i]);
          if (dpllType != Dpll_MAX)
          {
              T_osUint8 profileIdx;

              profileIdx = globalArgs.dpllProfile_ma[i];
              if (profileIdx != DpllProfile_MAX)
                  ProfileFunc_ca[profileIdx](hdlr,i);
          }
        }
    }

    if (globalArgs.modes_m & MODE_Output)
    {
        T_osUint8 outputPort;

        for (outputPort=0; outputPort<IDT_DRV_MAX_OUTPUT; outputPort++)
        {
#if 0
            if (globalArgs.outFreq_m[outputPort] != 0)
                IDTHlApi_ConfigureOutput(
                    hdlr,
                    outputPort,
                    globalArgs.outFreq_m[outputPort],
                    DPLL_INSTANCE_1,
                    0); /* Don't constrain to only
                         * this DPLL */
#endif
        }

    }

    if (globalArgs.modes_m & MODE_Sample)
    {
        printf("Sample configuration %d\n",globalArgs.sampleConfig_m);
        if (SampleConfigurationTable[globalArgs.sampleConfig_m-1].
            configFunction_m)
            SampleConfigurationTable[globalArgs.sampleConfig_m-1].
                configFunction_m(hdlr);
    }

    if (globalArgs.modes_m & MODE_Save)
    {
        if (strlen(globalArgs.filename_ma))
            IDTSample_SaveRgfFile(hdlr,globalArgs.filename_ma);
        else
            IDTSample_SaveRgfFile(hdlr,NULL);
    }
    
    if (globalArgs.modes_m & MODE_Load)
    {
        IDTSample_LoadRgfFile(hdlr,globalArgs.filename_ma);
    }
    

    /* Display output configuration for all output ports */
    {
        /* Surpess log messages */
        OS_SETLOGLVL(0);
        IDTHlApi_PrintInput(hdlr);
        IDTHlApi_PrintOutput(hdlr);
    }

    /* Deinitialize driver */
    IDTGeneral_DeInitDriver(hdlr);

    return 0;
}
