/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#include <stdio.h>
#include <string.h>

#include "Hal.h"

#include "ApllRegDef.h"
#include "ApllTypeDef.h"
#include "ApllApi.h"
#include "ApllLogging.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
#define APLL_DIVIDER_LSB(val)			((val)&0x00FF)
#define APLL_DIVIDER_MSB(val)			(((val)&0x7F00)>>8)
#define APLL_DIVIDER_VAL(lsb, msb)	    ((msb)<<8|(lsb))

#define APLL_PRE_DIV_LSB(val)		APLL_DIVIDER_LSB(val)
#define APLL_PRE_DIV_MSB(val)		APLL_DIVIDER_MSB(val)
#define APLL_PRE_DIV_VAL(lsb, msb)	APLL_DIVIDER_VAL(lsb, msb)
#define APLL_FB_DIV_LSB(val)		APLL_DIVIDER_LSB(val)
#define APLL_FB_DIV_MSB(val)		APLL_DIVIDER_MSB(val)
#define APLL_FB_DIV_VAL(lsb, msb)	APLL_DIVIDER_VAL(lsb, msb)

#define APLL_PRE_DIV_MIN			(0x0004)
#define APLL_PRE_DIV_MAX			(0x7FFF)
#define APLL_FB_DIV_MIN			    (0x0005)
#define APLL_FB_DIV_MAX			    (0x7FFF)

#define APLL1_REG_HIGH_BIT		    (0)
#define APLL2_REG_HIGH_BIT		    (1)
#define APLL_REG_HIGH_BIT_OFFSET    (7)

/**
 * @brief Utility macro to translate  APLL register address
 * @param apllInstId APLL instance
 * @param base APLL register base address
 */
#define APLL_REG_OFFSET(apllInstId, base) \
	(apllInstId == APLL_INST_1) ? (base) : \
    (base|(APLL2_REG_HIGH_BIT<<APLL_REG_HIGH_BIT_OFFSET))

/**
 * @brief Utility macro to sanity check  APLL #1 register address
 * @param offset APLL register address
 */
#define APLL1_REG_VALID_OFFSET(offset) \
	((offset >= REG_CONTROL) && (offset <= REG_CONFIG_QB))

/**
 * @brief Utility macro to translate  APLL  #2 register address
 * @param offset APLL register base address
 */
#define APLL2_REG_VALID_OFFSET(offset) \
        (offset >= APLL_REG_OFFSET(APLL_INST_2, REG_CONTROL)) && \
        (offset <= APLL_REG_OFFSET(APLL_INST_2, REG_CONFIG_QB))

#define INVALID_XTAL_SEL_VALUE	(0xFF)

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

/**
 * @brief Function to verify APLL instance Id
 * @param apllInstId APLL instance
 * @return E_osTrue - valid APLL instance Id, E_osFalse - invalid APLL instance Id
 */
static T_osBool IDTApll_validApllInstId(T_idtApllId apllInstId)
{
	IDT_APLL_API_TRACE(("Apll id: %d\n", apllInstId));

    if (apllInstId >= APLL_INST_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll id: %d\n", apllInstId));
        return E_osFalse;
    }
    else
        return E_osTrue;
}

/**
 * @brief Function to translate XTAL register bit field to XTAL enum Id
 * @param apllInstId APLL instance
 * @param field APLL XTAL bitfield
 * @return APLL XTAL Id<ul>
 *                <li>APLL_XTAL_1_APLL1 - XTAL1 for APLL1</li>
 *                <li>APLL_XTAL_2_APLL2 - XTAL2 for APLL2</li>
 *                <li>APLL_XTAL_3_APLL1 - XTAL3 for APLL1</li>
 *                <li>APLL_XTAL_4_APLL2 - XTAL4 for APLL2</li>
 *                <li>APLL_XTAL_MAX - Invalid XTAL Id</li></ul>
 */
static T_idtApllXtalId IDTApll_xtalBitField2xtalType(T_idtApllId apllInstId, T_idtApllRegVal field)
{
	IDT_APLL_API_TRACE(("Apll id: %d, "
					"Apll xtal id bit field: %d\n",
					apllInstId, field));

	if (APLL_INST_1 == apllInstId)
	{
		if (0 == field)
			return APLL_XTAL_1_APLL1;
		else if (1 == field)
			return APLL_XTAL_3_APLL1;
		else
		{
			IDT_APLL_API_LOG_ERR(("Invalid xtal selection field value: %d\n", field));
			return APLL_XTAL_MAX;
		}
	}
	else if (APLL_INST_2 == apllInstId)
	{
		if (0 == field)
			return APLL_XTAL_2_APLL2;
		else if (1 == field)
			return APLL_XTAL_4_APLL2;
		else
		{
			IDT_APLL_API_LOG_ERR(("Invalid xtal selection field value: %d\n", field));
			return APLL_XTAL_MAX;
		}
	}
	else
		IDT_APLL_API_LOG_ERR(("Invalid apll id: %d\n", apllInstId));

	return APLL_XTAL_MAX;
}

/**
 * @brief Function to translate  XTAL enum Id to XTAL register bit field
 * @param apllInstId APLL instance
 * @param APLL XTAL Id<ul>
 *                <li>APLL_XTAL_1_APLL1 - XTAL1 for APLL1</li>
 *                <li>APLL_XTAL_2_APLL2 - XTAL2 for APLL2</li>
 *                <li>APLL_XTAL_3_APLL1 - XTAL3 for APLL1</li>
 *                <li>APLL_XTAL_4_APLL2 - XTAL4 for APLL2</li></ul>
 * @return XTAL register bit field
 */
static T_idtApllRegVal IDTApll_xtalType2xtalBitField(T_idtApllId apllInstId, T_idtApllXtalId xtalId)
{
	IDT_APLL_API_TRACE(("Apll id: %d, Apll xtal id: %d\n",
					apllInstId, xtalId));

	if (APLL_INST_1 == apllInstId)
	{
		if (APLL_XTAL_1_APLL1 == xtalId)
			return 0;
		else if (APLL_XTAL_3_APLL1 == xtalId)
			return 1;
		else
		{
			IDT_APLL_API_LOG_ERR(("Invalid xtal id: %d for apll: %d \n", xtalId, apllInstId));
			return INVALID_XTAL_SEL_VALUE;
		}
	}
	else if (APLL_INST_2 == apllInstId)
	{
		if (APLL_XTAL_2_APLL2 == xtalId)
			return 0;
		else if (APLL_XTAL_4_APLL2 == xtalId)
			return 1;
		else
		{
			IDT_APLL_API_LOG_ERR(("Invalid xtal id: %d for apll: %d\n", xtalId, apllInstId));
			return INVALID_XTAL_SEL_VALUE;
		}
	}
	else
		IDT_APLL_API_LOG_ERR(("Invalid apll id: %d\n", apllInstId));

	return INVALID_XTAL_SEL_VALUE;
}

/**
 * @brief Function to retrieve APLL configuration
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @return APLL configuration<ul>
 *            <li>APLL_CONFIG0 - configuration 0</li>
 *            <li>APLL_CONFIG1 - configuration 1</li></ul>
 */
T_idtApllConfigSel IDTApll_GetConfigSel(T_idtDrvHdlr hdlr, T_idtApllId apllInstId)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_CONFIG_MASK;
    T_idtApllRegMask lsb = BF_CONFIG_LSB;
    T_idtApllRegVal field;
    T_idtApllConfigSel config = APLL_CONFIG_MAX;

	IDT_APLL_API_TRACE(("Apll id: %d\n", apllInstId));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return config;

    reg = APLL_REG_OFFSET(apllInstId, REG_CONTROL);
    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    config = (T_idtApllConfigSel)field;
    return config;
}

/**
 * @brief Function to provision APLL based on input configuration
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *           <li>APLL_INST_1 - APLL #1</li>
 *           <li>APLL_INST_2 - APLL #2</li></ul>
 * @param apllConfig APLL configuration<ul>
 *           <li>APLL_CONFIG0 - configuration 0</li>
 *           <li>APLL_CONFIG1 - configuration 1</li></ul>
 */
void IDTApll_SetConfigSel(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel apllConfig)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_CONFIG_MASK;
    T_idtApllRegMask lsb = BF_CONFIG_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration selection: %d\n",
					    apllInstId, apllConfig));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (apllConfig >= APLL_CONFIG_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll configuration: %d\n", apllConfig));
        return;
    }

    reg = APLL_REG_OFFSET(apllInstId, REG_CONTROL);
    field = (T_idtApllRegVal)apllConfig;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve selected APLL input frequency source (external/internal) 
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @return Input clock source for APLL<ul>
 *            <li>APLL_CLK_SEL_EXTERNAL - external clock source</li>
 *            <li>APLL_CLK_SEL_INTERNAL - internal clock source</li></ul>
 */
T_idtApllInputClkSrc IDTApll_GetInputClkSrc(T_idtDrvHdlr hdlr, T_idtApllId apllInstId)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_CLK_SEL_MASK;
    T_idtApllRegMask lsb = BF_CLK_SEL_LSB;
    T_idtApllRegVal field;
    T_idtApllInputClkSrc clk = APLL_CLK_SEL_MAX;

	IDT_APLL_API_TRACE(("Apll id: %d\n", apllInstId));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return clk;

    reg = APLL_REG_OFFSET(apllInstId, REG_CLK_SEL);
    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    clk = (T_idtApllInputClkSrc)field;
    return clk;
}

/**
 * @brief Function to provision APLL input frequency source (external/internal)
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @param inputClk Input clock source for APLL<ul>
 *            <li>APLL_CLK_SEL_EXTERNAL - external clock source</li>
 *            <li>APLL_CLK_SEL_INTERNAL - internal clock source</li></ul>
 */
void IDTApll_SetInputClkSrc(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllInputClkSrc inputClk)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_CLK_SEL_MASK;
    T_idtApllRegMask lsb = BF_CLK_SEL_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "input clock source: %d\n",
					    apllInstId, inputClk));

    OS_ASSERT_PTR_NULL(hdlr);
 
    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (inputClk >= APLL_CLK_SEL_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll input clock: %d\n", inputClk));
        return;
    }

    reg = APLL_REG_OFFSET(apllInstId, REG_CLK_SEL);
    field = (T_idtApllRegVal)inputClk;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve pre- (input) divider value
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @param apllConfig APLL configuration<ul>
 *            <li>APLL_CONFIG0 - configuration 0</li>
 *            <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @return Divisor value
 */
T_idtApllDividerValue IDTApll_GetPreDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel apllConfig)
{
    T_idtApllRegAddr regLsb,
                   regMsb;
    T_idtApllRegMask maskLsb,
                   maskMsb,
                   lsbLsb,
                   lsbMsb;
    T_idtApllRegVal preLsb, /* bits 7 to 0 */ 
                  preMsb; /* bits 14 to 8 */
    T_idtApllDividerValue div = INVALID_DIVIDER_VALUE;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration: %d\n",
					    apllInstId, apllConfig));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return div;

    if (APLL_CONFIG0 == apllConfig) 
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL0_LSB);
        maskLsb = BF_PDSEL0_LSB_MASK;
        lsbLsb = BF_PDSEL0_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL0_MSB);
        maskMsb = BF_PDSEL0_MSB_MASK;
        lsbMsb = BF_PDSEL0_MSB_LSB;
    }
    else if (APLL_CONFIG1 == apllConfig)
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL1_LSB);
        maskLsb = BF_PDSEL1_LSB_MASK;
        lsbLsb = BF_PDSEL1_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL1_MSB);
        maskMsb = BF_PDSEL1_MSB_MASK;
        lsbMsb = BF_PDSEL1_MSB_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll configuration: %d\n", apllConfig));
        return div; 
    }

    preLsb = IDTHAL_ReadBitfield_Ext(hdlr, regLsb, maskLsb, lsbLsb);
    preMsb = IDTHAL_ReadBitfield_Ext(hdlr, regMsb, maskMsb, lsbMsb);
    div = (T_idtApllDividerValue)APLL_PRE_DIV_VAL(preLsb, preMsb);   
    return div;
}

/**
 * @brief Function to retrieve pre- (input) divisor range
 * @param hdlr Device driver handler
 * @param minDiv minimum divisor
 * @param maxDiv maximum divisor
 * \note
 *          Divisor "1" is also a valid divisor
 */
void IDTApll_GetPreDivRange(T_idtDrvHdlr hdlr, T_idtApllDividerValue *minDiv, T_idtApllDividerValue *maxDiv)
{
	IDT_APLL_API_TRACE((""));

    OS_ASSERT_PTR_NULL(minDiv);
    OS_ASSERT_PTR_NULL(maxDiv);

    IDT_APLL_API_LOG_INFO(("Apll pre-divider range: %d - %d\n", APLL_PRE_DIV_MIN, APLL_PRE_DIV_MAX));
    *minDiv = APLL_PRE_DIV_MIN;
    *maxDiv = APLL_PRE_DIV_MAX;
}

/**
 * @brief Function to provision pre- (input) divider value
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @param apllConfig APLL configuration<ul>
 *            <li>APLL_CONFIG0 - configuration 0</li>
 *            <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @param div Divisor value
 */
void IDTApll_SetPreDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel apllConfig, T_idtApllDividerValue div)
{
    T_idtApllRegAddr regLsb,
                   regMsb;
    T_idtApllRegMask maskLsb,
                   maskMsb,
                   lsbLsb,
                   lsbMsb;
    T_idtApllRegVal preLsb, /* bits 7 to 0 */ 
                  preMsb; /* bits 14 to 8 */

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration: %d, "
					    "Apll pre-divider: %d\n",
					    apllInstId, apllConfig, div));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (div > APLL_PRE_DIV_MAX || ((div < APLL_PRE_DIV_MIN) && (div != 1)))
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll pre-divider input: %d\n", div));
        return;
    }

    if (APLL_CONFIG0 == apllConfig)
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL0_LSB);
        maskLsb = BF_PDSEL0_LSB_MASK;
        lsbLsb = BF_PDSEL0_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL0_MSB);
        maskMsb = BF_PDSEL0_MSB_MASK;
        lsbMsb = BF_PDSEL0_MSB_LSB;
    }
    else if (APLL_CONFIG1 == apllConfig)
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL1_LSB);
        maskLsb = BF_PDSEL1_LSB_MASK;
        lsbLsb = BF_PDSEL1_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_PDSEL1_MSB);
        maskMsb = BF_PDSEL1_MSB_MASK;
        lsbMsb = BF_PDSEL1_MSB_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll configuration: %d\n", apllConfig));
        return; 
    }

    preLsb = APLL_PRE_DIV_LSB(div);
    preMsb = APLL_PRE_DIV_MSB(div);

    IDTHAL_WriteBitfield_Ext(hdlr, regLsb, maskLsb, lsbLsb, preLsb);
    IDTHAL_WriteBitfield_Ext(hdlr, regMsb, maskMsb, lsbMsb, preMsb);
}

/**
 * @brief Function to retrieve feedback (oscillator) divisor range
 * @param hdlr Device driver handler
 * @param minDiv minimum divisor
 * @param maxDiv maximum divisor
 */
void IDTApll_GetFeedbackDivRange(T_idtDrvHdlr hdlr, T_idtApllDividerValue *minDiv, T_idtApllDividerValue *maxDiv)
{
	IDT_APLL_API_TRACE((""));

    OS_ASSERT_PTR_NULL(minDiv);
    OS_ASSERT_PTR_NULL(maxDiv);

    IDT_APLL_API_LOG_INFO(("Apll feedback divider range: %d - %d\n", APLL_FB_DIV_MIN, APLL_FB_DIV_MAX));
    *minDiv = APLL_FB_DIV_MIN;
    *maxDiv = APLL_FB_DIV_MAX;
}

/**
 * @brief Function to retrieve feedback (oscillator) divider value
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @param apllConfig APLL configuration<ul>
 *            <li>APLL_CONFIG0 - configuration 0</li>
 *            <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @return Divisor value
 */
T_idtApllDividerValue IDTApll_GetFeedbackDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel apllConfig)
{
    T_idtApllRegAddr regLsb,
                   regMsb;
    T_idtApllRegMask maskLsb,
                   maskMsb,
                   lsbLsb,
                   lsbMsb;
    T_idtApllRegVal preLsb, /* bits 7 to 0 */ 
                  preMsb; /* bits 14 to 8 */
    T_idtApllDividerValue div = INVALID_DIVIDER_VALUE;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration: %d\n",
					    apllInstId, apllConfig));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return div;

    if (APLL_CONFIG0 == apllConfig) 
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_M0_LSB);
        maskLsb = BF_M0_LSB_MASK;
        lsbLsb = BF_M0_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_M0_MSB);
        maskMsb = BF_M0_MSB_MASK;
        lsbMsb = BF_M0_MSB_LSB;
    }
    else if (APLL_CONFIG1 == apllConfig)
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_M1_LSB);
        maskLsb = BF_M1_LSB_MASK;
        lsbLsb = BF_M1_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_M1_MSB);
        maskMsb = BF_M1_MSB_MASK;
        lsbMsb = BF_M1_MSB_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll configuration: %d\n", apllConfig));
        return div; 
    }

    preLsb = IDTHAL_ReadBitfield_Ext(hdlr, regLsb, maskLsb, lsbLsb);
    preMsb = IDTHAL_ReadBitfield_Ext(hdlr, regMsb, maskMsb, lsbMsb);
    div = (T_idtApllDividerValue)APLL_FB_DIV_VAL(preLsb, preMsb);   
    return div;

}

/**
 * @brief Function to provision feedback (oscillator) divider value
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @param apllConfig APLL configuration<ul>
 *            <li>APLL_CONFIG0 - configuration 0</li>
 *            <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @param div Divisor value
 */
void IDTApll_SetFeedbackDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel apllConfig, T_idtApllDividerValue div)
{
    T_idtApllRegAddr regLsb,
                   regMsb;
    T_idtApllRegMask maskLsb,
                   maskMsb,
                   lsbLsb,
                   lsbMsb;
    T_idtApllRegVal preLsb, /* bits 7 to 0 */ 
                  preMsb; /* bits 14 to 8 */

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration: %d, "
					    "Apll feedback divider: %d\n",
					    apllInstId, apllConfig, div));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (div > APLL_FB_DIV_MAX || div < APLL_FB_DIV_MIN)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll feedback divider: %d\n", div));
        return;
    }

     if (APLL_CONFIG0 == apllConfig)
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_M0_LSB);
        maskLsb = BF_M0_LSB_MASK;
        lsbLsb = BF_M0_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_M0_MSB);
        maskMsb = BF_M0_MSB_MASK;
        lsbMsb = BF_M0_MSB_LSB;
    }
    else if (APLL_CONFIG1 == apllConfig)
    {
        regLsb = APLL_REG_OFFSET(apllInstId, REG_M1_LSB);
        maskLsb = BF_M1_LSB_MASK;
        lsbLsb = BF_M1_LSB_LSB;
        regMsb = APLL_REG_OFFSET(apllInstId, REG_M1_MSB);
        maskMsb = BF_M1_MSB_MASK;
        lsbMsb = BF_M1_MSB_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll configuration: %d\n", apllConfig));
        return; 
    }

    preLsb = APLL_FB_DIV_LSB(div);
    preMsb = APLL_FB_DIV_MSB(div);

    IDTHAL_WriteBitfield_Ext(hdlr, regLsb, maskLsb, lsbLsb, preLsb);
    IDTHAL_WriteBitfield_Ext(hdlr, regMsb, maskMsb, lsbMsb, preMsb);
}

/**
 * @brief Function to retrieve output port A divisor
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *             <li>APLL_INST_1 - APLL #1</li>
 *             <li>APLL_INST_2 - APLL #2</li></ul>
 * @param config APLL configuration<ul>
 *             <li>APLL_CONFIG0 - configuration 0</li>
 *             <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @return Output divider assigned to output port A<ul>
 *             <li>APLL_QA_ODSEL_DIV_25 - divide by 25</li>
 *             <li>APLL_QA_ODSEL_DIV_5 - divide by 5</li>
 *             <li>APLL_QA_ODSEL_DIV_4 - divide by 4</li>
 *             <li>APLL_QA_ODSEL_DIV_2 - divide by 2</li>
 *             <li>APLL_QA_ODSEL_DIV_1 - divide by 1</li></ul>
 */
T_idtApllQaDiv IDTApll_GetQaDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel config)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;
    T_idtApllQaDiv div = APLL_QA_ODSEL_DIV_MAX;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration: %d\n",
					    apllInstId, config));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return div;

    if (APLL_CONFIG0 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELA0);
        mask = BF_ODBSELA0_MASK;
        lsb = BF_ODBSELA0_LSB;
    }
    else if (APLL_CONFIG1 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELA1);
        mask = BF_ODBSELA1_MASK;
        lsb = BF_ODBSELA1_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Inavlid apll configuration: %d\n", config));
        return div;        
    }

    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    div = (T_idtApllQaDiv)field;
    return div;
}

/**
 * @brief Function to provision output port A divisor
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *             <li>APLL_INST_1 - APLL #1</li>
 *             <li>APLL_INST_2 - APLL #2</li></ul>
 * @param config APLL configuration<ul>
 *             <li>APLL_CONFIG0 - configuration 0</li>
 *             <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @param div Divisor value<ul>
 *     <li>APLL_QA_ODSEL_DIV_25 - divide by 25</li>
 *     <li>APLL_QA_ODSEL_DIV_5 - divide by 5</li>
 *     <li>APLL_QA_ODSEL_DIV_4 - divide by 4</li>
 *     <li>APLL_QA_ODSEL_DIV_2 - divide by 2</li>
 *     <li>APLL_QA_ODSEL_DIV_1 - divide by 1</li></ul>
 */
void IDTApll_SetQaDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel config, T_idtApllQaDiv div)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration: %d, "
					    "Apll QA output divider: %d\n",
					    apllInstId, config, div));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (div >= APLL_QA_ODSEL_DIV_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid QA output divider: %d\n", div));
        return;
    }

    if (APLL_CONFIG0 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELA0);
        mask = BF_ODBSELA0_MASK;
        lsb = BF_ODBSELA0_LSB;
    }
    else if (APLL_CONFIG1 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELA1);
        mask = BF_ODBSELA1_MASK;
        lsb = BF_ODBSELA1_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Inavlid apll configuration: %d\n", config));
        return;        
    }

    field = (T_idtApllRegVal)div;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve output port B divisor
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *             <li>APLL_INST_1 - APLL #1</li>
 *             <li>APLL_INST_2 - APLL #2</li></ul>
 * @param config APLL configuration<ul>
 *             <li>APLL_CONFIG0 - configuration 0</li>
 *             <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @return Output divider assigned to output port B<ul>
 *             <li>APLL_QB_ODSEL_DIV_8 - divide by 8</li>
 *             <li>APLL_QB_ODSEL_DIV_5 - divide by 5</li>
 *             <li>APLL_QB_ODSEL_DIV_4 - divide by 4</li>
 *             <li>APLL_QB_ODSEL_DIV_2 - divide by 2</li>
 *             <li>APLL_QB_ODSEL_DIV_1 - divide by 1</li></ul>
 */
T_idtApllQaDiv IDTApll_GetQbDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel config)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;
    T_idtApllQbDiv div = APLL_QB_ODSEL_DIV_MAX;

	IDT_APLL_API_TRACE(("Apll chip ID: %d, "
					    "Apll configuration: %d\n",
					    apllInstId, config));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return div;

    if (APLL_CONFIG0 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELB0);
        mask = BF_ODBSELB0_MASK;
        lsb = BF_ODBSELB0_LSB;
    }
    else if (APLL_CONFIG1 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELB1);
        mask = BF_ODBSELB1_MASK;
        lsb = BF_ODBSELB1_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Inavlid apll configuration: %d\n", config));
        return div;        
    }

    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    div = (T_idtApllQbDiv)field;
    return div;
}

/**
 * @brief Function to provision output port B divisor
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *             <li>APLL_INST_1 - APLL #1</li>
 *             <li>APLL_INST_2 - APLL #2</li></ul>
 * @param config APLL configuration<ul>
 *             <li>APLL_CONFIG0 - configuration 0</li>
 *             <li>APLL_CONFIG1 - configuration 1</li></ul>
 * @param div Divisor value<ul>
 *             <li>APLL_QB_ODSEL_DIV_8 - divide by 8</li>
 *             <li>APLL_QB_ODSEL_DIV_5 - divide by 5</li>
 *             <li>APLL_QB_ODSEL_DIV_4 - divide by 4</li>
 *             <li>APLL_QB_ODSEL_DIV_2 - divide by 2</li>
 *             <li>APLL_QB_ODSEL_DIV_1 - divide by 1</li></ul>    
 */
void IDTApll_SetQbDiv(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllConfigSel config, T_idtApllQbDiv div)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll configuration: %d, "
					    "Apll QB output divider: %d\n",
					    apllInstId, config, div));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (div >= APLL_QB_ODSEL_DIV_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid QB output divider: %d\n", div));
        return;
    }

    if (APLL_CONFIG0 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELB0);
        mask = BF_ODBSELB0_MASK;
        lsb = BF_ODBSELB0_LSB;
    }
    else if (APLL_CONFIG1 == config)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_ODBSELB1);
        mask = BF_ODBSELB1_MASK;
        lsb = BF_ODBSELB1_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Inavlid apll configuration: %d\n", config));
        return;
    }

    field = (T_idtApllRegVal)div;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve whether feedback frequency doubling is enabled
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @return Feedback frequency doubling value<ul>
 *            <li>APLL_FREQ_SEL_SINGLE - Feedback frequency not doubled</li>
 *            <li>APLL_FREQ_SEL_DOUBLE - Feedback frequency doubled</li></ul>
 */
T_idtApllFreqDoublerSel IDTApll_GetFreqDoublerSel(T_idtDrvHdlr hdlr, T_idtApllId apllInstId)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_SEL_DOUBLE_MASK;
    T_idtApllRegMask lsb = BF_SEL_DOUBLE_LSB;
    T_idtApllRegVal field;
    T_idtApllFreqDoublerSel doubler = APLL_FREQ_SEL_MAX;

	IDT_APLL_API_TRACE(("Apll chip ID: %d\n", apllInstId));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return doubler;

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    doubler = (T_idtApllFreqDoublerSel)field;
    return doubler;
}

/**
 * @brief Function to enable/disable feedback frequency doubling
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *                   <li>APLL_INST_1 - APLL #1</li>
 *                   <li>APLL_INST_2 - APLL #2</li></ul>
 * @param doubler Feedback frequency doubling value<ul>
 *                   <li>APLL_FREQ_SEL_SINGLE - Feedback frequency not doubled</li>
 *                   <li>APLL_FREQ_SEL_DOUBLE - Feedback frequency doubled</li></ul>
 */
void IDTApll_SetFreqDoublerSel(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllFreqDoublerSel doubler)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_SEL_DOUBLE_MASK;
    T_idtApllRegMask lsb = BF_SEL_DOUBLE_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll frequency doubler: %d\n",
					    apllInstId, doubler));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;
  
    if (doubler >= APLL_FREQ_SEL_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll freqency doubler selection: %d\n", doubler));
        return;
    }

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = (T_idtApllRegVal)doubler;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve oscillator currently used
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *             <li>APLL_INST_1 - APLL #1</li>
 *             <li>APLL_INST_2 - APLL #2</li></ul>
 * @return current oscillator used<ul>
 *             <li>APLL_XTAL_1_APLL1 - XTAL1 for APLL1</li>
 *             <li>APLL_XTAL_2_APLL2 - XTAL2 for APLL2</li>
 *             <li>APLL_XTAL_3_APLL1 - XTAL3 for APLL1</li>
 *             <li>APLL_XTAL_4_APLL2 - XTAL4 for APLL2</li></ul>
 */
T_idtApllXtalId IDTApll_GetXtalId(T_idtDrvHdlr hdlr, T_idtApllId apllInstId)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_VCXO_SEL_MASK;
    T_idtApllRegMask lsb = BF_VCXO_SEL_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d\n", apllInstId));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return APLL_XTAL_MAX;

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    return IDTApll_xtalBitField2xtalType(apllInstId, field);
}

/**
 * @brief Function to select oscillator
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *             <li>APLL_INST_1 - APLL #1</li>
 *             <li>APLL_INST_2 - APLL #2</li></ul>
 * @param xtal Oscillator to select<ul>
 *             <li>APLL_XTAL_1_APLL1 - XTAL1 for APLL1</li>
 *             <li>APLL_XTAL_2_APLL2 - XTAL2 for APLL2</li>
 *             <li>APLL_XTAL_3_APLL1 - XTAL3 for APLL1</li>
 *             <li>APLL_XTAL_4_APLL2 - XTAL4 for APLL2</li></ul>
 */
void IDTApll_SetXtalId(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllXtalId xtal)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_VCXO_SEL_MASK;
    T_idtApllRegMask lsb = BF_VCXO_SEL_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll xtal id: %d\n",
					    apllInstId, xtal));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (xtal >= APLL_XTAL_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll xtal selection: %d\n", xtal));
        return;
    }

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = IDTApll_xtalType2xtalBitField(apllInstId, xtal);

	if (INVALID_XTAL_SEL_VALUE == field)
		return;
	
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve output port signal type
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *           <li>APLL_INST_1 - APLL #1</li>
 *           <li>APLL_INST_2 - APLL #2</li></ul>
 * @param output Output port<ul>
 *           <li>APLL_OUTPUT_QA - Output port A</li>
 *           <li>APLL_OUTPUT_QB - Output port B</li></ul>
 * @return output signal type<ul>
 *           <li>APLL_OUT_SIG_LVPECL - Output LVPECL signal</li>
 *           <li>APLL_OUT_SIG_LVDS - Output LVDS signal</li></ul>
 */
T_idtApllOutputSigType IDTApll_GetOutputSigType(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllOutput output)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;
    T_idtApllOutputSigType sig = APLL_OUT_SIG_MAX;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll output: %d\n",
					    apllInstId, output));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return sig;

    if (APLL_OUTPUT_QA == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QA);
        mask = BF_LVDS_QA_MASK;
        lsb = BF_LVDS_QA_LSB;
    }
    else if (APLL_OUTPUT_QB == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QB);
        mask = BF_LVDS_QB_MASK;
        lsb = BF_LVDS_QB_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll output: %d\n", output));
        return sig;
    }

    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    sig = (T_idtApllOutputSigType)field;
    return sig;
}

/**
 * @brief Function to provision output port signal type
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *           <li>APLL_INST_1 - APLL #1</li>
 *           <li>APLL_INST_2 - APLL #2</li></ul>
 * @param output Output port<ul>
 *           <li>APLL_OUTPUT_QA - Output port A</li>
 *           <li>APLL_OUTPUT_QB - Output port B</li></ul>
 * @param outputSigType output signal type<ul>
 *           <li>APLL_OUT_SIG_LVPECL - Output LVPECL signal</li>
 *           <li>APLL_OUT_SIG_LVDS - Output LVDS signal</li></ul>
 */
void IDTApll_SetOutputSigType(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllOutput output, T_idtApllOutputSigType outputSigType)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll output: %d, "
					    "Apll output signal type: %d\n",
					    apllInstId, output, outputSigType));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (outputSigType >= APLL_OUT_SIG_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid output signal type: %d\n", outputSigType));
        return;
    }

    if (APLL_OUTPUT_QA == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QA);
        mask = BF_LVDS_QA_MASK;
        lsb = BF_LVDS_QA_LSB;
    }
    else if (APLL_OUTPUT_QB == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QB);
        mask = BF_LVDS_QB_MASK;
        lsb = BF_LVDS_QB_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll output: %d\n", output));
        return;
    }

    field = (T_idtApllRegVal) outputSigType;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve output port status (enable/disable)
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *           <li>APLL_INST_1 - APLL #1</li>
 *           <li>APLL_INST_2 - APLL #2</li></ul>
 * @param output Output port<ul>
 *           <li>APLL_OUTPUT_QA - Output port A</li>
 *           <li>APLL_OUTPUT_QB - Output port B</li></ul>
 * @return Enable /Disable output port<ul>
 *           <li>APLL_OUTPUT_DISABLE - Disable output</li>
 *           <li>APLL_OUTPUT_ENABLE - Enable output</li></ul>
 */
T_idtApllOutputEn IDTApll_GetOutputEnState(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllOutput output)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;
    T_idtApllOutputEn enable = APLL_OUTPUT_EN_MAX;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll output: %d\n",
					    apllInstId, output));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return enable;

    if (APLL_OUTPUT_QA == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QA);
        mask = BF_OE_A_MASK;
        lsb = BF_OE_A_LSB;
    }
    else if (APLL_OUTPUT_QB == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QB);
        mask = BF_OE_B_MASK;
        lsb = BF_OE_B_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll output: %d\n", output));
        return enable;
    }

    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    enable = (T_idtApllOutputEn)field;
    return enable;
}

/**
 * @brief Function to control (enable/disable) output port.
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *           <li>APLL_INST_1 - APLL #1</li>
 *           <li>APLL_INST_2 - APLL #2</li></ul>
 * @param output Output port<ul>
 *           <li>APLL_OUTPUT_QA - Output port A</li>
 *           <li>APLL_OUTPUT_QB - Output port B</li></ul>
 * @param enable Enable /Disable output<ul>
 *           <li>APLL_OUTPUT_DISABLE - Disable output</li>
 *           <li>APLL_OUTPUT_ENABLE - Enable output</li></ul>
 */
void IDTApll_SetOutputEnState(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllOutput output, T_idtApllOutputEn enable)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask;
    T_idtApllRegMask lsb;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Apll output: %d, "
					    "Apll enable output: %d\n",
					    apllInstId, output, enable));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (enable >= APLL_OUTPUT_EN_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll output state: %d\n", enable));
        return;
    }
   
    if (APLL_OUTPUT_QA == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QA);
        mask = BF_OE_A_MASK;
        lsb = BF_OE_A_LSB;
    }
    else if (APLL_OUTPUT_QB == output)
    {
        reg = APLL_REG_OFFSET(apllInstId, REG_CONFIG_QB);
        mask = BF_OE_B_MASK;
        lsb = BF_OE_B_LSB;
    }
    else
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll output: %d\n", output));
        return;
    } 

    field = (T_idtApllRegVal)enable;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to reset APLL
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @param reset Value to set master reset sync<ul>
 *            <li>APLL_OUT_MR_SYNC - Normal operation</li>
 *            <li>APLL_IN_MR_SYNC - In master sync reset</li></ul>
 */
void IDTApll_SetMasterResetSync(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllMasterResetSync reset)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_MR_SYNC_MASK;
    T_idtApllRegMask lsb = BF_MR_SYNC_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Reset apll: %d\n",
					    apllInstId, reset));
	
    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (reset >= APLL_MR_SYNC_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll reset input: %d\n", reset));
        return;
    }

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = (T_idtApllRegVal)reset;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve reset status of device
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @return Master reset sync<ul>
 *            <li>APLL_OUT_MR_SYNC - Normal operation</li>
 *            <li>APLL_IN_MR_SYNC - In master sync reset</li></ul>
 */
T_idtApllMasterResetSync IDTApll_GetMasterResetSync(T_idtDrvHdlr hdlr, T_idtApllId apllInstId)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_MR_SYNC_MASK;
    T_idtApllRegMask lsb = BF_MR_SYNC_LSB;
    T_idtApllMasterResetSync reset = APLL_MR_SYNC_MAX;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d\n", apllInstId));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return reset;

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    reset = (T_idtApllMasterResetSync)field;
    return reset;
}

/**
 * @brief Function to set APLL bypass setting
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *           <li>APLL_INST_1 - APLL #1</li>
 *           <li>APLL_INST_2 - APLL #2</li></ul>
 * @param bypass Value to set bypass<ul>
 *           <li>APLL_BYPASS_NORMAL_OP - Normal operation</li>
 *           <li>APLL_BYPASS_BYPASS - Bypass VCXO, connect input to output</li></ul>
 */
void IDTApll_SetBypass(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllBypass bypass)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_BYPASS_MASK;
    T_idtApllRegMask lsb = BF_BYPASS_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Bypass: %d\n",
					    apllInstId, bypass));
	
    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (bypass >= APLL_BYPASS_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll bypass input: %d\n", bypass));
        return;
    }

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = (T_idtApllRegVal)bypass;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve bypass status of device
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @return Bypass status<ul>
 *            <li>APLL_BYPASS_NORMAL_OP - Normal operation</li>
 *            <li>APLL_BYPASS_BYPASS - Bypass VCXO, connect input to output</li></ul>
 */
T_idtApllBypass IDTApll_GetBypass(T_idtDrvHdlr hdlr, T_idtApllId apllInstId)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_BYPASS_MASK;
    T_idtApllRegMask lsb = BF_BYPASS_LSB;
    T_idtApllBypass bypass = APLL_BYPASS_MAX;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d\n", apllInstId));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return bypass;

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    bypass = (T_idtApllMasterResetSync)field;
    return bypass;
}

/**
 * @brief Function to set APLL shutoff setting
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @param shutoff Value to set shutoff<ul>
 *            <li>APLL_SHUTOFF_NORMAL_OP - Normal operation</li>
 *            <li>APLL_SHUTOFF_SHUTOFF - VCXO shutoff</li></ul>
 */
void IDTApll_SetShutoff(T_idtDrvHdlr hdlr, T_idtApllId apllInstId, T_idtApllShutoff shutoff)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_SHUTOFF_MASK;
    T_idtApllRegMask lsb = BF_SHUTOFF_LSB;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d, "
					    "Shutoff: %d\n",
					    apllInstId, shutoff));
	
    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return;

    if (shutoff >= APLL_SHUTOFF_MAX)
    {
        IDT_APLL_API_LOG_ERR(("Invalid apll shutoff input: %d\n", shutoff));
        return;
    }

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = (T_idtApllRegVal)shutoff;
    IDTHAL_WriteBitfield_Ext(hdlr, reg, mask, lsb, field);
}

/**
 * @brief Function to retrieve shutoff status of device
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance<ul>
 *            <li>APLL_INST_1 - APLL #1</li>
 *            <li>APLL_INST_2 - APLL #2</li></ul>
 * @return Shutoff status<ul>
 *            <li>APLL_SHUTOFF_NORMAL_OP - Normal operation</li>
 *            <li>APLL_SHUTOFF_SHUTOFF - VCXO shutoff</li></ul>
 */
T_idtApllShutoff IDTApll_GetShutoff(T_idtDrvHdlr hdlr, T_idtApllId apllInstId)
{
    T_idtApllRegAddr reg;
    T_idtApllRegMask mask = BF_SHUTOFF_MASK;
    T_idtApllRegMask lsb = BF_SHUTOFF_LSB;
    T_idtApllShutoff shutoff = APLL_SHUTOFF_MAX;
    T_idtApllRegVal field;

	IDT_APLL_API_TRACE(("Apll id: %d\n", apllInstId));

    OS_ASSERT_PTR_NULL(hdlr);

    if (!IDTApll_validApllInstId(apllInstId))
        return shutoff;

    reg = APLL_REG_OFFSET(apllInstId, REG_VCXO_SEL);
    field = IDTHAL_ReadBitfield_Ext(hdlr, reg, mask, lsb);
    shutoff = (T_idtApllShutoff)field;
    return shutoff;
}
