/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.  All
 * Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "osNumeric.h"
#include "Define.h"
#include "Input.h"
#include "General.h"
#include "DpllCnfg.h"
#include "Dpll.h"

#include "inputFreq.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/**
 * @brief Structure to contain translation information for frequency
 * to bitfield value
 */
typedef struct S_IDTFreq2bfVal{
    T_osUint32              freq_m;  /**< Frequency (in Hz) */
    T_idtInputFrequencyBitfieldValue bfVal_m; /**< Bitfield value */
} T_IDTFreq2bfVal;

/**
 * @brief Structure to use as high frequency (HF) divider information
 */
typedef struct S_idtHfDivEntry {
    T_osUint8 divValue_m; /**< Divisor value */
    T_idtHighFreqencyDivisor hfDivValue_m; /**< HF divisor bit field
                                            * value */
} T_idtHfDivEntry;

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/**
 * @brief Table listing supported input DPLL frequencies (bypass
 * dividers).
 */
static const T_IDTFreq2bfVal IDTHlApi_freq2BfValBypassTable[] = {
    /* Freq (Hz) BF Value */
    {      2000, Freq_2K },
    /* Skip 1.544MHz and 2.048 MHz frequencies since it requies us to
     * configure the network (SONET/SDH) type. We'll leave this
     * configuration option open until we provision output ports. This
     * allows us the greatest flexability when configuring output path
     * muxes. */
    {      4000, Freq_4K },
    {      8000, Freq_8K },
    {   6250000, Freq_625M },
    {   6480000, Freq_648M },
    {  10000000, Freq_10M },
    {  19440000, Freq_1944M },
    {  25920000, Freq_2592M },
    {  38880000, Freq_3888M }
};

/**
 * @brief Size of Bypass Table
 */
static const int IDTHlApi_freq2BfValBypassTableSize = 
    (sizeof(IDTHlApi_freq2BfValBypassTable)/sizeof(T_IDTFreq2bfVal));

/**
 * @brief Table listing supported input DPLL frequencies. Input
 * frequencies from the (input) pin must be divided down to one of the
 * listed frequencies.
 *
 * This table is list by priority. Don't not change unless IDT is
 * consulted.
 */
static const T_IDTFreq2bfVal IDTHlApi_freq2BfValDivNTable[] = {
    /* Freq (Hz) BF Value */
    {      8000, Freq_8K },
    /* Skip 1.544MHz and 2.048 MHz frequencies since it requies us to
     * configure the network (SONET/SDH) type. We'll leave this
     * configuration option open until we provision output ports. This
     * allows us the greatest flexability when configuring output path
     * muxes. */
    {   6250000, Freq_625M },
    {   6480000, Freq_648M },
    {  10000000, Freq_10M },
    {  19440000, Freq_1944M },
    {  25920000, Freq_2592M },
    {  38880000, Freq_3888M },
    {      2000, Freq_2K },
    {      4000, Freq_4K }
};

/**
 * @brief Size of DivN table
 */
static const T_osUint8 IDTHlApi_freq2BfValDivNTableSize = 
    (sizeof(IDTHlApi_freq2BfValDivNTable)/sizeof(T_IDTFreq2bfVal));

/**
 * @brief HF divider enable table 
 */ 
static const T_idtHfDivEntry hfDivEnable_ca[] = {
    { 4, HF_Divide_4 },
    { 5, HF_Divide_5 }
};

/**
 * @brief Size of HF divider enabled table
 */
static const T_osUint8 hfDivEnableSize_c = 
    (sizeof(hfDivEnable_ca)/sizeof(T_idtHfDivEntry));


/**
 * @brief Size of HF divider disable table
 */
static const T_idtHfDivEntry hfDivDisable_ca[] = {
    { 1, HF_Bypass }
};

/**
 * @brief HF divider disable table 
 */ 
static const T_osUint8 hfDivDisableSize_c = 
    (sizeof(hfDivDisable_ca)/sizeof(T_idtHfDivEntry));

/**
 * @brief HF divider none table 
 */ 
static const T_idtHfDivEntry hfDivNone_ca[] = {
    { 1, HF_MAX }
};

/**
 * @brief Size of HF divider none table
 */
static const T_osUint8 hfDivNoneSize_c = 
    (sizeof(hfDivNone_ca)/sizeof(T_idtHfDivEntry));

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

/**
 * @brief Function to provision input port for 1PPS (1 Hz)
 *
 * @param hdlr driver handler
 *
 * @param nInputNo input port number
 */
void IDTHlApi_ConfigureInput1PPS(T_idtDrvHdlr hdlr,
                                 T_osUint8    nInputNo)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo));

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo != 0);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr,nInputNo,internalPort);
    OS_ASSERT(internalPort>=3);

    if (hdlr->deviceConfig_m->inputFreqValidFunc_m)
    {
        OS_ASSERTS((hdlr->deviceConfig_m->inputFreqValidFunc_m(hdlr, nInputNo, 1, NULL)),
                    ("Invalid 1Hz input for input port/%d dev/%s\n",
                    nInputNo, hdlr->name_m));
    }

    /* Enable input */
    IDTInput_SetInputEnable (hdlr, nInputNo, 1); 

    /* Bypass dividers */
    IDTInput_SetPreDivider(hdlr, nInputNo, Divider_Bypass);

    /* Specify input frequency */
    IDTInput_SetInputFrequency (hdlr, nInputNo, Freq_1PPS);
}

/**
 * @brief Function to provision input AMI port
 *
 * @param hdlr driver handler
 *
 * @param nInputNo input port number
 *
 * @param amiSignal Selected AMI signal /see
 * T_idtAmiInputFrequencyBitFieldValue
 */
void IDTHlApi_ConfigureInputAmi(
    T_idtDrvHdlr                        hdlr,
    T_osUint8                           nInputNo,
    T_idtAmiInputFrequencyBitFieldValue amiSignal)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo));

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo != 0);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr,nInputNo,internalPort);
    OS_ASSERT(internalPort<=2);

    /* Enable input */
    IDTInput_SetInputEnable (hdlr, nInputNo, 1); 

    /* Select AMI signal */
    IDTInput_SetAmiInputFrequency (hdlr, nInputNo, amiSignal);

    /* Specify input frequency */
    IDTInput_SetInputFrequency (hdlr, nInputNo, Freq_8K);
}

/**
 * @brief Function to attempt provision an input frequency bypassing
 * dividers.
 * @param hdlr Device driver handle
 * @param nInputNo Input port number 
 * @param freqInHz Desired input frequency (in Hz)
 * @return 1-Provisioned, 0-Failed to provision
 */
static T_osUint8 IDTHlApi_TryBypassDivider (T_idtDrvHdlr hdlr,
                                            T_osUint8    nInputNo,
                                            T_osUint32   freqInHz)
{
    T_osInt16 i;
    const T_IDTFreq2bfVal *entry_p;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d freqInHz/%d\n",
               __FUNCTION__, hdlr->name_m, nInputNo, freqInHz));

    /* Bypass (i.e. input frequency matches exactly one of the
     * supported DPLL input frequencies) */
    for (i=0; i<IDTHlApi_freq2BfValBypassTableSize; i++)
    {
        entry_p = &IDTHlApi_freq2BfValBypassTable[i];
        if (entry_p->freq_m == freqInHz) 
        {

            /* Bypass dividers */
            IDTInput_SetPreDivider(hdlr, nInputNo, Divider_Bypass);

            /* Specify input frequency */
            IDTInput_SetInputFrequency (hdlr, 
                                        nInputNo, 
                                        entry_p->bfVal_m);

            /* Success! */
            return 1;
        }
    }

    /* Failure! Bad */
    return 0;
}

/**
 * @brief Function to attempt provision an input frequency using DivN
 * divider.
 * @param hdlr Device driver handle
 * @param nInputNo Input port number 
 * @param freqInHz Desired input frequency (in Hz)
 * @return 1-Provisioned, 0-Failed to provision
 */
static T_osUint8 IDTHlApi_TryDivNDivider (T_idtDrvHdlr hdlr,
                                          T_osUint8    nInputNo,
                                          T_osUint32   freqInHz)
{
    T_osInt16 i;
    const T_IDTFreq2bfVal *entry_p;
    T_osUint32 quotient;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d freqInHz/%d\n",
               __FUNCTION__, hdlr->name_m, nInputNo, freqInHz));

    /* Try to match DivN (cycle through supported frequencies) */
    for (i=0; i<IDTHlApi_freq2BfValDivNTableSize; i++) 
    {
        entry_p = &IDTHlApi_freq2BfValDivNTable[i];

        /* Even divisor to frequency (no remainder) */
        if ((freqInHz % entry_p->freq_m) == 0) {

            /* Check if quotient is too large */
            quotient = (freqInHz / entry_p->freq_m);

            if (quotient <= (1<<15)) { /* Less than or equal to
                                        * 32768 */
                /* DivN divider mode */
                IDTInput_SetPreDivider(hdlr, nInputNo, Divider_DivN);

                /* Specify input frequency */
                IDTInput_SetInputFrequency (hdlr, 
                                            nInputNo, 
                                            entry_p->bfVal_m); 

                /* Set divisor for divN divider */
                IDTInput_SetDivisionFactor(hdlr, 
                                           nInputNo, 
                                           quotient-1);

                /* Success! */ 
                return 1;
            }
        }
    }
    /* Failure. Boo! */
    return 0;
}

/**
 * @brief Function to provision input port.
 *
 * @param hdlr driver handler
 *
 * @param nInputNo input port number
 *
 * @param nFrequency input frequency (in kHz)
 */
void IDTHlApi_ConfigureInputFrequency(T_idtDrvHdlr hdlr,
                                      T_osUint8    nInputNo,
                                      T_osUint32   nFrequency)
{
    T_osInt16 divisorIdx;
    T_osUint8 internalPort;
    T_osUint32 freqInHz;

    const T_idtHfDivEntry *hfDivisorTable_p;
    T_osUint8 hfDivisorTableSize;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d nFrequency/%d\n",
               __FUNCTION__, hdlr->name_m, nInputNo, nFrequency));

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo != 0);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    OS_ASSERT(nFrequency<=625000); /* Maximum input frequency is 625MHz */
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr,nInputNo,internalPort);

    if (hdlr->deviceConfig_m->inputFreqValidFunc_m)
    {
        OS_ASSERTS((hdlr->deviceConfig_m->inputFreqValidFunc_m(hdlr, nInputNo, nFrequency*1000, NULL)),
                    ("Invalid input frequency/%d for input port/%d dev/%s\n",
                    nFrequency, nInputNo, hdlr->name_m));
    }

    /* Translated input frequency from kHz to Hz */
    freqInHz = nFrequency * 1000;

    if ((internalPort == 5) || (internalPort == 6))
    {
        if (nFrequency > 155520)
        {
            /* Need to divide down input frequency */
            hfDivisorTable_p = hfDivEnable_ca;
            hfDivisorTableSize = hfDivEnableSize_c;
        }
        else
        {
            hfDivisorTable_p = hfDivDisable_ca;
            hfDivisorTableSize = hfDivDisableSize_c;
        }
    }
    else
    {
        /* Input port does not support HF divider */
        hfDivisorTable_p = hfDivNone_ca;
        hfDivisorTableSize = hfDivNoneSize_c;
    }

    /* Loop though HF table - we do this because there are cases where we
     * can divide down input freuency by mutiple HF divisors but the
     * resultant frequency can not be further divided by DivN divider to a
     * supported DPLL input frequency.
     *
     * Hence, the loop. We first divide try dividing down using HF and
     * then try bypass/DivN. If neither work, then try different HF
     * value */
    for (divisorIdx=0; divisorIdx<hfDivisorTableSize; divisorIdx++)
    {
        /* Divide down initial input frequency (high freuency divider) */
        freqInHz = ((nFrequency * 1000) / 
                    hfDivisorTable_p[divisorIdx].divValue_m);
        if (hfDivisorTable_p[divisorIdx].hfDivValue_m != HF_MAX)
            IDTInput_SetInputHFdivider(
                hdlr, 
                nInputNo, 
                hfDivisorTable_p[divisorIdx].hfDivValue_m);

        /* Enable input */
        IDTInput_SetInputEnable (hdlr, nInputNo, 1); 

        /* See if frequency matches exactly one of the supported DPLL
         * input frequencies */
        if (IDTHlApi_TryBypassDivider(hdlr,
                                      nInputNo,
                                      freqInHz))
            return;

        /* See if frequency is a multiple of one of the supported DPLL
         * input frequencies */
        if (IDTHlApi_TryDivNDivider(hdlr,
                                    nInputNo,
                                    freqInHz))
            return;

        /* Ok - so we've reach a point that we don't support bypass
         * not divN, loop again the try differen HF divider. Cross you
         * fingers. */
    }

    /* Ok - so the frequency was not evenly divisable, loop again and
     * try another divider */
    OS_ERROR(("Invalid input frequency %d kHz. Unable to divide evenly into "
              "2kHz,4kHz,8kHz,2.048MHz,...\n",
              nFrequency));
}

/**
 * @brief Function to disable all inputs priorities.
 *
 * This is different to enabling inputs. Input priorities are used by
 * DPLLs to select input ports to lock to. Disable input priorites
 * tells DPLLs to ignore all inputs.
 *
 * Use this function when a custom priority is desired. This functions
 * will clear all input priorites first. Input priorities can then be
 * set by calling IDTInput_SetPriority function.
 * 
 * @param hdlr Device driver handle
 */
void IDTHlApi_DisableAllInputPriorities(T_idtDrvHdlr hdlr)
{
    int i;

    OS_ASSERT_PTR_NULL(hdlr);

    for (i=1; i<=IDT_DRV_MAX_INPUT; i++) {
        int internalPort;

        /* Skip unsupported ports */
        internalPort = hdlr->deviceConfig_m->inputExt2IntXlate_ma[i];
        if (internalPort == 0) continue;

        /* DPLL #1 ignore all inputs */
        if (IDTDpll_GetTypeOfDpll(hdlr,DPLL_INSTANCE_1) != Dpll_MAX)
            IDTInput_SetPriority(hdlr, i, DPLL_INSTANCE_1, 
                                 0); /* Disable */

        /* DPLL #2 ignore all inputs */
        if (IDTDpll_GetTypeOfDpll(hdlr,DPLL_INSTANCE_2) != Dpll_MAX)
            IDTInput_SetPriority(hdlr, i, DPLL_INSTANCE_2, 
                                 0); /* Disable */
    }
}
