/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "Define.h"
#include "osNumeric.h"
#include "DpllCnfg.h"
#include "General.h"

/**
 * \file profiles.c
 *
 * This sample file contains DPLL configurations used to conform to
 * various standards.
 */


/**
 * @brief DPLL configuration for GR-253 (SONET) Stratum 3
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_gr253SonetStratum3 (T_idtDrvHdlr hdlr,
                                  T_idtDpllInstance nDpll)
{
  T_idtDpllType dpllType;

  /* Exit function if we don't not support this DPLL */
  dpllType = IDTDpll_GetTypeOfDpll(hdlr, nDpll);
  if (dpllType == Dpll_MAX) return;

  /* Configure frequency monitoring */
  IDTDpll_SetFrequencyMonitoringFactor(hdlr, FreqMonFactor_0p8230);

  /* Hard Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Hard Rejecting threshold = (14+1) * 0.823 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetHardAlarmThreshold(hdlr, 10, 14);

  /* Soft Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Soft Rejecting threshold = (14+1) * 0.8230 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetSoftAlarmThreshold(hdlr, 10, 14);

  /* Loop Bandwidth parameters
   *     - Start   = 4Hz 
   *     - Acquire = 4Hz
   *     - Lock    = 0.1Hz 
   */
  if ((dpllType == Dpll_T0) && 
      (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T0))
  {
    IDTDpll_SetBandWidthDamping(hdlr, 
                                nDpll, dpllBandwidthLimitStart,
                                dpllBandwidth_4Hz, DampingFactor_2p5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitAcquire,
                                dpllBandwidth_4Hz, DampingFactor_2p5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_0p1Hz, DampingFactor_5);
  }
  else
  {
    /* T4 had lower bandwidth */
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_18Hz, DampingFactor_5);

  }

  /* Pull in range (ppm) = 8572 * 0.0014
   *                     = 12.0008 ppm
   */
  if (dpllType == Dpll_T0)
    IDTDpll_SetDpllHardAlarmLimit(hdlr, nDpll, 8572);

  /* Phase slope unlimited */
  if (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll])
      IDTDpll_SetPhaseSlope(hdlr, nDpll, phaseSlope_noLimit);
}

/**
 * @brief DPLL configuration for GR-1244 Stratum 3
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_gr1244Stratum3 (T_idtDrvHdlr hdlr,
                              T_idtDpllInstance nDpll)
{
  T_idtDpllType dpllType;

  /* Exit function if we don't not support this DPLL */
  dpllType = IDTDpll_GetTypeOfDpll(hdlr, nDpll);
  if (dpllType == Dpll_MAX) return;

  /* Configure frequency monitoring */
  IDTDpll_SetFrequencyMonitoringFactor(hdlr, FreqMonFactor_0p8230);

  /* Hard Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Hard Rejecting threshold = (14+1) * 0.823 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetHardAlarmThreshold(hdlr, 10, 14);

  /* Soft Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Soft Rejecting threshold = (14+1) * 0.8230 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetSoftAlarmThreshold(hdlr, 10, 14);

  /* Loop Bandwidth parameters
   *     - Start   = 2.5Hz 
   *     - Acquire = 2.5Hz
   *     - Lock    = 1.2Hz 
   */
  if ((dpllType == Dpll_T0) && 
      (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T0))
  {
    IDTDpll_SetBandWidthDamping(hdlr, 
                                nDpll, dpllBandwidthLimitStart,
                                dpllBandwidth_2p5Hz, DampingFactor_10);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitAcquire,
                                dpllBandwidth_2p5Hz, DampingFactor_5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_1p2Hz, DampingFactor_5);
  }
  else
  {
    /* T4 had lower bandwidth */
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_18Hz, DampingFactor_5);

  }

  /* Pull in range (ppm) = 8572 * 0.0014
   *                     = 12.0008 ppm
   */
  if (dpllType == Dpll_T0)
    IDTDpll_SetDpllHardAlarmLimit(hdlr, nDpll, 8572);

  /* Phase slope GR-1244 Stratum2,3E,3 (855ns/s) */
  if (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll])
      IDTDpll_SetPhaseSlope(hdlr, nDpll, phaseSlope_gr1244st3e);
}

/**
 * @brief DPLL configuration for SMC
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_smc (T_idtDrvHdlr hdlr,
                   T_idtDpllInstance nDpll)
{
  T_idtDpllType dpllType;

  /* Exit function if we don't not support this DPLL */
  dpllType = IDTDpll_GetTypeOfDpll(hdlr, nDpll);
  if (dpllType == Dpll_MAX) return;

  /* Configure frequency monitoring */
  IDTDpll_SetFrequencyMonitoringFactor(hdlr, FreqMonFactor_3p2920);

  /* Hard Accepting threshold = (11+1) * 3.292 ppm
   *                          = +/- 39.504 ppm
   * Hard Rejecting threshold = (15+1) * 3.292 ppm
   *                          = +/- 56.672 ppm
   */
  IDTDpll_SetHardAlarmThreshold(hdlr, 11, 15);

  /* Soft Accepting threshold = (11+1) * 3.292 ppm
   *                          = +/- 39.504 ppm
   * Soft Rejecting threshold = (15+1) * 3.292 ppm
   *                          = +/- 52.672 ppm
   */
  IDTDpll_SetSoftAlarmThreshold(hdlr, 11, 15);

  /* Loop Bandwidth parameters
   *     - Start   = 4Hz 
   *     - Acquire = 4Hz
   *     - Lock    = 0.1Hz 
   */
  if ((dpllType == Dpll_T0) && 
      (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T0))
  {
    IDTDpll_SetBandWidthDamping(hdlr, 
                                nDpll, dpllBandwidthLimitStart,
                                dpllBandwidth_4Hz, DampingFactor_2p5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitAcquire,
                                dpllBandwidth_4Hz, DampingFactor_2p5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_0p1Hz, DampingFactor_5);
  }
  else
  {
    /* T4 had lower bandwidth */
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_18Hz, DampingFactor_5);

  }

  /* Pull in range (ppm) = 8572 * 0.0014
   *                     = 12.0008 ppm
   */
  if (dpllType == Dpll_T0)
    IDTDpll_SetDpllHardAlarmLimit(hdlr, nDpll, 8572);

  /* Phase slope unlimited */
  if (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll])
      IDTDpll_SetPhaseSlope(hdlr, nDpll, phaseSlope_noLimit);
}

/**
 * @brief DPLL configuration for G-8262 Option 1
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_g8262EecOption1 (T_idtDrvHdlr hdlr,
                               T_idtDpllInstance nDpll)
{
  T_idtDpllType dpllType;

  /* Exit function if we don't not support this DPLL */
  dpllType = IDTDpll_GetTypeOfDpll(hdlr, nDpll);
  if (dpllType == Dpll_MAX) return;

  /* Configure frequency monitoring */
  IDTDpll_SetFrequencyMonitoringFactor(hdlr, FreqMonFactor_0p8230);

  /* Hard Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Hard Rejecting threshold = (14+1) * 0.823 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetHardAlarmThreshold(hdlr, 10, 14);

  /* Soft Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Soft Rejecting threshold = (14+1) * 0.8230 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetSoftAlarmThreshold(hdlr, 10, 14);

  /* Loop Bandwidth parameters
   *     - Start   = 2.5Hz 
   *     - Acquire = 2.5Hz
   *     - Lock    = 1.2Hz 
   */
  if ((dpllType == Dpll_T0) && 
      (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T0))
  {
    IDTDpll_SetBandWidthDamping(hdlr, 
                                nDpll, dpllBandwidthLimitStart,
                                dpllBandwidth_2p5Hz, DampingFactor_10);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitAcquire,
                                dpllBandwidth_2p5Hz, DampingFactor_5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_1p2Hz, DampingFactor_5);
  }
  else
  {
    /* T4 had lower bandwidth */
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_18Hz, DampingFactor_5);

  }

  /* Pull in range (ppm) = 8572 * 0.0014
   *                     = 12.0008 ppm
   */
  if (dpllType == Dpll_T0)
    IDTDpll_SetDpllHardAlarmLimit(hdlr, nDpll, 8572);

  /* Phase slope GR-813,G.8262 (7.5us/s) */
  if (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll])
      IDTDpll_SetPhaseSlope(hdlr, nDpll, phaseSlope_gr813);
}

/**
 * @brief DPLL configuration for G-8262 Option 2
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_g8262EecOption2 (T_idtDrvHdlr hdlr,
                               T_idtDpllInstance nDpll)
{
  T_idtDpllType dpllType;

  /* Exit function if we don't not support this DPLL */
  dpllType = IDTDpll_GetTypeOfDpll(hdlr, nDpll);
  if (dpllType == Dpll_MAX) return;

  /* Configure frequency monitoring */
  IDTDpll_SetFrequencyMonitoringFactor(hdlr, FreqMonFactor_0p8230);

  /* Hard Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Hard Rejecting threshold = (14+1) * 0.823 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetHardAlarmThreshold(hdlr, 10, 14);

  /* Soft Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Soft Rejecting threshold = (14+1) * 0.8230 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetSoftAlarmThreshold(hdlr, 10, 14);

  /* Loop Bandwidth parameters
   *     - Start   = 4Hz 
   *     - Acquire = 4Hz
   *     - Lock    = 0.1Hz 
   */
  if ((dpllType == Dpll_T0) && 
      (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T0))
  {
    IDTDpll_SetBandWidthDamping(hdlr, 
                                nDpll, dpllBandwidthLimitStart,
                                dpllBandwidth_4Hz, DampingFactor_2p5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitAcquire,
                                dpllBandwidth_4Hz, DampingFactor_2p5);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_0p1Hz, DampingFactor_5);
  }
  else
  {
    /* T4 had lower bandwidth */
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_18Hz, DampingFactor_5);

  }

  /* Pull in range (ppm) = 8572 * 0.0014
   *                     = 12.0008 ppm
   */
  if (dpllType == Dpll_T0)
    IDTDpll_SetDpllHardAlarmLimit(hdlr, nDpll, 8572);

  /* Phase slope GR-813,G.8262 (7.5us/s) */
  if (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll])
      IDTDpll_SetPhaseSlope(hdlr, nDpll, phaseSlope_gr813);
}

/**
 * @brief DPLL configuration for G-813 Option 1
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_g813Option1 (T_idtDrvHdlr hdlr,
                           T_idtDpllInstance nDpll)
{
  /* Same as G8262 option 1 */
  IDTHlApi_g8262EecOption1(hdlr, nDpll);
}

/**
 * @brief DPLL configuration for G-813 Option 2
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_g813Option2 (T_idtDrvHdlr hdlr,
                           T_idtDpllInstance nDpll)
{
  /* Same as IDTHlApi_smc */
  IDTHlApi_smc(hdlr, nDpll);
}


/**
 * @brief DPLL configuration for 1 pulse per second (PPS)
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */ 
void IDTHlApi_pps (T_idtDrvHdlr hdlr,
                   T_idtDpllInstance nDpll)
{
  T_idtDpllType dpllType;

  /* Exit function if we don't not support this DPLL */
  dpllType = IDTDpll_GetTypeOfDpll(hdlr, nDpll);
  if (dpllType == Dpll_MAX) return;

  /* Disable hitless switching */
  IDTDpll_SetHitlessSwitchingFeature(hdlr, 0, 0);

  /* Enable 1 PPS fast lock only for phase */
  IDTDpll_SetPpsFastLock(hdlr, 0, 1);

  /* Configure frequency monitoring */
  IDTDpll_SetFrequencyMonitoringFactor(hdlr, FreqMonFactor_0p8230);

  /* Hard Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Hard Rejecting threshold = (14+1) * 0.823 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetHardAlarmThreshold(hdlr, 10, 14);

  /* Soft Accepting threshold = (10+1) * 0.8230 ppm
   *                          = +/- 9.053 ppm
   * Soft Rejecting threshold = (14+1) * 0.8230 ppm
   *                          = +/- 12.345 ppm
   */
  IDTDpll_SetSoftAlarmThreshold(hdlr, 10, 14);

  /* Loop Bandwidth parameters
   *     - Start   = 0.015Hz 
   *     - Acquire = 0.015Hz
   *     - Lock    = 0.015Hz 
   */
  if ((dpllType == Dpll_T0) && 
      (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T0))
  {
    IDTDpll_SetBandWidthDamping(hdlr, 
                                nDpll, dpllBandwidthLimitStart,
                                dpllBandwidth_15mHz, DampingFactor_1p2);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitAcquire,
                                dpllBandwidth_15mHz, DampingFactor_1p2);
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_15mHz, DampingFactor_1p2);
  }
  else
  {
    /* T4 had lower bandwidth */
    IDTDpll_SetBandWidthDamping(hdlr,  
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_18Hz, DampingFactor_5);

  }

  /* Pull in range (ppm) = 9320 * 0.0014
   *                     = 13.048 ppm
   */
  if (dpllType == Dpll_T0)
    IDTDpll_SetDpllHardAlarmLimit(hdlr, nDpll, 9320);

  /* Phase slope unlimited */
  if (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll])
      IDTDpll_SetPhaseSlope(hdlr, nDpll, phaseSlope_noLimit);

  /* Phase alarm time out = 2 * 5 seconds
   *                      = 10 seconds 
   */
  IDTGeneral_SetTimeoutValue(hdlr, phaseTimeoutMultiplier_2, 5);

  /* Freeze (enable) T0 limit */
  IDTDpll_SetDpllFrozen(hdlr, nDpll, 1);

  /* Configure coarse phase loss detector */
  IDTDpll_SetPhaseCoarseDetector(hdlr, nDpll,
                                 1, /* Enable coarse phase loss detection */
                                 1, /* Use coarse limit setting */
                                 1, /* Limit output of PFD to +/- 1 UI */
                                 0); /* Use coarse limit setting */

  /* Set coarse phase loss detector +/- 1 UI */
  IDTDpll_SetPhaseCoarseLimit(hdlr, nDpll, coarsePhaseLimit_1);

  /* Disable fine phase limit */
  IDTDpll_SetPhaseFineDetectorEnable(hdlr, nDpll, 0);

  /* Set holdover mode to slow averaging */
  IDTDpll_SetHoldoverMode(hdlr, nDpll, Holdover_Slow_Average, 1);

  /* Set temp-holdover mode to slow averaging */
  IDTDpll_SetTempHoldoverMode(hdlr, nDpll, Temp_Holdover_Slow_Average);
}

/**
 * @brief DPLL configuration for wideband
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 */
void IDTHlApi_wideband(T_idtDrvHdlr hdlr,
                         T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    
    /* Exit function if we don't not support this DPLL */
    dpllType = IDTDpll_GetTypeOfDpll(hdlr, nDpll);
    if (dpllType == Dpll_MAX) return;

    /* Configure frequency monitoring */
    IDTDpll_SetFrequencyMonitoringFactor(hdlr, FreqMonFactor_0p8230);

    /* Hard Accepting threshold = (10+1) * 0.8230 ppm
         *                          = +/- 9.053 ppm
         * Hard Rejecting threshold = (14+1) * 0.823 ppm
         *                          = +/- 12.345 ppm
         */
    IDTDpll_SetHardAlarmThreshold(hdlr, 10, 14);

    /* Soft Accepting threshold = (10+1) * 0.8230 ppm
         *                          = +/- 9.053 ppm
         * Soft Rejecting threshold = (14+1) * 0.8230 ppm
         *                          = +/- 12.345 ppm
         */
    IDTDpll_SetSoftAlarmThreshold(hdlr, 10, 14);

    /* Always use Locked BW and damping factor */
    IDTDpll_SetAutoSelBandWidth(hdlr, nDpll, 0);

    /* Set Locked BW to 18Hz and damping factor to 5 */
    IDTDpll_SetBandWidthDamping(hdlr, 
                                nDpll, dpllBandwidthLimitLocked,
                                dpllBandwidth_18Hz, DampingFactor_5);

    /* Pull in range (ppm) = 8572 * 0.0014
         *                              = 12.0008 ppm
         */
    if (dpllType == Dpll_T0)
        IDTDpll_SetDpllHardAlarmLimit(hdlr, nDpll, 8572);

    /* Phase slope unlimited */
    if (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll])
        IDTDpll_SetPhaseSlope(hdlr, nDpll, phaseSlope_noLimit);
}

