/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __APLL_H__
#define __APLL_H__

#include "Define.h"
#include "outputFreq.h"
#include "ApllTypeDef.h"
#include "ApllFreqProfile.h"

#ifdef __cplusplus
extern "C"
{
#endif

/** \addtogroup APLL
 *  @{
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/

/**
 * @brief Utility macro to translate output port to APLL instance
 * @param hdlr Device driver handle
 * @param output External output port number
 * @param outputApll APLL instance
 */
#define IDT_GET_OUTPUT_APLL_CONFIG(hdlr, output, outputApll)    \
    do {                            \
            OS_ASSERT((output)<=IDT_DRV_MAX_OUTPUT);   \
            OS_ASSERT((output)>0);             \
            outputApll = hdlr->deviceConfig_m->outPutApllConfig[(output)]; \
    } while(0)

/**
 * @brief Structure containing APLL per output configuration
 */
typedef struct 
{
    T_idtApllConfigSel apllCfg; /**< APLL stored register configuration selection */
    T_idtApllInputClkSrc inputClk; /**< APLL sourced clock selection */
    T_idtApllDividerValue preDiv; /**< APLL pre-divider value */
    T_idtApllDividerValue	fbDiv; /**< APLL feedback divider value */
    T_idtApllDividerValue outputDiv; /**< APLL output divider value */
    T_idtApllFreqDoublerSel dbleSel;  /**< APLL frequency doubler in feedback path */
    T_idtApllXtalId xtal; /**< APLL crystal selection */
    T_idtApllOutputSigType sigType; /**< APLL output signal configuration */
    T_idtApllOutputEn enOutput; /**< APLL output enable */
} T_idtApllConfigPerOutput;

/**
 * @brief Structure containing APLL full configuration
 */
typedef struct
{
	T_idtApllConfigSel apllCfg; /**< APLL stored register configuration selection */
	T_idtApllInputClkSrc inputClk; /**< APLL sourced clock selection */
	struct preDiv_s
	{
		T_idtApllDividerValue preDivConfig0; /**< APLL pre-divider value for configuration 0 */
		T_idtApllDividerValue preDivConfig1; /**< APLL pre-divider value for configuration 1 */
	} preDiv;
	struct fbDiv_s
	{
		T_idtApllDividerValue fbDivConfig0; /**< APLL feedback divider value for configuration 0 */
		T_idtApllDividerValue fbDivConfig1; /**< APLL feedback divider value for configuration 1 */
	} fbDiv;
	struct outputDiv_s
	{
		struct qaDiv_s
		{
			T_idtApllDividerValue outputDivConfig0; /**< APLL QA output divider value for configuration 0 */
			T_idtApllDividerValue	outputDivConfig1; /**< APLL QB output divider value for configuration 1 */
		} qaDiv;
		struct qbDiv_s
		{
			T_idtApllDividerValue outputDivConfig0; /**< APLL QB output divider value for configuration 0 */
			T_idtApllDividerValue	outputDivConfig1; /**< APLL QB output divider value for configuration 1 */
		} qbDiv;
	} outputDiv;
	T_idtApllFreqDoublerSel dbleSel;  /**< APLL frequency doubler in feedback path */
    T_idtApllMasterResetSync mrSync; /**< APLL Master reset sync */
    T_idtApllBypass byPass; /**< APLL bypass configuration */
    T_idtApllShutoff shutOff; /**< APLL shut off mode */
	T_idtApllXtalId xtal; /**< APLL crystal selection */
	struct outputConfig_s
	{
		struct qaConfig_s
		{
			T_idtApllOutputSigType outputSigType; /**< APLL QA output signal type */
			T_idtApllOutputEn	enOutput; /**< APLL QA output enable */
		} qaConfig;
		struct qbConfig_s
		{
			T_idtApllOutputSigType outputSigType; /**< APLL QB output signal type */
			T_idtApllOutputEn enOutput; /**< APLL QB output enable */
		} qbConfig;
	} outputConfig;
}T_idtApllConfig;

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/

void IDTApll_SetApllConfigPerOutput(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId,
	T_idtApllOutput output,
	T_idtApllConfigPerOutput *apllConfigPerOutput);

void IDTApll_GetApllConfigPerOutput(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId,
	T_idtApllOutput output,
	T_idtApllConfigPerOutput *apllConfigPerOutput);

void IDTApll_SetApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId,
	T_idtApllConfig *apllConfig);

void IDTApll_GetApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId,
	T_idtApllConfig *apllConfig);

outputFrequency_t IDTApll_ApllInput2DpllOutput(T_idtApllInputFreqType apllInputFreq);

T_idtApllOutputFreqType IDTApll_DpllOutput2ApllOutput(outputFrequency_t dpllOutput);

const T_idtApllFreqMapping* IDTApll_GetApllFreqTableEntry(
	T_idtDrvHdlr hdlr,
	T_idtOutputApllConfig outputApll,
	outputFrequency_t nOutFreq);

T_idtApllConfigSel IDTApll_GetNonActiveApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId);

void IDTApll_Switch2NonActiveApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId  apllId);

T_idtApllXtalId IDTApll_GetXtalIdFromXtalFreq(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId,
	T_idtApllXtalType apllVcxoFreq);

T_osBool IDTApll_XtalConfigured(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId);

T_osBool IDTApll_ValidXtalInput(
    const T_idtDrvDeviceConfig *deviceConfig,
    T_idtApllXtal *xtalList,
    T_osUint8 numberOfXtal);

T_idtApllXtal* IDTApll_GetSupportedXtal(
    const T_idtDrvDeviceConfig *deviceConfig,
    T_osUint8 *numberOfXtal);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __APLL_H__ */


