/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#ifndef __DEFINE_H__
#define __DEFINE_H__

#include "ApllTypeDef.h"
#include "RegisterDef.h"
#include "osAdapt.h"
#include "osNumeric.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/
typedef struct drvHdlr_s *T_idtDrvHdlr;

/**
 * @brief Enumerated type for selecting DPLLs (T0 or T4)
 */
typedef enum
{
    Dpll_T0 = 0, /**< Select T0 DPLL */
    Dpll_T4 = 1, /**< Select T4 DPLL */
    Dpll_MAX /**< \internal Not used */
} T_idtDpllType;


/* nMODE: MPU interface mode */
#define MPU_I2C_MODE   		0x00
#define MPU_SERIAL_MODE   	0x01

/**
 * @brief Utility macro to set APLL configuration to NULL
 * for the output path that does not have APLL
 */
#define NULL_APLL_CONFIG    \
{                           \
    APLL_INST_MAX,          \
    0,                      \
    APLL_OUTPUT_MAX,        \
    0,                      \
}

/**
 * @brief Utility macro to check if APLL is configured
 * @param apllConfig APLL configuration
 */
#define APLL_CONFIGURED(apllConfig)				   \
   ((apllConfig.apllInst != APLL_INST_MAX) &&      \
    (apllConfig.apllInput) &&	                   \
    (apllConfig.apllOutput != APLL_OUTPUT_MAX) &&  \
    (apllConfig.extOutput))	                       \
    ? 1 : 0

/*End of definition used in Output.c */

/** 
 * @brief Enumerated type listing supported device(s)  
 */
typedef enum
{
	IDT_82V3910, /**< 82V3910 */
    IDT_82V3911, /**< 82V3911 */
    IDT_82V33930, /**< 82V33930 */
    IDT_8V89316, /**< 8V89316 */
    IDT_8V89317, /**< 8V89317 */
	IDT_DEVICE_MAX /**< \internal Not used. */
} T_idtDeviceType;

/** @brief Define maximum number of supported inputs */
#define IDT_DRV_MAX_INPUT    14

/** @brief Define maximum number of supported outputs */
#define IDT_DRV_MAX_OUTPUT   11

/** @brief Define maximum number of APLL XTALs per device */
#define IDT_DRV_MAX_APLL_XTAL	 4

/** @brief Define maximum number of XTALs per APLL */
#define IDT_DRV_MAX_XTAL_PER_APLL 2

/** @brief Define maximum number of supported DPLLs */
typedef enum {
    DPLL_INSTANCE_1,   /**< DPLL instance #1 */
    DPLL_INSTANCE_2,   /**< DPLL instance #2 */
    DPLL_INSTANCE_MAX  /**< \internal Not used */
} T_idtDpllInstance;

/**
 * @brief Function type for reading from i2c device
 * @param usrData optional data passed to write function
 * @param deviceAddr device address for bus that requires slave address (e.g. i2c)
 * @param addrOff address offset
 * @return read data
 */
typedef T_osUint8 (*T_idtReadFunc)(void *usrData, T_osUint8 deviceAddr, T_osUint8 addrOff);

/**
 * @brief Function type to writing to i2c device
 * @param usrData optional user data passed to read function
 * @param deviceAddr for bus that requires slave address (e.g. i2c)
 * @param addrOff address offset
 * @param data data to write
 */
typedef void (*T_idtWriteFunc)(void *usrData, T_osUint8 deviceAddr, T_osUint8 addrOff, T_osUint8 data);

/**
 * @brief Function type to initialize device access
 *
 * Optional binding for device access
 *
 * @param userData optional user data passed to initialization function
 */
typedef void (*T_idtAccessInitFunc)(void *userData);

/**
 * @brief Function type to deintialize device access
 *
 * Optional binding for device access
 *
 * @param userData optional user data passed to deinitialization function
 */
typedef void (*T_idtAccessDeInitFunc)(void *userData);

/**
 * @brief Function type to translate i2c address used internally by the driver
 * @param i2cAddr 7-bit unshifted i2c slave address for the WAN PLL
 * @param transData Helper data for translating
 * @return Translated i2c address used internally by the device driver
 */
typedef T_osUint8 (*T_idtI2CaddrTransFunc)(T_osUint8 i2cAddr, void* transData);

/**
 * @brief Function type to sanity check the input frequency
 * @param hdlr driver handler
 * @param nInputNo input port number
 * @param nFrequency input frequency (in Hz)
 * @param auxData auxiliary data (optional)
 * @return E_osTrue - nFrequency is supported,
 *              E_osFalse - nFrequency is not supported
 */
typedef T_osBool (*T_idtInputFreqValid)(T_idtDrvHdlr hdlr, T_osUint8 nInputNo, T_osUint32 nFrequency, void* auxData);

/**
 * @brief Function type to sanity check the output frequency
 * @param hdlr driver handler
 * @param nOutN output port number
 * @param nOutFreq Output frequency /see outputFrequency_t for list of
 * supported frequencies
 * @param auxData auxiliary data (optional)
 * @return E_osTrue - nOutFreq is supported,
 *              E_osFalse - nOutFreq is not supported
 */
typedef T_osBool (*T_idtOuputFreqValid)(T_idtDrvHdlr hdlr, T_osUint8 nOutN, T_osUint32 nOutFreq, void* auxData);

/**
 * @brief Initialization structure detailing device access information
 * for device driver.
 */
typedef struct S_idtDrvAccess
{
	T_idtAccessInitFunc initFunc_m; /**< Access initialization */
	T_idtAccessDeInitFunc deinitFunc_m; /**< Access deinitialization */
	T_idtReadFunc readFunc_m; /**< Read function */
	T_idtWriteFunc writeFunc_m; /**< Write function */
	void *userData_m; /**< Extra user data */
} T_idtDrvAccess;

/**
 *@brief Structure containing APLL inforatmion
 */
typedef struct 
{
    T_idtApllId apllInst; /**< APLL instanace ID */
    T_osUint8 apllInput; /**< APLL input port number */
    T_idtApllOutput apllOutput; /**< APLL output port number */
    T_osUint8 extOutput; /**< External output port number */
} T_idtOutputApllConfig;

/**
 * @brief Device type/variant information.
 *
 * This data structure contains device variant information.
 *
 */
typedef struct S_idtDrvDeviceConfig
{
    T_osUint8 inputExt2IntXlate_ma[IDT_DRV_MAX_INPUT + 1]; /**< Input translation table (external to internal) */
    T_osUint8 inSyncXlate_ma[2]; /**< Input sync translation table */
    T_osUint8 outputExt2IntXlate_ma[IDT_DRV_MAX_OUTPUT + 1]; /**< Output translation table (external to internal) */
    T_idtOutputApllConfig outPutApllConfig[IDT_DRV_MAX_OUTPUT + 1]; /**< Output APLL configuration */
    T_osUint8 outSyntXlate_ma[2]; /**< Output sync translation table */
    T_osUint8 numberOfSonetGigEthPath_m; /**< Number of Sonet GigE path */
    T_osUint8 numberOfApll_m; /**< Number of APLLs */
    T_osUint8 populatedApll_ma[APLL_INST_MAX]; /**< Populated APLL(s) */
    T_osUint8 pllExt2IntXlate_ma[DPLL_INSTANCE_MAX]; /**< DPLL translation table (external to internal) */
    T_osUint8 pllInt2ExtXlate_ma[DPLL_INSTANCE_MAX]; /**< DPLL translation table (internal to external) */
    T_idtDpllType t0PllMode_m; /**< If T0 exists, T0 mode */
    T_osUint8 dcoModeSupport_ma[DPLL_INSTANCE_MAX]; /**< DPLL support DCO mode */
    T_osUint8 phaseSlopeLimiting_ma[DPLL_INSTANCE_MAX]; /**< DPLL support phase slope limiting */
    T_osUint8 maxNumXtalPerApll_m; /**< Maximum number of XTALs per APLL */
    T_osUint8 supportedXtalId_ma[APLL_XTAL_MAX]; /**< Supported XTAL id */
    T_osUint8 supportedXtalFreq_ma[APLL_XTAL_MAX][APLL_XTAL_FREQ_MAX]; /**< Supported XTAL frequency */
    T_idtInputFreqValid inputFreqValidFunc_m; /**< Input frequency validation function */
    T_idtOuputFreqValid outputFreqValidFunc_m; /**< Output frequency validation function */
} T_idtDrvDeviceConfig;

/**
 * @brief Stucture used to keep track of output path
 * configuration. Not used if basic driver used. High level API uses
 * this.
 */
struct S_IDTPathConfiguration;

/**
 * @brief Device driver handler. 
 *
 * All device driver functions have driver handlers as an input
 * parameter. This handler allows the device driver to support an
 * unlimited number devices.
 */
struct drvHdlr_s
{
    const char *name_m; /**< Name of driver instance */
    T_idtDeviceType deviceType_m; /**< Device Type */
    T_idtDrvAccess drvAccess_m; /**< Device access functions */
    const T_idtDrvDeviceConfig *deviceConfig_m; /**< Device configuration */
	T_osUint8 numberOfApllXtal_m; /**< Number of APLL XTALs */
	T_idtApllXtal xtalList[IDT_DRV_MAX_APLL_XTAL]; /**< List of APLL XTALs */
    struct S_IDTPathConfiguration *currOutPathCfg_mp; /**< Current output path configuration  - used by high level API code. Basic driver doesn't require this. */ 
	void *dpllI2CTransData_mp; /**< Helper data for translating I2C address mapping to dpll register space */
	void *apllI2CTransData_mp; /**< Helper data for translating I2C address mapping to apll register space */
	T_idtI2CaddrTransFunc i2cAddrTransFunc_m; /**< Function to translate I2C address */
	T_osUint8 i2cAddr; /**< 7-bit unshifted i2c slave address for PLL */
};

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __DEFINE_H__ */
