/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __GENERAL_H__
#define __GENERAL_H__

#include "osNumeric.h"
#include "Define.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \defgroup General General Function Module
 *
 * @brief Functions in this group are related to device wide provisioning and
 * status.
 *
 * @{
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/

/** @brief Enumerated type listing types of selectable network type */
typedef enum
{
	Network_SDH = 0, /**< SDH network */
	Network_SONET = 1, /**< SONET network */
	Network_MAX /**< \internal Not used. */
} networkType_t;

/** @brief Enumerated type listing phase alarm timeout multipliers */
typedef enum
{
	phaseTimeoutMultiplier_2 = 0, /**< 2x Multiplier */
	phaseTimeoutMultiplier_4 = 1, /**< 4x Multiplier  */
	phaseTimeoutMultiplier_8 = 2, /**< 8x Multiplier  */
	phaseTimeoutMultiplier_16 = 3, /**< 18x Multiplier  */
	phaseTimeoutMultiplier_MAX /**< \internal Not used. */
} phaseTimeoutMultiplier_t;

/**
 * @brief I2C address translation information used internally by the driver
 */
typedef struct
{
	T_osUint8 bitLocation;
	T_osUint8 bitValue;
} T_idtI2CtransData;

#define Activate_Edge_Rising	0
#define Activate_Edge_Falling	1

/*Interrupt */
#define INT_Active_Low	0
#define INT_Active_High 1

/** @brief Conversion from parts per trillion to register unit value
 * for nominal (oscillator) offset. */
#define NOMINAL_PPT_TO_2COMPL(ppt) (T_osUint32)(((ppt)*10)/884)

/** @brief Conversion from parts per trillion to register unit value
 * for nominal (oscillator) offset. */
#define NOMINAL_2COMPL_TO_PPT(twoCompl) (long)(((twoCompl)*884)/10)

/** @brief Maximum oscillator offset value */
#define NOMINAL_MAX_PPT      94893

/** @brief Minimum oscillator offset value */
#define NOMINAL_MIN_PPT     -94893

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/

T_osUint16 IDTGeneral_GetDeviceID(T_idtDrvHdlr hdlr); /*Get the ID of the device; */

void IDTGeneral_SetOscFreqOffset(T_idtDrvHdlr hdlr, long pptOffset); /* Set compensation to the oscillator offset*/
long IDTGeneral_GetOscFreqOffset(T_idtDrvHdlr hdlr); /* Get compensation to the oscillator offset*/

void IDTGeneral_SetNetworkType(T_idtDrvHdlr hdlr, networkType_t nType); /*Set the network type,SDH or SONET*/
networkType_t IDTGeneral_GetNetworkType(T_idtDrvHdlr hdlr); /* Get the network type,SDH or SONET*/

void IDTGeneral_SetRevertiveMode(T_idtDrvHdlr hdlr, T_osUint8 nEnable); /*Set the device running in Revertive mode or not*/
T_osUint8 IDTGeneral_GetRevertiveMode(T_idtDrvHdlr hdlr); /* Get the device running in Revertive mode or not*/

void IDTGeneral_SetTimeoutValue(T_idtDrvHdlr hdlr, phaseTimeoutMultiplier_t nMultiFactor,
		T_osUint8 nTimeout); /*Set value of time out of phase alarm*/
void IDTGeneral_GetTimeoutValue(T_idtDrvHdlr hdlr, 
                     phaseTimeoutMultiplier_t *nMultiFactor_p,
                     T_osUint8 *nTimeout_p);  /*Get value of time out of phase alarm*/

void IDTGeneral_SetOscActiveEdge(T_idtDrvHdlr hdlr, T_osUint8 nEdge); /*Set the activate edge of mail oscillator;*/
T_osUint8 IDTGeneral_GetOscActiveEdge(T_idtDrvHdlr hdlr); /*Get the activate edge of mail oscillator;*/

void IDTGeneral_SetProtectMode(T_idtDrvHdlr hdlr); /*Enable protected register access mode*/
void IDTGeneral_SetSingleUnprotectMode(T_idtDrvHdlr hdlr); /*Set single unprotection mode*/
void IDTGeneral_SetFullyUnprotectMode(T_idtDrvHdlr hdlr); /*Set fully unprotection mode*/

void IDTGeneral_SetFlagOnTDO(T_idtDrvHdlr hdlr, T_osUint8 nEnable); /*Set TDO pin to indicate the fail of selected clock*/
T_osUint8 IDTGeneral_GetFlagOnTDO(T_idtDrvHdlr hdlr); /*Get TDO pin to indicate the fail of selected clock*/

void IDTGeneral_SetUltraFastSwitch(T_idtDrvHdlr hdlr, T_osUint8 nEnable); /*Set Ultra-fast-switch enable or not*/
T_osUint8 IDTGeneral_GetUltraFastSwitch(T_idtDrvHdlr hdlr); /* Get Ultra-fast-switch enable status */

void IDTGeneral_SetHardAlarm(T_idtDrvHdlr hdlr, T_osUint8 nEnable); /*Set Hard-alarm enable when the input exceeds the threshold;*/
T_osUint8 IDTGeneral_GetHardAlarm(T_idtDrvHdlr hdlr); /*Get Hard-alarm enable when the input exceeds the threshold;*/

T_osUint8 IDTGeneral_i2cTranslate(T_osUint8 i2cAddr, void* transData);

void IDTGeneral_InitDeviceCommon(T_idtDrvHdlr hdlr);

T_idtDrvHdlr IDTGeneral_InitDriverCommon(const char      *name, 
                                         T_idtDrvAccess  drvAccess,
                                         T_idtDeviceType deviceType,
                                         const T_idtDrvDeviceConfig *deviceConfig,
                                         int             useHighLevelApi); /* Initialize driver */
void IDTGeneral_DeInitDriver(T_idtDrvHdlr hdlr); /* Deinitialize driver */

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __GENERAL_H__ */
