/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#ifndef __HAL_H__
#define __HAL_H__

#include "osNumeric.h"
#include "Define.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \defgroup Hal Hardware Abstraction Layer Module
 *
 * @brief Functions in this group are related to hardware abstraction layer.
 *
 * @{
 */
/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/
#define IDT_HAL_USE_MACRO 0

/**
 * @brief Define function to set bitfield value of read data from device
 *
 * This macro function can be used after a register is read from a device.
 * This macro <b>doesn't</b> access the device.
 *
 * @param regVal register data value.
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @param data Bit field data to set
 */
#define IDTHAL_ModifyBitfield(regVal,mask,lsb,data)  \
   regVal = ((regVal & ~(mask)) |                    \
            (((data) << (lsb)) & (mask)))

/**
 * @brief Define function to get bitfield value of read data from device
 *
 * This macro function can be used after a register is read from a device.
 * This macro <b>doesn't</b> access the device.
 *
 * @param regVal register data value.
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 */
#define IDTHAL_GetBitfield(regVal,mask,lsb)  \
  (((regVal) & (mask)) >> (lsb))

#if (IDT_HAL_USE_MACRO)
/**
 * @brief Define function to write a bitfield to a device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @param data Bit field data to write
 */
#define IDTHAL_WriteBitfield(hdlr,offset,mask,lsb,data) \
do {                                                 \
  T_osUint8 regVal = IDTHal_Read(hdlr,offset);	         \
  IDTHAL_ModifyBitfield(regVal,mask,lsb,data);       \
  IDTHal_Write(hdlr,offset,regVal);                     \
} while(0)

/**
 * @brief Define function to write a bitfield to a device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @param data Bit field data to write
 */
#define IDTHAL_WriteBitfield_Ext(hdlr,offset,mask,lsb,data) \
do {                                                     \
  T_osUint8 regVal = IDTHal_Read_Ext(hdlr,offset);	         \
  IDTHAL_ModifyBitfield(regVal,mask,lsb,data);           \
  IDTHal_Write_Ext(hdlr,offset,regVal);                     \
} while(0)

/**
 * @brief Define function to read a bitfield from device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 */
#define IDTHAL_ReadBitfield(hdlr,offset,mask,lsb) \
(IDTHAL_GetBitfield(IDTHal_Read(hdlr,offset),(mask),(lsb)))

/**
 * @brief Define function to read a bitfield from device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 */
#define IDTHAL_ReadBitfield_Ext(hdlr,offset,mask,lsb) \
(IDTHAL_GetBitfield(IDTHal_Read_Ext(hdlr,offset),(mask),(lsb)))
#else
/**
 * @brief Define function to write a bitfield to a device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @param data Bit field data to write
 */
void IDTHAL_WriteBitfield(T_idtDrvHdlr hdlr,
                          T_osUint8    offset,
                          T_osUint8    mask,
                          T_osUint8    lsb,
                          T_osUint8    data);

/**
 * @brief Define function to write a bitfield to a device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @param data Bit field data to write
 */
void IDTHAL_WriteBitfield_Ext(T_idtDrvHdlr hdlr,
                              T_osUint8    offset,
                              T_osUint8    mask,
                              T_osUint8    lsb,
                              T_osUint8    data);

/**
 * @brief Define function to read a bitfield from device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @return Bit field value
 */
T_osUint8 IDTHAL_ReadBitfield(T_idtDrvHdlr hdlr,
                              T_osUint8    offset,
                              T_osUint8    mask,
                              T_osUint8    lsb);

/**
 * @brief Define function to read a bitfield from device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @return Bit field value
 */
T_osUint8 IDTHAL_ReadBitfield_Ext(T_idtDrvHdlr hdlr,
                                  T_osUint8    offset,
                                  T_osUint8    mask,
                                  T_osUint8    lsb);
#endif

/**
 * @brief Function to read from device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 */
T_osUint8 IDTHal_Read(T_idtDrvHdlr hdlr, T_osUint8 offset);

/**
 * @brief Define function to write to device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param data Data to write
 */
void IDTHal_Write(T_idtDrvHdlr hdlr, T_osUint8 offset, T_osUint8 data);

/**
 * @brief Function to read from device apll address space
 * @param hdlr Device driver handler
 * @param offset Register address offset
 */
T_osUint8 IDTHal_Read_Ext(T_idtDrvHdlr hdlr, T_osUint8 offset);

/**
 * @brief Function to write to device apll address space
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param data Data to write
 */
void IDTHal_Write_Ext(T_idtDrvHdlr hdlr, T_osUint8 offset, T_osUint8 data);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __HAL_H__ */
