/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __OUTPUT_H__
#define __OUTPUT_H__

#include "osNumeric.h"
#include "Define.h"
#include "Apll.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \defgroup Output Output Function Module
 *
 * @brief Functions in this group are related to output port provisioning.
 *
 * @{
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/

/**
 * @brief Utility macro to check if output port is supported and to
 * translated from external port to internal port
 * @param hdlr Device driver handle
 * @param ext External output port number
 * @param internal Internal output number
 */
#define IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr,ext,internal)     \
    do {                                \
        OS_ASSERT((ext)<=IDT_DRV_MAX_OUTPUT);          \
        OS_ASSERT((ext)>0);                    \
        OS_ASSERTS(hdlr->deviceConfig_m->outputExt2IntXlate_ma[(ext)], \
                    ("Unsupported output port %d\n",(ext)));    \
        internal = hdlr->deviceConfig_m->outputExt2IntXlate_ma[(ext)]; \
    } while(0)

/**
 * @brief Utility macro to check if output port is populated
 * @param hdlr Device driver handle
 * @param ext External output port number
 * @return a positive number - popluated, 0 - not populated
 */
#define IDT_EXT_OUTPORT_POPULATED(hdlr,ext) \
    (hdlr->deviceConfig_m->outputExt2IntXlate_ma[(ext)])

/**
 * @brief Enumerated type listing possible output port sources (from
 *    DPLL(s)) This enumerated type is used in conjunction with DPLL
 *    instance to select output DPLL frequency to output
 *    dividers. The following table outlines
 *
 */
typedef enum
{
    OutputPath_SonetGigEth, /**< From corresponding Sonet/GigE path */
    OutputPath_7776kHz, /**< From SONET/SDH path 77.76MHz */
    OutputPath_Eth, /**< From Ethernet path 25MHz */
    OutputPath_16E1_16T1, /**< From 16E1/16T1 path
                           * 32.768MHz/24.704MHz*/
    OutputPath_12E1_E3_T3, /**< From 12E1/E3/T3 path
                            * 24.576MHz/34.368MHz/44.736MHz */
    OutputPath_GSM_16E1_16T1, /**< From GSM/16E1/16T1 path
                               * 26MHz/32.768MHz/16.384MHz*/
    OutputPath_GPS, /**< From GPS path 40MHz*/
    OutputPath_OBSAI,  /**< From OBSAI path 30.72MHz*/
    OutputPath_24T1, /**< From 24T1 path 37.056MHz*/
    OutputPath_MAX /**< \internal Not Used */
} T_outputPathType;

/**
 * @brief Enumerated type listing type of supported output port
 * interfaces
 */
typedef enum {
   OUTPUTIF_AMI, /**< AMI */
   OUTPUTIF_CMOS, /**< CMOS */
   OUTPUTIF_LVDS, /**< LVDS */
   OUTPUTIF_PECL, /**< PECL */
   OUTPUTIF_MAX /**< \internal Not used. */
} T_outputInterface;

/**
 * @brief Enumerated type listing supported frame syncs
 */
typedef enum {
    frameSync_FRSYNC, /**< FR Sync (8 kHz / 1 PPS) */
    frameSync_MFRSYNC, /**< MFR Sync (2kHz / 1 PPS) */
    frameSync_MAX /**< \internal Not used */
} T_frameSync;

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/

void IDTOutput_SetOutputConfig(T_idtDrvHdlr hdlr,
                               T_osUint8 nOutN,
                               T_idtDpllInstance nDpll,
                               T_outputPathType nPath,
                               T_osUint8 nFactor,
                               T_idtApllConfigPerOutput *apllConfig); /*Set the configuration of Output port */

void IDTOutput_GetOutputConfig(T_idtDrvHdlr hdlr,
                               T_osUint8 nOutN,
                               T_idtDpllInstance *nDpll_p,
                               T_outputPathType *nPath_p,
                               T_osUint8 *nFactor_p,
                               T_idtApllConfigPerOutput *apllConfig); /* Get output port configuration */

void IDTOutput_SetOutputInvert(T_idtDrvHdlr hdlr,
                               T_osUint8 nOutN,
                               T_osUint8 invert); /* Set the specified output port to generate a invert signal; */

T_osUint8 IDTOutput_GetOutputInvert(T_idtDrvHdlr hdlr,
                                    T_osUint8 nOutN); /* Get specified output signal inversion status */

void IDTOutput_SetOutputEnable(T_idtDrvHdlr hdlr,
                               T_osUint8 nOutN,
                               T_osUint8 nEnable); /* Enable output port. */

T_osUint8 IDTOutput_GetOutputEnable(T_idtDrvHdlr hdlr,
                                    T_osUint8 nOutN); /* Get enable/disable status of output port */

void IDTOutput_SetOuputpuInterface(T_idtDrvHdlr hdlr,
                                   T_osUint8 nOutN,
                                   T_outputInterface outIf); /* Set output port interface type */

T_outputInterface IDTOutput_GetOuputpuInterface(T_idtDrvHdlr hdlr,
                                                T_osUint8 nOutN); /* Get output port interface type */

void IDTOutput_SetFrameSync(T_idtDrvHdlr hdlr,
                            T_frameSync frameSync,
                            T_osUint8 enable,
                            T_osUint8 invert,
                            T_osUint8 pulse); /* Provision frame sync */

void IDTOutput_GetFrameSync(T_idtDrvHdlr hdlr,
                            T_frameSync frameSync,
                            T_osUint8 *enable_p,
                            T_osUint8 *invert_p,
                            T_osUint8 *pulse_p); /* Get frame sync config */

void IDTOutput_SetFrameSyncPulseEdge(T_idtDrvHdlr hdlr,
                                     T_osUint8   isRisingEdge); /* Provision frame sync pulse trigger */

T_osUint8 IDTOutput_GetFrameSyncPulseEdge(T_idtDrvHdlr hdlr); /* Get frame sync pulse trigger */

void IDTOutput_DisableApllInput(T_idtDrvHdlr hdlr, T_idtOutputApllConfig outputApllConfig); /* Disbale input to APLL */

void IDTOutput_DisableAllIntOutput(T_idtDrvHdlr hdlr); /* Disable all internal outputs */

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __OUTPUT_H__ */

