/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "Apll.h"
#include "ApllApi.h"
#include "osAdapt.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * @brief Check if APLL instance Id and APLL XTAL Id combo is valid
 * @param hdlr Device driver handler
 * @param apllId APLL instance<ul>
 *                 <li>APLL_INST_1 - APLL #1</li>
 *                 <li>APLL_INST_2 - APLL #2</li></ul>
 * @param apllXtalId APLL XTAL Id<ul>
 *                 <li>APLL_XTAL_1_APLL1 - XTAL1 for APLL1</li>
 *                 <li>APLL_XTAL_2_APLL2 - XTAL2 for APLL2</li>
 *                 <li>APLL_XTAL_3_APLL1 - XTAL3 for APLL1</li>
 *                 <li>APLL_XTAL_4_APLL2 - XTAL4 for APLL2</li></ul>
 * @return E_osTrue -valid combination, E_osFalse - invalid combination
 */
static T_osBool IDTApll_ValidApllInstIdXtalIdCombo(
	T_idtApllId apllId,
	T_idtApllXtalId apllXtalId)
{
	OS_TRACE(("APLL id: %d, xtal id: %d\n", apllId, apllXtalId));

	if ((APLL_INST_1 == apllId)&&((APLL_XTAL_1_APLL1 == apllXtalId) || (APLL_XTAL_3_APLL1 == apllXtalId)))
		return E_osTrue;
	else if ((APLL_INST_2 == apllId)&&((APLL_XTAL_2_APLL2 == apllXtalId) || (APLL_XTAL_4_APLL2 == apllXtalId)))
		return E_osTrue;

	return E_osFalse;
}

/**
 * @brief Convert APLL output port A divider value to ENUM
 * @param qaOutputVal ALL output port A divider value
 * @return APLL output port A divider ENUM<ul>
 *                 <li>APLL_QA_ODSEL_DIV_25 - divide by 25</li>
 *                 <li>APLL_QA_ODSEL_DIV_5 - divide by 5</li>
 *                 <li>APLL_QA_ODSEL_DIV_4 - divide by 4</li>
 *                 <li>APLL_QA_ODSEL_DIV_2 - divide by 2</li>
 *                 <li>APLL_QA_ODSEL_DIV_1 - divide by 1</li>
 *                 <li>APLL_QA_ODSEL_DIV_MAX - invalid divider value</li></ul>
 */
static T_idtApllQaDiv IDTApll_QaOutputVal2Enum(T_idtApllDividerValue qaOutputVal)
{
	T_idtApllQaDiv qaDiv;

	OS_TRACE(("APLL QA output divider value: %d\n", qaOutputVal));

	switch (qaOutputVal)
	{
		case 25:
			qaDiv = APLL_QA_ODSEL_DIV_25;
		break;

		case 5:
			qaDiv = APLL_QA_ODSEL_DIV_5;
		break;

		case 4:
			qaDiv = APLL_QA_ODSEL_DIV_4;
		break;

		case 2:
			qaDiv = APLL_QA_ODSEL_DIV_2;
		break;

		case 1:
			qaDiv = APLL_QA_ODSEL_DIV_1;
		break;

		default:
			OS_ERROR(("Invalid QA ouput divider value: %d\n", qaOutputVal));
			qaDiv = APLL_QA_ODSEL_DIV_MAX;
		break;
	}

	return qaDiv;
}

/**
 * @brief Convert APLL output port B divider value to ENUM
 * @param qaOutputVal ALL output port B divider value
 * @return APLL output port B divider ENUM<ul>
 *                <li>APLL_QB_ODSEL_DIV_8 - divide by 8</li>
 *                <li>APLL_QB_ODSEL_DIV_5 - divide by 5</li>
 *                <li>APLL_QB_ODSEL_DIV_4 - divide by 4</li>
 *                <li>APLL_QB_ODSEL_DIV_2 - divide by 2</li>
 *                <li>APLL_QB_ODSEL_DIV_1 - divide by 1</li>
 *                <li>APLL_QB_ODSEL_DIV_MAX - invalid divider value</li></ul> 
 */
static T_idtApllQbDiv IDTApll_QbOutputVal2Enum(T_idtApllDividerValue qbOutputVal)
{
	T_idtApllQaDiv qbDiv;

	OS_TRACE(("APLL QB output divider value: %d\n", qbOutputVal));

	switch (qbOutputVal)
	{
		case 8:
			qbDiv = APLL_QB_ODSEL_DIV_8;
		break;

		case 5:
			qbDiv = APLL_QB_ODSEL_DIV_5;
		break;

		case 4:
			qbDiv = APLL_QB_ODSEL_DIV_4;
		break;

		case 2:
			qbDiv = APLL_QB_ODSEL_DIV_2;
		break;

		case 1:
			qbDiv = APLL_QB_ODSEL_DIV_1;
		break;

		default:
			OS_ERROR(("Invalid QB output divider value: %d\n", qbOutputVal));
			qbDiv = APLL_QB_ODSEL_DIV_MAX;
		break;
	}

	return qbDiv;
}

/**
 * @brief Convert APLL output port A divider ENUM to value
 * @param qaOutputEnum APLL output port A divider ENUM<ul>
 *                 <li>APLL_QA_ODSEL_DIV_25 - divide by 25</li>
 *                 <li>APLL_QA_ODSEL_DIV_5 - divide by 5</li>
 *                 <li>APLL_QA_ODSEL_DIV_4 - divide by 4</li>
 *                 <li>APLL_QA_ODSEL_DIV_2 - divide by 2</li>
 *                 <li>APLL_QA_ODSEL_DIV_1 - divide by 1</li><ul>
 *@return APLL output port A divider value
 */
static T_idtApllDividerValue IDTApll_Enum2QaOutputVal(T_idtApllQaDiv qaOutputEnum)
{
	T_idtApllDividerValue qaDiv;

	OS_TRACE(("APLL QA output enum: %d\n", qaOutputEnum));

	switch (qaOutputEnum)
	{
		case APLL_QA_ODSEL_DIV_25:
			qaDiv = 25;
		break;

		case APLL_QA_ODSEL_DIV_5:
			qaDiv = 5;
		break;

		case APLL_QA_ODSEL_DIV_4:
			qaDiv = 4;
		break;

		case APLL_QA_ODSEL_DIV_2:
			qaDiv = 2;
		break;

		case APLL_QA_ODSEL_DIV_1:
			qaDiv = 1;
		break;

		default:
			OS_ERROR(("Invalid QA ouput divider %d\n", qaOutputEnum));
			qaDiv = INVALID_DIVIDER_VALUE;
		break;
	}

	return qaDiv;
}

/**
 * @brief Convert APLL output port B divider ENUM to value
 * @param APLL output port B divider ENUM<ul>
 *                <li>APLL_QB_ODSEL_DIV_8 - divide by 8</li>
 *                <li>APLL_QB_ODSEL_DIV_5 - divide by 5</li>
 *                <li>APLL_QB_ODSEL_DIV_4 - divide by 4</li>
 *                <li>APLL_QB_ODSEL_DIV_2 - divide by 2</li>
 *                <li>APLL_QB_ODSEL_DIV_1 - divide by 1</li></ul>
 *@return APLL output port B divider value
 */
static T_idtApllDividerValue IDTApll_Enum2QbOutputVal(T_idtApllQbDiv qbOutputEnum)
{
	T_idtApllDividerValue qbDiv;

	OS_TRACE(("APLL QB output enum: %d\n", qbOutputEnum));

	switch(qbOutputEnum)
	{
		case APLL_QB_ODSEL_DIV_8:
			qbDiv = 8;
		break;

		case APLL_QB_ODSEL_DIV_5:
			qbDiv = 5;
		break;

		case APLL_QB_ODSEL_DIV_4:
			qbDiv = 4;
		break;

		case APLL_QB_ODSEL_DIV_2:
			qbDiv = 2;
		break;

		case APLL_QB_ODSEL_DIV_1:
			qbDiv = 1;
		break;

		default:
			OS_ERROR(("Invalid QB output divider %d\n", qbOutputEnum));
			qbDiv = 0;
		break;
	}

	return qbDiv;
}

/**
 * @brief Set APLL per output configuration
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance
 * @param output Output port
 * @param apllConfigPerOutput APLL per port configuration
 */
void IDTApll_SetApllConfigPerOutput(
	T_idtDrvHdlr hdlr, 
	T_idtApllId apllInstId, 
	T_idtApllOutput output,
	T_idtApllConfigPerOutput *apllConfigPerOutput)
{
	OS_TRACE(("APLL id: %d, output: %d\n", apllInstId, output));

	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(apllConfigPerOutput);

	/* select xtal type */
	IDTApll_SetXtalId(hdlr, apllInstId, apllConfigPerOutput->xtal);
	/* select input clock */
	IDTApll_SetInputClkSrc(hdlr, apllInstId, apllConfigPerOutput->inputClk);
	/* set pre-divider */
	IDTApll_SetPreDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg, apllConfigPerOutput->preDiv);
	/* set feedback divider */
	IDTApll_SetFeedbackDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg, apllConfigPerOutput->fbDiv);
	/* set frequency doubler */
	IDTApll_SetFreqDoublerSel(hdlr, apllInstId, apllConfigPerOutput->dbleSel);
	/* set output divider */
	if (APLL_OUTPUT_QA == output)
	{
		T_idtApllQaDiv qaDiv;
		qaDiv = IDTApll_QaOutputVal2Enum(apllConfigPerOutput->outputDiv);
		IDTApll_SetQaDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg, qaDiv);
	}
	else
	{
		T_idtApllQaDiv qbDiv;
		qbDiv = IDTApll_QbOutputVal2Enum(apllConfigPerOutput->outputDiv);
		IDTApll_SetQbDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg, qbDiv);
	}
	/* select output type */
	IDTApll_SetOutputSigType(hdlr, apllInstId, output, apllConfigPerOutput->sigType);
	IDTApll_SetOutputEnState(hdlr, apllInstId, output, apllConfigPerOutput->enOutput);
}

/**
 * @brief Get APLL per output configuration
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance
 * @param output Output port
 * @param apllConfigPerOutput APLL per port configuration
 */
void IDTApll_GetApllConfigPerOutput(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllInstId,
	T_idtApllOutput output,
	T_idtApllConfigPerOutput *apllConfigPerOutput)
{
	OS_TRACE(("APLL id: %d, output: %d\n", apllInstId, output));

	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(apllConfigPerOutput);

	apllConfigPerOutput->apllCfg = IDTApll_GetConfigSel(hdlr, apllInstId);
	apllConfigPerOutput->inputClk = IDTApll_GetInputClkSrc(hdlr, apllInstId);
	apllConfigPerOutput->preDiv = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg);
	apllConfigPerOutput->fbDiv = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg);
	if (APLL_OUTPUT_QA == output)
	{
		T_idtApllQaDiv qaDiv;
		qaDiv = IDTApll_GetQaDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg);
		apllConfigPerOutput->outputDiv = IDTApll_Enum2QaOutputVal(qaDiv);
	}
	else if (APLL_OUTPUT_QB == output)
	{
		T_idtApllQbDiv qbDiv;
		qbDiv = IDTApll_GetQbDiv(hdlr, apllInstId, apllConfigPerOutput->apllCfg);
		apllConfigPerOutput->outputDiv = IDTApll_Enum2QbOutputVal(qbDiv);
	}
	apllConfigPerOutput->dbleSel = IDTApll_GetFreqDoublerSel(hdlr, apllInstId);
	apllConfigPerOutput->xtal = IDTApll_GetXtalId(hdlr, apllInstId);
	apllConfigPerOutput->sigType = IDTApll_GetOutputSigType(hdlr, apllInstId, output);
    apllConfigPerOutput->enOutput = IDTApll_GetOutputEnState(hdlr, apllInstId, output);
}

/**
 * @brief Set APLL full configuration
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance
 * @param apllConfig APLL configuration
 */
void IDTApll_SetApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllInstId,
	T_idtApllConfig *apllConfig)
{
	T_idtApllQaDiv qaDiv;
	T_idtApllQbDiv qbDiv;

	OS_TRACE(("APLL id: %d\n", apllInstId));

	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(apllConfig);

	/* Select APLL active configuration set 0 or 1 */
	IDTApll_SetConfigSel(hdlr, apllInstId, apllConfig->apllCfg);
	/* Select APLL input clock source internal vs. external */
	IDTApll_SetInputClkSrc(hdlr, apllInstId, apllConfig->inputClk);
	/* Set APLL pre-dividers for configuration 0 and 1 */
	IDTApll_SetPreDiv(hdlr, apllInstId, APLL_CONFIG0, apllConfig->preDiv.preDivConfig0);
	IDTApll_SetPreDiv(hdlr, apllInstId, APLL_CONFIG1, apllConfig->preDiv.preDivConfig1);
	/* Set APLL feedback dividers for configuration 0 and 1 */
	IDTApll_SetFeedbackDiv(hdlr, apllInstId, APLL_CONFIG0, apllConfig->fbDiv.fbDivConfig0);
	IDTApll_SetFeedbackDiv(hdlr, apllInstId, APLL_CONFIG1, apllConfig->fbDiv.fbDivConfig1);
	/* Set APLL QA output dividers for configuration 0 and 1 */
	qaDiv = IDTApll_QaOutputVal2Enum(apllConfig->outputDiv.qaDiv.outputDivConfig0);
	IDTApll_SetQaDiv(hdlr, apllInstId, APLL_CONFIG0, qaDiv);
	qaDiv = IDTApll_QaOutputVal2Enum(apllConfig->outputDiv.qaDiv.outputDivConfig1);
	IDTApll_SetQaDiv(hdlr, apllInstId, APLL_CONFIG1, qaDiv);
	/* Set APLL QB output dividers for configuration 0 and 1 */
	qbDiv = IDTApll_QbOutputVal2Enum(apllConfig->outputDiv.qbDiv.outputDivConfig0);
	IDTApll_SetQbDiv(hdlr, apllInstId, APLL_CONFIG0, qbDiv);
	qbDiv = IDTApll_QaOutputVal2Enum(apllConfig->outputDiv.qbDiv.outputDivConfig1);
	IDTApll_SetQbDiv(hdlr, apllInstId, APLL_CONFIG1, qbDiv);
	/* Set APLL frequency doubler */
	IDTApll_SetFreqDoublerSel(hdlr, apllInstId, apllConfig->dbleSel);
	/* Set APLL master reset sync state */
	IDTApll_SetMasterResetSync(hdlr, apllInstId, apllConfig->mrSync);
    /* Set APLL bypass */
    IDTApll_SetBypass(hdlr, apllInstId, apllConfig->byPass);
    /* Set APLL shutoff */
    IDTApll_SetShutoff(hdlr, apllInstId, apllConfig->shutOff);
	/* Set APLL external XTAL id */
	IDTApll_SetXtalId(hdlr, apllInstId, apllConfig->xtal);
	/* Set APLL QA output signal type */
	IDTApll_SetOutputSigType(hdlr, apllInstId, APLL_OUTPUT_QA, apllConfig->outputConfig.qaConfig.outputSigType);
	/* Set APLL QA output state */
	IDTApll_SetOutputEnState(hdlr, apllInstId, APLL_OUTPUT_QA, apllConfig->outputConfig.qaConfig.enOutput);
	/* Set APLL QB output signal type */
	IDTApll_SetOutputSigType(hdlr, apllInstId, APLL_OUTPUT_QB, apllConfig->outputConfig.qbConfig.outputSigType);
	/* Set APLL QB output state */
	IDTApll_SetOutputEnState(hdlr, apllInstId, APLL_OUTPUT_QB, apllConfig->outputConfig.qbConfig.enOutput);
}

/**
 * @brief Get APLL full configuration
 * @param hdlr Device driver handler
 * @param apllInstId APLL instance
 * @param apllConfig APLL configuration
 */
void IDTApll_GetApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllInstId,
	T_idtApllConfig *apllConfig)
{
	T_idtApllQaDiv qaDiv;
	T_idtApllQbDiv qbDiv;

	OS_TRACE(("APLL id: %d\n", apllInstId));

	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(apllConfig);

	/* Get APLL active configuration set */
	apllConfig->apllCfg = IDTApll_GetConfigSel(hdlr, apllInstId);
	/* Get APLL input clock source */
	apllConfig->inputClk = IDTApll_GetInputClkSrc(hdlr, apllInstId);
	/* Get APLL pre-dividers for configuration 0 and 1 */
	apllConfig->preDiv.preDivConfig0 = IDTApll_GetPreDiv(hdlr, apllInstId, APLL_CONFIG0);
	apllConfig->preDiv.preDivConfig1 = IDTApll_GetPreDiv(hdlr, apllInstId, APLL_CONFIG1);
	/* Get APLL feedback dividers for configuration 0 and 1 */
	apllConfig->fbDiv.fbDivConfig0 = IDTApll_GetFeedbackDiv(hdlr, apllInstId, APLL_CONFIG0);
	apllConfig->fbDiv.fbDivConfig1 = IDTApll_GetFeedbackDiv(hdlr, apllInstId, APLL_CONFIG1);
	/* Get APLL QA output dividers for configuration 0 and 1 */
	qaDiv = IDTApll_GetQaDiv(hdlr, apllInstId, APLL_CONFIG0);
	apllConfig->outputDiv.qaDiv.outputDivConfig0 = IDTApll_Enum2QaOutputVal(qaDiv);
	qaDiv = IDTApll_GetQaDiv(hdlr, apllInstId, APLL_CONFIG1);
	apllConfig->outputDiv.qaDiv.outputDivConfig1 = IDTApll_Enum2QaOutputVal(qaDiv);
	/* Get APLL QB output dividers for configuration 0 and 1 */
	qbDiv = IDTApll_GetQbDiv(hdlr, apllInstId, APLL_CONFIG0);
	apllConfig->outputDiv.qbDiv.outputDivConfig0 = IDTApll_Enum2QbOutputVal(qbDiv);
	qbDiv = IDTApll_GetQbDiv(hdlr, apllInstId, APLL_CONFIG1);
	apllConfig->outputDiv.qbDiv.outputDivConfig1 = IDTApll_Enum2QbOutputVal(qbDiv);
	/* Get APLL frequency doubler */
	apllConfig->dbleSel = IDTApll_GetFreqDoublerSel(hdlr, apllInstId);
	/* Get APLL master reset sync state */
	apllConfig->mrSync = IDTApll_GetMasterResetSync(hdlr, apllInstId);
    /* Get APLL bypass */
    apllConfig->byPass = IDTApll_GetBypass(hdlr, apllInstId);
    /* Get APLL shutoff */
    apllConfig->shutOff = IDTApll_GetShutoff(hdlr, apllInstId);
	/* Get APLL external XTAL id */
	apllConfig->xtal = IDTApll_GetXtalId(hdlr, apllInstId);
	/* Get APLL QA output signal type */
	apllConfig->outputConfig.qaConfig.outputSigType = IDTApll_GetOutputSigType(hdlr, apllInstId, APLL_OUTPUT_QA);
	/* Get APLL QA output state */
	apllConfig->outputConfig.qaConfig.enOutput = IDTApll_GetOutputEnState(hdlr, apllInstId, APLL_OUTPUT_QA);
	/* Get APLL QB output signal type */
	apllConfig->outputConfig.qbConfig.outputSigType = IDTApll_GetOutputSigType(hdlr, apllInstId, APLL_OUTPUT_QB);
	/* Set APLL QB output state */
	apllConfig->outputConfig.qbConfig.enOutput = IDTApll_GetOutputEnState(hdlr, apllInstId, APLL_OUTPUT_QB);
}

/**
 * @brief Translate APLL input frequency to DPLL output frequency
 * @param apllInputFreq APLL input frequency
 * @return DPLL output frequency
 */
outputFrequency_t IDTApll_ApllInput2DpllOutput(T_idtApllInputFreqType apllInputFreq)
{
	OS_TRACE(("APLL input frequency: %d\n", apllInputFreq));

    switch (apllInputFreq)
    {
        case APLL_INPUT_FREQ_19440KHz:
            return OutFreq_19440k;
        break;

        case APLL_INPUT_FREQ_25000KHz:
	    	return OutFreq_25000k;
		break;

		case APLL_INPUT_FREQ_77760KHz:
		    return OutFreq_77760k;
		break;

		case APLL_INPUT_FREQ_125000KHz:
		    return OutFreq_125000k;
		break;

		case APLL_INPUT_FREQ_155520KHz:
		    return OutFreq_155520k;
		break;

		case APLL_INPUT_FREQ_25781_DOT_25KHz:
		    return OutFreq_25000k_66_64;
		break;

        case APLL_INPUT_FREQ_312500KHz:
            return OutFreq_312500k;
        break;

        case APLL_INPUT_FREQ_128906_DOT_25KHz:
            return OutFreq_125000k_66_64;
        break;

        case APLL_INPUT_FREQ_156250KHz:
            return OutFreq_156250k;
        break;

        case APLL_INPUT_FREQ_8KHz:
            return OutFreq_8k;
        break;

		default:
		    OS_ERROR(("APLL input frequency %d not support by DPLL output\n", apllInputFreq));
		    return OutFreq_Invalid;
		break;
    }
    return OutFreq_Invalid;
}

/**
 * @brief Translate DPLL output frequency to APLL output frequency
 * @param dpllOutput DPLL output frequency
 * @return APLL output frequency
 */
T_idtApllOutputFreqType IDTApll_DpllOutput2ApllOutput(outputFrequency_t dpllOutput)
{
	OS_TRACE(("Enter. DPLL output frequency: %d\n", dpllOutput));

	switch(dpllOutput)
	{
		case OutFreq_24883_DOT_2k:
			return APLL_OUTPUT_FREQ_24883_DOT_2KHz;
		break;
			
		case OutFreq_25000k:
    		return APLL_OUTPUT_FREQ_25000KHz;
		break;

		case OutFreq_25000k_66_64:
    		return APLL_OUTPUT_FREQ_25781_DOT_25KHz;
		break;

		case OutFreq_77760k:
    		return APLL_OUTPUT_FREQ_77760KHz;
		break;

		case OutFreq_78125k:
			return APLL_OUTPUT_FREQ_78125KHz;
		break;

		case OutFreq_78125k_66_64:
			return APLL_OUTPUT_FREQ_80566_DOT_40625KHz;
		break;

		case OutFreq_124416k:
			return APLL_OUTPUT_FREQ_124416KHz;
		break;

		case OutFreq_125000k:
    		return APLL_OUTPUT_FREQ_125000KHz;
		break;
		
		case OutFreq_125000k_66_64:
    		return APLL_OUTPUT_FREQ_128906_DOT_25KHz;
		break;

		case OutFreq_155520k:
			return APLL_OUTPUT_FREQ_155520KHz;
		break;
		
		case OutFreq_156250k:
    		return APLL_OUTPUT_FREQ_156250KHz;
		break;

		case OutFreq_156250k_66_64:
    		return APLL_OUTPUT_FREQ_161132_DOT_8125KHz;
		break;

		case OutFreq_311040k:
    		return APLL_OUTPUT_FREQ_311040KHz;
		break;

		case OutFreq_312500k:
    		return APLL_OUTPUT_FREQ_312500KHz;
		break;

		case OutFreq_312500k_66_64:
    		return APLL_OUTPUT_FREQ_322265_DOT_625KHz;
		break;

		case OutFreq_622080k:
    		return APLL_OUTPUT_FREQ_622080KHz;
		break;

		case OutFreq_625000k:
    		return APLL_OUTPUT_FREQ_625000KHz;
		break;

		case OutFreq_625000k_66_64:
    		return APLL_OUTPUT_FREQ_644531_DOT_25KHz;
		break;

		default:
			OS_ERROR(("Unsupported output frequency: %d\n", dpllOutput));
    		return APLL_OUTPUT_FREQ_MAX;
		break;
	}

	return APLL_OUTPUT_FREQ_MAX;
}

/**
 * @brief Get APLL frequency configuration table entry based on the output frequency
 * @param hdlr Device driver handler
 * @param outputApll Output APLL configuration
 * @param nOutFreq Output frequency
 * @return APLL frequency configuration
 */
const T_idtApllFreqMapping* IDTApll_GetApllFreqTableEntry(
	T_idtDrvHdlr hdlr,
	T_idtOutputApllConfig outputApll,
	outputFrequency_t nOutFreq)
{
    T_idtApllOutput apllOutput = outputApll.apllOutput;
	T_idtApllOutputFreqType apllOutputFreq = IDTApll_DpllOutput2ApllOutput(nOutFreq);
	const T_idtApllFreqMapping *apllFreqConfigList;
	T_osUint8 numberOfVcxo;
	T_osBool vcxoFreqMatch = E_osFalse;
	int i, j;

	OS_TRACE(("APLL output frequency: %d\n", apllOutputFreq));

	OS_ASSERT_PTR_NULL(hdlr);

	if (APLL_OUTPUT_FREQ_MAX == apllOutputFreq)
		return NULL;

	numberOfVcxo = hdlr->numberOfApllXtal_m;
	apllFreqConfigList = IDTApll_GetConfigListByFreq(apllOutputFreq);

	OS_TRACE(("%d apll xtals configured\n", numberOfVcxo));

	for (i = 0; !IDTApll_NullFreqTableEntry((const T_idtApllFreqMapping *)&apllFreqConfigList[i]); i++)
	{	
		/* check if the frequency matches any configured VCXO type, if not, conitinue */
		for (j = 0; j < numberOfVcxo; j++)
		{
			OS_TRACE(("xtal number %d, frequency %d\n", j, hdlr->xtalList[j].apllXtalFreq));

			if (apllFreqConfigList[i].apllXtalFreq == hdlr->xtalList[j].apllXtalFreq)
			{
				OS_TRACE(("Found matching xtal, index: %d, frequency: %d\n", j, hdlr->xtalList[j].apllXtalFreq));
			    break;
			}
		}
		
		if (numberOfVcxo == j)
			/* Didn't find a match VCXO */
			continue;

		vcxoFreqMatch = E_osTrue;

		/* check if the output matches, if not, conitinue */
		if (APLL_OUT_QA_ONLY == apllFreqConfigList[i].apllOutputs)
		{
			if (apllOutput != APLL_OUTPUT_QA)
				continue;
		}
		else if (APLL_OUT_QB_ONLY == apllFreqConfigList[i].apllOutputs)
		{
			if (apllOutput != APLL_OUTPUT_QB)
				continue;
		}

		/* found a good configuration! */
        OS_TRACE(("Found a good apll configuration for output frequency: %d\n", nOutFreq));
		return &apllFreqConfigList[i];
	}

	if (!vcxoFreqMatch)
	{
		OS_LOGGER(1, ("%s: External xtal frequency does not support requested output frequency\n", __FUNCTION__));
	}
	else
	{
		OS_LOGGER(1, ("%s: Requested output frequency not supported on the ouput\n", __FUNCTION__));
	}

	return NULL;
}

/**
 * @brief Get non-active APLL configuration
 * @param hdlr Device driver handler
 * @param apllInst APLL instance
 * @return Non-active APLL configuration
 */
T_idtApllConfigSel IDTApll_GetNonActiveApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllInst)
{
	
	T_idtApllConfigSel apllConfig;

	OS_TRACE(("APLL id: %d\n", apllInst));

	OS_ASSERT_PTR_NULL(hdlr);

	apllConfig = IDTApll_GetConfigSel(hdlr, apllInst);
	if (APLL_CONFIG0 == apllConfig)
		return APLL_CONFIG1;
	else if (APLL_CONFIG1 == apllConfig)
		return APLL_CONFIG0;
	else
		OS_ERROR(("Invalid APLL configration: %d\n", apllConfig));

	return APLL_CONFIG_MAX;
}

/**
 * @brief Switch to non-active APLL configuration
 * @param hdlr Device driver handler
 * @param apllId APLL instance
 */
void IDTApll_Switch2NonActiveApllConfig(
	T_idtDrvHdlr hdlr,
	T_idtApllId  apllId)
{
    T_idtApllConfigSel apllConfig;

    OS_TRACE(("APLL instance: %d\n", apllId));
    OS_ASSERT_PTR_NULL(hdlr);

    apllConfig = IDTApll_GetNonActiveApllConfig(hdlr, apllId);
    IDTApll_SetConfigSel(hdlr, apllId, apllConfig);
}

/**
 * @brief Get APLL configured crystal ID from crystal frequency
 * @param hdlr Device driver handler
 * @param apllInst APLL instance
 * @param apllVcxoFreq Crytal frequency
 * @return Crytal ID
 */
T_idtApllXtalId IDTApll_GetXtalIdFromXtalFreq(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllInst,
	T_idtApllXtalType apllVcxoFreq)
{
	int i;
	T_osUint8 numberOfVcxo = hdlr->numberOfApllXtal_m;
	T_idtApllXtalId	apllVcxoId;
	T_idtApllXtalType tmpApllVcxoFreq;

	OS_TRACE(("APLL id: %d, xtal frequency: %d\n", apllInst, apllVcxoFreq));

	OS_ASSERT_PTR_NULL(hdlr);
	
	for (i = 0; i < numberOfVcxo; i++)
	{
		apllVcxoId = hdlr->xtalList[i].apllXtalId;
		tmpApllVcxoFreq = hdlr->xtalList[i].apllXtalFreq;
		
		if ((tmpApllVcxoFreq == apllVcxoFreq) &&
			IDTApll_ValidApllInstIdXtalIdCombo(apllInst, apllVcxoId))
            {
                OS_TRACE(("Found a configured xtal, id: %d, frequency: %d\n",
                            apllVcxoId, apllVcxoFreq));
				return apllVcxoId;
            }
	}

	return APLL_XTAL_MAX;
}

/**
 * @brief Checks if there is any XTAL confiugred for the APLL
 * @param hdlr Device driver handler
 * @param apllId APLL instance
 * @return TRUE, if there is at least one XTAL configured; FALSE, otherwise
 */
T_osBool IDTApll_XtalConfigured(
	T_idtDrvHdlr hdlr,
	T_idtApllId apllId)
{
    T_osUint8 i;

    OS_TRACE(("APLL id: %d\n", apllId));

    OS_ASSERT_PTR_NULL(hdlr);

    for (i=0; i<IDT_DRV_MAX_APLL_XTAL; i++)
    {
        if (APLL_INST_1 == apllId)
        {
            if (((APLL_XTAL_1_APLL1 == hdlr->xtalList[i].apllXtalId)||
                (APLL_XTAL_3_APLL1 == hdlr->xtalList[i].apllXtalId))&&
                (hdlr->xtalList[i].apllXtalFreq != APLL_XTAL_FREQ_MAX))
                    return E_osTrue;
        }
        else if (APLL_INST_2 == apllId)
        {
            if (((APLL_XTAL_2_APLL2 == hdlr->xtalList[i].apllXtalId)||
                (APLL_XTAL_4_APLL2 == hdlr->xtalList[i].apllXtalId)) &&
                (hdlr->xtalList[i].apllXtalFreq != APLL_XTAL_FREQ_MAX))
                    return E_osTrue;
        }
        else
        {
            OS_LOGGER(1, ("%s: Invalid APLL id: %d\n", __FUNCTION__, apllId));
            return E_osFalse;
        }
    }

    OS_LOGGER(1, ("%s: No XTAL configured for APLL %d\n", __FUNCTION__, apllId));
    return E_osFalse;
}

/**
 * @brief Checks if there is input XTAL information is valid for the device
 * @param deviceConfig APLL device configuration
 * @param xtalList List of XTAL information for the device
 * @param numberOfXtal Number of XTAL in the xtalList
 * @return TRUE, if input XTAL information is valid; FALSE, otherwise
 */
T_osBool IDTApll_ValidXtalInput(
    const T_idtDrvDeviceConfig *deviceConfig,
    T_idtApllXtal *xtalList,
    T_osUint8 numberOfXtal)
{
    int perDeviceXtalLimit, i;

    OS_TRACE(("xtalList: %p, numberOfXtal: %d\n", xtalList, numberOfXtal));
    OS_ASSERT_PTR_NULL(deviceConfig);

    perDeviceXtalLimit = deviceConfig->numberOfApll_m *
                            deviceConfig->maxNumXtalPerApll_m;
 
	if (numberOfXtal > perDeviceXtalLimit)
    {
        OS_LOGGER(1, ("%s: numberOfXtal(%d) exceeds XTAL limit (%d)\n",
                        __FUNCTION__, numberOfXtal, perDeviceXtalLimit));
        return E_osFalse;
    }

    if ((numberOfXtal > 0) && (NULL == xtalList))
    {
        OS_LOGGER(1, ("%s: NULL xtal list\n", __FUNCTION__));
        return E_osFalse;
    }

    for (i=0; i<numberOfXtal; i++)
    {
        if (xtalList[i].apllXtalId >= APLL_XTAL_MAX || xtalList[i].apllXtalId < APLL_XTAL_1_APLL1)
        {
            OS_LOGGER(1, ("%s: Invalid xtal id: %d\n", __FUNCTION__, xtalList[i].apllXtalId));
            return E_osFalse;
        }

        if (xtalList[i].apllXtalFreq >= APLL_XTAL_FREQ_MAX || xtalList[i].apllXtalFreq < APLL_XTAL_FREQ_25000KHz)
        {
            OS_LOGGER(1, ("%s: Invalid xtal frequency: %d\n", __FUNCTION__, xtalList[i].apllXtalFreq));
            return E_osFalse;
        }

        if (!(deviceConfig->supportedXtalId_ma[xtalList[i].apllXtalId]))
        {
            OS_LOGGER(1, ("%s: Input xtal id %d not supported\n", __FUNCTION__, xtalList[i].apllXtalId));
            return E_osFalse;
        }

        if (!(deviceConfig->supportedXtalFreq_ma[xtalList[i].apllXtalId][xtalList[i].apllXtalFreq]))
        {
            OS_LOGGER(1, ("%s: Input xtal frequency %d not supported on xtal %d\n",
                            __FUNCTION__, xtalList[i].apllXtalFreq, xtalList[i].apllXtalId));
            return E_osFalse;
        }
    }

    return E_osTrue;
}

/**
 * @brief Return supported xtal configuration by the device
 * @param deviceConfig APLL device configuration
 * @param numberOfXtal Number of xtal configuration in the returned list
 * @return The list of suppoted XTAL configuration
 *
 * Caller needs to free the list after finishing using it.
 */
T_idtApllXtal* IDTApll_GetSupportedXtal(
    const T_idtDrvDeviceConfig *deviceConfig,
    T_osUint8 *numberOfXtal)
{
    T_osInt8 i, j;
    T_osUint8 xtalListLength = 0;
    T_idtApllXtal* xtalList = NULL;

    OS_ASSERT_PTR_NULL(deviceConfig);
    OS_ASSERT_PTR_NULL(numberOfXtal);

    for (i=0; i<APLL_XTAL_MAX; i++)
    {
        if (!(deviceConfig->supportedXtalId_ma[i]))
            continue;

        for (j=0; j<APLL_XTAL_FREQ_MAX; j++)
        {
            if (!(deviceConfig->supportedXtalFreq_ma[j]))
                continue;

            xtalListLength++;
            xtalList = realloc(xtalList, xtalListLength*sizeof(T_idtApllXtal));

            if (!xtalList)
            {
                OS_LOGGER(1, ("%s: Fail to allocate memory for xtal configuration list\n", __FUNCTION__));
                *numberOfXtal = 0;
                return NULL;
            }

            xtalList[xtalListLength-1].apllXtalId = (T_idtApllXtalId)i;
            xtalList[xtalListLength-1].apllXtalFreq = (T_idtApllXtalType)j;
        }
    }

    *numberOfXtal = xtalListLength;
    return xtalList;
}

