/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "osNumeric.h"

#include "Define.h"
#include "Hal.h"
#include "DpllCnfg.h"
#include "Dpll.h"
#include "General.h"
#include "Output.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * @brief   Get ID number of the device
 * @param hdlr Device driver handler
 * @return	16-bit ID number
 */
T_osUint16 IDTGeneral_GetDeviceID(T_idtDrvHdlr hdlr)
{
	T_osUint16 ID;
	T_osUint8 nIntInd;

	OS_LOGGER(5, ("%s dev/%s\n", __FUNCTION__, hdlr->name_m));
	OS_ASSERT_PTR_NULL(hdlr);

	nIntInd = IDTHal_Read(hdlr, REG_ID_LOW);
	ID = IDTHal_Read(hdlr, REG_ID_HIGH);
	ID = (ID << 8) | nIntInd;
	return ID;
}

/**
 * @brief   Set compensation to the oscillator offset
 * @param hdlr Device driver handler
 * @param pptOffset Crystal oscillator frequency offset (in parts per
 * trillion)
 */
void IDTGeneral_SetOscFreqOffset(T_idtDrvHdlr hdlr, long pptOffset)
{

	T_osUint32 registerOffsetVal;

	OS_LOGGER(5,
			("%s dev/%s pptOffset/%ld\n", __FUNCTION__, hdlr->name_m, pptOffset));
	OS_ASSERT_PTR_NULL(hdlr);

	/* Translate ppt to register value
	 *
	 * register value is in 24 bit 2's compliment */
	if (pptOffset < 0)
	{
		/* Negative values */
		registerOffsetVal = NOMINAL_PPT_TO_2COMPL(-pptOffset);
		registerOffsetVal = (~(registerOffsetVal) + 1) & 0x00FFFFFF;
	}
	else
	{
		/* Positive values */
		registerOffsetVal = NOMINAL_PPT_TO_2COMPL(pptOffset);
	}

	/* IDTHal_Write oscillator offset to registers */
	IDTHal_Write(hdlr, REG_NOMINAL_FREQ_LOW_CNFG,
			(T_osUint8) registerOffsetVal);
	IDTHal_Write(hdlr, REG_NOMINAL_FREQ_MID_CNFG,
			(T_osUint8) (registerOffsetVal >> 8));
	IDTHal_Write(hdlr, REG_NOMINAL_FREQ_HIGH_CNFG,
			(T_osUint8) (registerOffsetVal >> 16));
}

/**
 * @brief Get compensated oscillator offset
 * @param hdlr Device driver handler
 * @return Crystal oscillator frequency offset (in parts per
 * trillion)
 */
long IDTGeneral_GetOscFreqOffset(T_idtDrvHdlr hdlr)
{

	T_osUint32 registerOffsetVal;

	OS_ASSERT_PTR_NULL(hdlr);

	/* Read oscillator offset from registers */
	registerOffsetVal = (T_osUint32) IDTHal_Read(hdlr,
			REG_NOMINAL_FREQ_LOW_CNFG);
	registerOffsetVal |= (T_osUint32) (IDTHal_Read(hdlr,
			REG_NOMINAL_FREQ_MID_CNFG) << BF_NOMINAL_FREQ_VALUE_A_SIZE);
	registerOffsetVal |= (T_osUint32) (IDTHal_Read(hdlr,
			REG_NOMINAL_FREQ_HIGH_CNFG)
			<< (BF_NOMINAL_FREQ_VALUE_B_SIZE + BF_NOMINAL_FREQ_VALUE_A_SIZE));

	/* Convert from 24 bit 2's compliment to standard long */
	if (registerOffsetVal & 0x00800000)
	{
		/* Negative value */
		return -NOMINAL_2COMPL_TO_PPT((~(registerOffsetVal - 1)&0xFFFFFF));
	}
	else
	{
		/* Positive value - no changes required */
		return NOMINAL_2COMPL_TO_PPT(registerOffsetVal);
	}
}

/**
 * @brief   Set the network type, SDH or SONET
 * @param hdlr Device driver handler
 * @param	  nType <ul><li>Network_SDH = SDH network</li>
 *			 <li>Network_SONET = SONET network</li></ul>
 */
void IDTGeneral_SetNetworkType(T_idtDrvHdlr hdlr, networkType_t nType)
{

	OS_LOGGER(5, ("%s dev/%s nType/%u\n", __FUNCTION__, hdlr->name_m, nType));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHAL_WriteBitfield(hdlr, REG_INPUT_MODE_CNFG, BF_IN_SONET_SDH_MASK,
			BF_IN_SONET_SDH_LSB, nType);

    /* If high level API is initialized then cache value */
    if (hdlr->currOutPathCfg_mp)
        hdlr->currOutPathCfg_mp->networkType_m = nType;
}

/**
 * @brief   Get the network type, SDH or SONET
 * @param hdlr Device driver handler
 * @return	  nType <ul><li>Network_SDH = SDH network</li>
 *			 <li>Network_SONET = SONET network</li></ul>
 */
networkType_t IDTGeneral_GetNetworkType(T_idtDrvHdlr hdlr)
{
	OS_ASSERT_PTR_NULL(hdlr);

	return ((networkType_t)IDTHAL_ReadBitfield(hdlr, REG_INPUT_MODE_CNFG, BF_IN_SONET_SDH_MASK,
			BF_IN_SONET_SDH_LSB));
}

/**
 * @brief   Set the device running in Revertive mode or not
 * @param hdlr Device driver handler
 * @param	nEnable <ul><li>0= Works in Non-Revertive mode</li>
 *			 <li>1= Works in Revertive mode</li></ul>
 */
void IDTGeneral_SetRevertiveMode(T_idtDrvHdlr hdlr, T_osUint8 nEnable)
{

	OS_LOGGER(5,
			("%s dev/%s nEnable/%u\n", __FUNCTION__, hdlr->name_m, nEnable));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHAL_WriteBitfield(hdlr, REG_INPUT_MODE_CNFG, BF_REVERTIVE_MODE_MASK,
			BF_REVERTIVE_MODE_LSB, nEnable);
}

/**
 * @brief   Get the device running in Revertive mode or not
 * @param hdlr Device driver handler
 * @return	nEnable <ul><li>0= Works in Non-Revertive mode</li>
 *			 <li>1= Works in Revertive mode</li></ul>
 */
T_osUint8 IDTGeneral_GetRevertiveMode(T_idtDrvHdlr hdlr)
{
	OS_ASSERT_PTR_NULL(hdlr);

	return IDTHAL_ReadBitfield(hdlr, REG_INPUT_MODE_CNFG, BF_REVERTIVE_MODE_MASK,
			BF_REVERTIVE_MODE_LSB);
}

/**
 * @brief   Set the value of time out of phase alarm
 *		   Timeout(second)= nMultiFactor * nTimeout
 * @param hdlr Device driver handler
 * @param nMultiFactor <ul><li>phaseTimeoutMultiplier_2 - Multiple 2</li>
 *				           <li>phaseTimeoutMultiplier_4 - Multiple 4</li>
 *				           <li>phaseTimeoutMultiplier_8 - Multiple 8</li>
 *				           <li>phaseTimeoutMultiplier_16 - Multiple 16</li></ul>
 * @param nTimeout  a 6-bit unsigned integer, unit is second;
 */
void IDTGeneral_SetTimeoutValue(T_idtDrvHdlr hdlr, phaseTimeoutMultiplier_t nMultiFactor,
		T_osUint8 nTimeout)
{

	T_osUint8 nIntInd = IDTHal_Read(hdlr, REG_PHASE_ALARM_TIME_OUT_CNFG);

	OS_LOGGER(5,
			("%s dev/%s nMultiFactor/%u nTimeout/%u\n", __FUNCTION__, hdlr->name_m, nMultiFactor, nTimeout));
	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT(nMultiFactor<phaseTimeoutMultiplier_MAX);
	OS_ASSERT(nTimeout<(1<<BF_TIMEOUT_VALUE_SIZE));

	IDTHAL_ModifyBitfield(nIntInd, BF_MULTI_FACTOR_MASK, BF_MULTI_FACTOR_LSB,
			nMultiFactor);

	IDTHAL_ModifyBitfield(nIntInd, BF_TIMEOUT_VALUE_MASK, BF_TIMEOUT_VALUE_LSB,
			nTimeout);

	IDTHal_Write(hdlr, REG_PHASE_ALARM_TIME_OUT_CNFG, nIntInd);
}

/**
 * @brief Get the value of time out of phase alarm
 *		   Timeout(second)= nMultiFactor * nTimeout
 * @param hdlr Device driver handler
 * @param nMultiFactor_p <ul><li>phaseTimeoutMultiplier_2 - Multiple 2</li>
 *				           <li>phaseTimeoutMultiplier_4 - Multiple 4</li>
 *				           <li>phaseTimeoutMultiplier_8 - Multiple 8</li>
 *				           <li>phaseTimeoutMultiplier_16 - Multiple 16</li></ul>
 * @param nTimeout_p  a 6-bit unsigned integer, unit is second;
 */
void IDTGeneral_GetTimeoutValue(T_idtDrvHdlr hdlr, phaseTimeoutMultiplier_t *nMultiFactor_p,
                     T_osUint8 *nTimeout_p)
{
        OS_ASSERT_PTR_NULL(hdlr);
        OS_ASSERT_PTR_NULL(nMultiFactor_p);
        OS_ASSERT_PTR_NULL(nTimeout_p);

        *nMultiFactor_p = IDTHAL_ReadBitfield(hdlr, REG_PHASE_ALARM_TIME_OUT_CNFG, BF_MULTI_FACTOR_MASK, BF_MULTI_FACTOR_LSB);

        *nTimeout_p = IDTHAL_ReadBitfield(hdlr, REG_PHASE_ALARM_TIME_OUT_CNFG, BF_TIMEOUT_VALUE_MASK, BF_TIMEOUT_VALUE_LSB);
}

/**
 * @brief   Set the activate edge of main oscillator
 * @param hdlr Device driver handler
 * @param	  nEdge <ul><li>0= Rising edge is activate edge</li>
 *		                <li>1= Falling edge is activate edge</li></ul>
 */
void IDTGeneral_SetOscActiveEdge(T_idtDrvHdlr hdlr, T_osUint8 nEdge)
{

	OS_LOGGER(5, ("%s dev/%s nEdge/%u\n", __FUNCTION__, hdlr->name_m, nEdge));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHAL_WriteBitfield(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG, BF_OSC_EDGE_MASK,
			BF_OSC_EDGE_LSB, nEdge);
}

/**
 * @brief Get the activate edge of main oscillator
 * @param hdlr Device driver handler
 * @return <ul><li>0= Rising edge is activate edge</li>
 *		                <li>1= Falling edge is activate edge</li></ul>
 */
T_osUint8 IDTGeneral_GetOscActiveEdge(T_idtDrvHdlr hdlr)
{
	OS_ASSERT_PTR_NULL(hdlr);

	return IDTHAL_ReadBitfield(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG, BF_OSC_EDGE_MASK,
			BF_OSC_EDGE_LSB);
}

/**
 * @brief Enable protected register access mode
 * @param hdlr Device driver handler
 */
void IDTGeneral_SetProtectMode(T_idtDrvHdlr hdlr)
{

	OS_LOGGER(5, ("%s dev/%s\n", __FUNCTION__, hdlr->name_m));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHal_Write(hdlr, REG_PROTECTION_CNFG, 0x00);
}

/**
 * @brief Set single access register access mode
 * @param hdlr Device driver handler
 */
void IDTGeneral_SetSingleUnprotectMode(T_idtDrvHdlr hdlr)
{

	OS_LOGGER(5, ("%s dev/%s\n", __FUNCTION__, hdlr->name_m));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHal_Write(hdlr, REG_PROTECTION_CNFG, 0x86);
}

/**
 * @brief Set fully un-protected register access mode
 * @param hdlr Device driver handler
 */
void IDTGeneral_SetFullyUnprotectMode(T_idtDrvHdlr hdlr)
{
	OS_LOGGER(5, ("%s dev/%s\n", __FUNCTION__, hdlr->name_m));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHal_Write(hdlr, REG_PROTECTION_CNFG, 0x85);
}

/**
 * @brief   Set TDO pin to indicate the fail of selected clock
 * @param hdlr Device driver handler
 * @param	nEnable <ul><li>0= Not enable to use TDO pin</li>
 *		                <li>1= enable to use the pin;</li></ul>
 */
void IDTGeneral_SetFlagOnTDO(T_idtDrvHdlr hdlr, T_osUint8 nEnable)
{
	OS_LOGGER(5,
			("%s dev/%s nEnable/%u\n", __FUNCTION__, hdlr->name_m, nEnable));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHAL_WriteBitfield(hdlr, REG_MON_SW_HS_CNFG, BF_LOS_FLAG_TO_TDO_MASK,
			BF_LOS_FLAG_TO_TDO_LSB, nEnable);
}

/**
 * @brief Get TDO pin to indicate the fail of selected clock
 * @param hdlr Device driver handler
 * @return <ul><li>0= Not enable to use TDO pin</li>
 *		       <li>1= enable to use the pin;</li></ul>
 */
T_osUint8 IDTGeneral_GetFlagOnTDO(T_idtDrvHdlr hdlr)
{
	OS_ASSERT_PTR_NULL(hdlr);

	return IDTHAL_ReadBitfield(hdlr, REG_MON_SW_HS_CNFG, BF_LOS_FLAG_TO_TDO_MASK,
			BF_LOS_FLAG_TO_TDO_LSB);
}

/**
 * @brief Set Ultra-fast-switch enable or not
 * @param hdlr Device driver handler
 * @param	nEnable <ul><li>0= The feature is disabled</li>
 *                      <li>1= The feature is enabled</li></ul>
 */
void IDTGeneral_SetUltraFastSwitch(T_idtDrvHdlr hdlr, T_osUint8 nEnable)
{
	OS_LOGGER(5,
			("%s dev/%s nEnable/%u\n", __FUNCTION__, hdlr->name_m, nEnable));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHAL_WriteBitfield(hdlr, REG_MON_SW_HS_CNFG, BF_ULTR_FAST_SW_MASK,
			BF_ULTR_FAST_SW_LSB, nEnable);
}

/**
 * @brief Get Ultra-fast-switch enable or not
 * @param hdlr Device driver handler
 * @return <ul><li>0= The feature is disabled</li>
 *             <li>1= The feature is enabled</li></ul>
 */
T_osUint8 IDTGeneral_GetUltraFastSwitch(T_idtDrvHdlr hdlr)
{
	OS_ASSERT_PTR_NULL(hdlr);

	return IDTHAL_ReadBitfield(hdlr, REG_MON_SW_HS_CNFG, BF_ULTR_FAST_SW_MASK,
			BF_ULTR_FAST_SW_LSB);
}

/**
 * @brief   Set Hard-alarm enable when the input exceeds the threshold
 * @param hdlr Device driver handler
 * @param	nEnable <ul><li>0= Disable Hard-alarm feature</li>
 *                      <li>1= Enable the feature</li></ul>
 */
void IDTGeneral_SetHardAlarm(T_idtDrvHdlr hdlr, T_osUint8 nEnable)
{
	OS_LOGGER(5,
			("%s dev/%s nEnable/%u\n", __FUNCTION__, hdlr->name_m, nEnable));
	OS_ASSERT_PTR_NULL(hdlr);

	IDTHAL_WriteBitfield(hdlr, REG_MON_SW_HS_CNFG, BF_FREQ_MON_HARD_EN_MASK,
			BF_FREQ_MON_HARD_EN_LSB, nEnable);
}

/**
 * @brief Get Hard-alarm enable when the input exceeds the threshold
 * @param hdlr Device driver handler
 * @return <ul><li>0= Disable Hard-alarm feature</li>
 *             <li>1= Enable the feature</li></ul>
 */
T_osUint8 IDTGeneral_GetHardAlarm(T_idtDrvHdlr hdlr)
{
	OS_ASSERT_PTR_NULL(hdlr);

	return IDTHAL_ReadBitfield(hdlr, REG_MON_SW_HS_CNFG, BF_FREQ_MON_HARD_EN_MASK,
			BF_FREQ_MON_HARD_EN_LSB);
}

/**
 * @brief I2C address translation function used internally by the driver
 * @param i2cAddr I2C slave address
 * @param transData Translation information
 * @return Translated address
 */
T_osUint8 IDTGeneral_i2cTranslate(T_osUint8 i2cAddr, void* transData)
{
	T_osUint8 bitLocation;
	T_osUint8 bitValue;

	OS_ASSERT_PTR_NULL(transData);

	bitLocation = ((T_idtI2CtransData *)transData)->bitLocation;
	bitValue = ((T_idtI2CtransData *)transData)->bitValue;

	OS_ASSERTS(bitLocation <= 6,
                ("Invalid bit location %d for 7-bit i2c slave address\n", 
                 bitLocation));
	OS_ASSERTS((bitValue == 0 || bitValue == 1),
                ("Invalid bit value %d\n", bitValue));

	if (bitValue)
		return i2cAddr|(1<<bitLocation);
	else
		return i2cAddr&(~(1<<bitLocation));
}

/**
 * @internal
 * @brief Initialize device, common.
 * @param Device driver handler
 */
void IDTGeneral_InitDeviceCommon(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    OS_LOGGER(5, ("%s dev/%s\n", __FUNCTION__, hdlr->name_m));

    IDTHal_Write(hdlr, 0x2D, 0x01);

    IDTHal_Write(hdlr, 0x35, 0xF0);
    IDTHal_Write(hdlr, 0x36, 0xF4);
    IDTHal_Write(hdlr, 0x37, 0x04);

    IDTHal_Write(hdlr, 0x38, 0x51);
    IDTHal_Write(hdlr, 0x39, 0x00);

    IDTHal_Write(hdlr, 0x3A, 0x10);
    IDTHal_Write(hdlr, 0x3B, 0x00);

    IDTHal_Write(hdlr, 0x3D, 0x21);

    IDTHal_Write(hdlr, 0x3F, 0xF0);
    IDTHal_Write(hdlr, 0x40, 0xF4);
    IDTHal_Write(hdlr, 0x41, 0x04);

    IDTHal_Write(hdlr, 0x42, 0x51);
    IDTHal_Write(hdlr, 0x43, 0x00);

    IDTHal_Write(hdlr, 0x44, 0x10);
    IDTHal_Write(hdlr, 0x45, 0x00);

    IDTHal_Write(hdlr, 0x47, 0x21);

    IDTHal_Write(hdlr, 0x2D, 0x00);
}

/**
 * @internal 
 * @brief Initialize device driver, common.
 *
 * This function creates a driver handler used as reference to a
 * specific device. It does not perform any device access.
 *
 * The isHighLevelApi determines whether device driver initializes
 * high level APIs. 
 *
 * @param name Name of device (ex. "myDev")
 * @param drvAccess Device access data. \see drvAccess_t structure
 * information
 * @param deviceType What supported device
 * @param deviceConfig Device configuration information
 * @param useHighLevelApi 0-Don't use high level API, 1-high level API enabled.
 * @return Device driver handler (used in all APIs)
 */
T_idtDrvHdlr IDTGeneral_InitDriverCommon(const char      *name, 
                                         T_idtDrvAccess  drvAccess,
                                         T_idtDeviceType deviceType,
                                         const T_idtDrvDeviceConfig *deviceConfig,
                                         int             useHighLevelApi)
{
	T_idtDrvHdlr handler;

	OS_LOGGER(5, ("%s name/%s\n", __FUNCTION__, name));
	OS_ASSERT_PTR_NULL(name);
    OS_ASSERT(deviceType<IDT_DEVICE_MAX);
    OS_ASSERT_PTR_NULL(drvAccess.readFunc_m);
    OS_ASSERT_PTR_NULL(drvAccess.writeFunc_m);

	/* Allocate and build device driver handler */
	handler = (T_idtDrvHdlr) OS_MALLOC(sizeof(struct drvHdlr_s));
	OS_ASSERT_PTR_NULL(handler);
	handler->name_m                   = name;
    handler->deviceType_m             = deviceType;
	handler->drvAccess_m.initFunc_m   = drvAccess.initFunc_m;
	handler->drvAccess_m.deinitFunc_m = drvAccess.deinitFunc_m;
	handler->drvAccess_m.readFunc_m   = drvAccess.readFunc_m;
	handler->drvAccess_m.writeFunc_m  = drvAccess.writeFunc_m;
	handler->drvAccess_m.userData_m   = drvAccess.userData_m;
	handler->deviceConfig_m           = deviceConfig;

	/* Check if this driver supports this device. 
	 *
	 * Singleton - supports only one device 
	 */
    OS_ASSERTS(handler->deviceConfig_m,
                ("Driver does not support device type %d\n",
                 deviceType));

    /* If we're going to use high level API - we need to initialize
     * it's data */
    if (useHighLevelApi)
    {
            /* Allocate & initialize */
            handler->currOutPathCfg_mp = (T_IDTPathConfiguration *)
                    OS_MALLOC(sizeof(struct S_IDTPathConfiguration));
            OS_ASSERT_PTR_NULL(handler->currOutPathCfg_mp);
            memcpy(handler->currOutPathCfg_mp,
                   &IDTDpll_pathConfigurationNULL_c,
                   sizeof(T_IDTPathConfiguration));
    }

    /* Initialize HAL interface */
    if (handler->drvAccess_m.initFunc_m)
       handler->drvAccess_m.initFunc_m(handler->drvAccess_m.userData_m);

	return handler;
}

/**
 * @brief De-initialize driver handler.
 *
 * De-allocates handler. Handler is invalid after calling this function
 * @param hdlr Device driver handler
 */
void IDTGeneral_DeInitDriver(T_idtDrvHdlr hdlr)
{
	OS_LOGGER(5, ("%s dev/%s\n", __FUNCTION__, hdlr->name_m));
	OS_ASSERT_PTR_NULL(hdlr);

	if (hdlr)
	{
        /* Free up data used by high level API */
        if (hdlr->currOutPathCfg_mp)
                OS_FREE(hdlr->currOutPathCfg_mp);

        /* De-initialize HAL interface */
        if (hdlr->drvAccess_m.deinitFunc_m)
           hdlr->drvAccess_m.deinitFunc_m(hdlr->drvAccess_m.userData_m);

        /* Clear structure / Free memory */
		hdlr->name_m = NULL;
		OS_FREE(hdlr);
	}
	hdlr = NULL;
}

