/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "osNumeric.h"

#include "Define.h"
#include "Hal.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
#define DEBUG_IDT_HAL

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

#if (!IDT_HAL_USE_MACRO)
/**
 * @brief Define function to write a bitfield to a device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @param data Bit field data to write
 */
void IDTHAL_WriteBitfield(T_idtDrvHdlr hdlr,
                          T_osUint8    offset,
                          T_osUint8    mask,
                          T_osUint8    lsb,
                          T_osUint8    data)
{
  T_osUint8 regVal;

  OS_ASSERT_PTR_NULL(hdlr);
  OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.writeFunc_m);

  regVal = IDTHal_Read(hdlr,offset);
  IDTHAL_ModifyBitfield(regVal,mask,lsb,data);
  IDTHal_Write(hdlr,offset,regVal);
}

/**
 * @brief Define function to write a bitfield to a device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @param data Bit field data to write
 */
void IDTHAL_WriteBitfield_Ext(T_idtDrvHdlr hdlr,
                              T_osUint8    offset,
                              T_osUint8    mask,
                              T_osUint8    lsb,
                              T_osUint8    data)
{
  T_osUint8 regVal;

  OS_ASSERT_PTR_NULL(hdlr);
  OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.writeFunc_m);

  regVal = IDTHal_Read_Ext(hdlr,offset);
  IDTHAL_ModifyBitfield(regVal,mask,lsb,data);
  IDTHal_Write_Ext(hdlr,offset,regVal);
}

/**
 * @brief Define function to read a bitfield from device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @return Bit field value
 */
T_osUint8 IDTHAL_ReadBitfield(T_idtDrvHdlr hdlr,
                              T_osUint8    offset,
                              T_osUint8    mask,
                              T_osUint8    lsb)
{
	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.readFunc_m);

    return ((T_osUint8)IDTHAL_GetBitfield(IDTHal_Read(
                                              hdlr,
                                              offset),
                                          (mask),
                                          (lsb)));
}

/**
 * @brief Define function to read a bitfield from device
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param mask Bitfield mask
 * @param lsb Least significant bit
 * @return Bit field value
 */
T_osUint8 IDTHAL_ReadBitfield_Ext(T_idtDrvHdlr hdlr,
                                  T_osUint8    offset,
                                  T_osUint8    mask,
                                  T_osUint8    lsb)
{
	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.readFunc_m);

    return ((T_osUint8)IDTHAL_GetBitfield(IDTHal_Read_Ext(
                                              hdlr,
                                              offset),
                                          (mask),
                                          (lsb)));
}
#endif

/**
 * @brief Function to read from device address space
 * @param hdlr Device driver handler
 * @param offset Register address offset
 */
T_osUint8 IDTHal_Read(T_idtDrvHdlr hdlr, T_osUint8 offset)
{
	T_osUint8 i2cAddr;
	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.readFunc_m);

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address configured: 0x%x\n", __FUNCTION__, hdlr->i2cAddr));
#endif

	if (hdlr->i2cAddrTransFunc_m)
		i2cAddr = hdlr->i2cAddrTransFunc_m(hdlr->i2cAddr, hdlr->dpllI2CTransData_mp);
	else
		i2cAddr = hdlr->i2cAddr;

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address translated: 0x%x, register: 0x%x\n",
	               __FUNCTION__, i2cAddr, offset));
#endif

	return hdlr->drvAccess_m.readFunc_m(hdlr->drvAccess_m.userData_m, i2cAddr, offset);
}

/**
 * @brief Function to write to device address space
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param data Data to write
 */
void IDTHal_Write(T_idtDrvHdlr hdlr, T_osUint8 offset, T_osUint8 data)
{
	T_osUint8 i2cAddr;
	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.writeFunc_m);

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address configured: 0x%x\n", __FUNCTION__, hdlr->i2cAddr));
#endif

	if (hdlr->i2cAddrTransFunc_m)
		i2cAddr = hdlr->i2cAddrTransFunc_m(hdlr->i2cAddr, hdlr->dpllI2CTransData_mp);
	else
		i2cAddr = hdlr->i2cAddr;

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address translated: 0x%x, register: 0x%x, value: 0x%x\n",
	               __FUNCTION__, i2cAddr, offset, data));
#endif

	hdlr->drvAccess_m.writeFunc_m(hdlr->drvAccess_m.userData_m, i2cAddr, offset, data);
}

/**
 * @brief Function to read from device apll address space
 * @param hdlr Device driver handler
 * @param offset Register address offset
 */
T_osUint8 IDTHal_Read_Ext(T_idtDrvHdlr hdlr, T_osUint8 offset)
{
	T_osUint8 i2cAddr;
	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.readFunc_m);

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address configured: 0x%x\n", __FUNCTION__, hdlr->i2cAddr));
#endif

	if (hdlr->i2cAddrTransFunc_m)
		i2cAddr = hdlr->i2cAddrTransFunc_m(hdlr->i2cAddr, hdlr->apllI2CTransData_mp);
	else
		i2cAddr = hdlr->i2cAddr;

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address translated: 0x%x, register: 0x%x\n",
	               __FUNCTION__, i2cAddr, offset));
#endif

	return hdlr->drvAccess_m.readFunc_m(hdlr->drvAccess_m.userData_m, i2cAddr, offset);
}

/**
 * @brief Function to write to device apll address space
 * @param hdlr Device driver handler
 * @param offset Register address offset
 * @param data Data to write
 */
void IDTHal_Write_Ext(T_idtDrvHdlr hdlr, T_osUint8 offset, T_osUint8 data)
{
	T_osUint8 i2cAddr;
	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT_PTR_NULL(hdlr->drvAccess_m.writeFunc_m);

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address configured: 0x%x\n", __FUNCTION__, hdlr->i2cAddr));
#endif

	if (hdlr->i2cAddrTransFunc_m)
		i2cAddr = hdlr->i2cAddrTransFunc_m(hdlr->i2cAddr, hdlr->apllI2CTransData_mp);
	else
		i2cAddr = hdlr->i2cAddr;

#ifdef DEBUG_IDT_HAL
	OS_LOGGER(9, ("%s: i2c address translated: 0x%x, register: 0x%x, value: 0x%x\n",
	               __FUNCTION__, i2cAddr, offset, data));
#endif

	hdlr->drvAccess_m.writeFunc_m(hdlr->drvAccess_m.userData_m, i2cAddr, offset, data);
}
