/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "osNumeric.h"
#include "osAdapt.h"

#include "Define.h"
#include "Hal.h"
#include "General.h"
#include "Input.h"
#include "Dpll.h"
#include "RegisterDef.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
/** @brief Structure used to store leaky bucket register offsets */
typedef struct S_bucketReg
{
    T_osUint8 upperThreshReg_m; /**< Upper threshold register offset */
    T_osUint8 lowerThreshReg_m; /**< Lower threshold register offset */
    T_osUint8 bucketSizeReg_m; /**< Bucket size register offset */
    T_osUint8 decayRateReg_m; /**< Decay rate register offset */
} bucketReg_t;

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
static const T_osUint8 IDTInput_inputPriorityRegArray[] =
{ REG_IN1_IN2_SEL_PRIORITY_CNFG, REG_IN3_IN4_SEL_PRIORITY_CNFG,
  REG_IN5_IN6_SEL_PRIORITY_CNFG, REG_IN7_IN8_SEL_PRIORITY_CNFG,
  REG_IN9_IN10_SEL_PRIORITY_CNFG, REG_IN11_IN12_SEL_PRIORITY_CNFG,
  REG_IN13_IN14_SEL_PRIORITY_CNFG };

static const T_osUint8 IDTInput_inputFredRegArray[] =
{ REG_IN1_CNFG, REG_IN2_CNFG, REG_IN3_CNFG, REG_IN4_CNFG, REG_IN5_CNFG,
  REG_IN6_CNFG, REG_IN7_CNFG, REG_IN8_CNFG, REG_IN9_CNFG, REG_IN10_CNFG,
  REG_IN11_CNFG, REG_IN12_CNFG, REG_IN13_CNFG, REG_IN14_CNFG, };

static const T_osUint8 IDTInput_inputStatusRegArray[] =
{ REG_IN1_IN2_STS, REG_IN3_IN4_STS, REG_IN5_IN6_STS, REG_IN7_IN8_STS,
  REG_IN9_IN10_STS, REG_IN11_IN12_STS, REG_IN13_IN14_STS };

static const bucketReg_t IDTInput_bucketRegArray[] =
{
    { REG_UPPER_THRESHOLD_0_CNFG, REG_LOWER_THRESHOLD_0_CNFG,
      REG_BUCKET_SIZE_0_CNFG, REG_DECAY_RATE_0_CNFG }, /* Bucket 0 */
    { REG_UPPER_THRESHOLD_1_CNFG, REG_LOWER_THRESHOLD_1_CNFG,
      REG_BUCKET_SIZE_1_CNFG, REG_DECAY_RATE_1_CNFG }, /* Bucket 1 */
    { REG_UPPER_THRESHOLD_2_CNFG, REG_LOWER_THRESHOLD_2_CNFG,
      REG_BUCKET_SIZE_2_CNFG, REG_DECAY_RATE_2_CNFG }, /* Bucket 2 */
    { REG_UPPER_THRESHOLD_3_CNFG, REG_LOWER_THRESHOLD_3_CNFG,
      REG_BUCKET_SIZE_3_CNFG, REG_DECAY_RATE_3_CNFG } /* Bucket 3 */
};

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * @brief Translate input frequency bit value to input frequency
 * family
 * @param hdlr Device driver handler
 * @param inFreqBitValue Input frequency bit value
 * @return Input frequency family
 */
T_idtInputFrequencyFamily IDTInput_InFreqBitValue2Family(
                            T_idtDrvHdlr      hdlr,
                            T_idtInputFrequencyBitfieldValue inFreqBitValue)
{
    switch (inFreqBitValue)
    {
        case Freq_8K:
            return InFreqFamily_8K;
        break;

        /* Freq_1544M has the same
         * bit value as Freq_2048M, we
         * need to check network type
         * to determine the frequency
         * family
         */
        case Freq_1544M: /*or Freq_2048M */
            if (IDTGeneral_GetNetworkType(hdlr) == Network_SDH)
                /* SDH */
                return InFreqFamily_2048M;
            else
                /* SONET */
                return InFreqFamily_1544M;
        break;

        case Freq_648M:
            return InFreqFamily_648M;
        break;

        case Freq_1944M:
            return InFreqFamily_1944M;
        break;

        case Freq_2592M:
            return InFreqFamily_2592M;
        break;

        case Freq_3888M:
            return InFreqFamily_3888M;
        break;

        case Freq_2K:
            return InFreqFamily_2K;
        break;

        case Freq_4K:
            return InFreqFamily_4K;
        break;

        case Freq_1PPS:
            return InFreqFamily_1PPS;
        break;

        case Freq_625M:
            return InFreqFamily_625M;
        break;

        case Freq_10M:
            return InFreqFamily_10M;
        break;

        default:
            return InFreqFamily_MAX;
        break;
    }
}

/**
 * @brief   Set specified port to the priority
 * @param hdlr Device driver handler
 * @param nInputNo Input port number (consult data sheet for valid
 * port numbers)
 * @param nDpll DPLL instance
 * @param nPriority the priority of the specified port
 */
void IDTInput_SetPriority(T_idtDrvHdlr      hdlr, 
                          T_osUint8         nInputNo, 
                          T_idtDpllInstance nDpll,
                          T_osUint8         nPriority)
{
    T_osUint8 internalPort;
    T_osUint8 nAddr, nShift;
    T_osUint8 regOffset;
    T_idtDpllType dpllType;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d nDpll/%d nPriority/%d\n",
               __FUNCTION__, hdlr->name_m, nInputNo, nDpll, nPriority));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT(nPriority<16);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    regOffset = (internalPort - 1) / 2;
    nShift = ((internalPort - 1) % 2) ? 4 : 0;
    nAddr = IDTInput_inputPriorityRegArray[regOffset];

    IDTHAL_WriteBitfield(hdlr,nAddr,
                         (0x0F << nShift), /* Mask */
                         nShift, /* LSB */
                         nPriority);
}

/**
 * @brief   Get the current priority of the specified port
 * @param hdlr Device driver handler
 * @param nInputNo Input port number (consult data sheet for valid
 * port numbers)
 * @param nDpll DPLL instance
 * @return	Priority of the port
 */
T_osUint8 IDTInput_GetPriority(T_idtDrvHdlr      hdlr, 
                               T_osUint8         nInputNo, 
                               T_idtDpllInstance nDpll)
{
    T_osUint8 internalPort;
    T_osUint8 nAddr, nShift;
    T_osUint8 regOffset;
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    regOffset = (internalPort - 1) / 2;
    nShift = ((internalPort - 1) % 2) ? 4 : 0;
    nAddr = IDTInput_inputPriorityRegArray[regOffset];

    return IDTHAL_ReadBitfield(hdlr,nAddr,
                               (0x0F << nShift), /* Mask */
                               nShift); /* LSB */
}

/**
 * @brief   Set the frequency of the input clock at the specified port
 * @param hdlr Device driver handler
 * @param	  nInputNo the specified input port number
 * @param	  nFrequency code of input clock frequency.<ul>
 *				<li>Freq_8K= 8 kHz</li>
 *				<li>Freq_1544M/Freq_2048M= 1.544/2.048 MHz</li>
 *				<li>Freq_648M= 6.48 MHz</li>
 *				<li>Freq_1944M= 19.44 MHz</li>
 *				<li>Freq_2592M= 25.92 MHz</li>
 *				<li>Freq_3888M= 38.88 MHz</li>
 *				<li>Freq_2K= 2 kHz</li>
 *				<li>Freq_4K= 4 kHz</li>
 *				<li>Freq_1PPS= 1 PPS</li>
 *				<li>Freq_625M= 6.25 MHz</li>
 *				<li>Freq_10M= 10 MHz</li></ul>
 */
void IDTInput_SetInputFrequency(T_idtDrvHdlr                     hdlr, 
                                T_osUint8                        nInputNo,
                                T_idtInputFrequencyBitfieldValue nFrequency)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d nFrequency/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo, nFrequency));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);

    OS_ASSERTS(((internalPort >= 3) || (nFrequency == Freq_8K)),
               ("Input Port %d can only be set to 8kHz\n",
                nInputNo));

    /* Input port to array index */
    internalPort--;

    IDTHAL_WriteBitfield(hdlr, 
                         IDTInput_inputFredRegArray[internalPort], 
                         BF_IN_FREQ_MASK,
                         BF_IN_FREQ_LSB, 
                         (T_osUint8) nFrequency);
}

/**
 * @brief   Get frequency of the input clock at the specified port
 * @param hdlr Device driver handler
 * @param	  nInputNo the specified input port number
 * @return	  nFrequency code of input clock frequency.<ul>
 *				<li>Freq_8K= 8 kHz</li>
 *				<li>Freq_1544M/Freq_2048M= 1.544/2.048 MHz</li>
 *				<li>Freq_648M= 6.48 MHz</li>
 *				<li>Freq_1944M= 19.44 MHz</li>
 *				<li>Freq_2592M= 25.92 MHz</li>
 *				<li>Freq_3888M= 38.88 MHz</li>
 *				<li>Freq_2K= 2 kHz</li>
 *				<li>Freq_4K= 4 kHz</li>
 *				<li>Freq_1PPS= 1 PPS</li>
 *				<li>Freq_625M= 6.25 MHz</li>
 *				<li>Freq_10M= 10 MHz</li></ul>
 */
T_idtInputFrequencyBitfieldValue IDTInput_GetInputFrequency(
    T_idtDrvHdlr hdlr,
    T_osUint8    nInputNo)
{
    T_osUint8 internalPort;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);

    /* Input port to array index */
    internalPort--;

    return IDTHAL_ReadBitfield(hdlr, 
                               IDTInput_inputFredRegArray[internalPort], 
                               BF_IN_FREQ_MASK,
                               BF_IN_FREQ_LSB);
}

/**
 * @brief Select one of the four activity monitor configurations for
 * the specified input port
 * @param hdlr Device driver handler
 * @param	  nInputNo the specified port
 * @param	  nBucket the bucket number assigned to the specified port.<ul>
 *			<li>0= bucket 0, controlled by the registers from 31H to 34H</li>
 *			<li>1= bucket 1, controlled by the registers from 35H to 38H</li>
 *			<li>2= bucket 2, controlled by the registers from 39H to 3cH</li>
 *			<li>3= bucket 3, controlled by the registers from 3dH to 40H</li></ul>
 */
void IDTInput_SetActivityMonitor(T_idtDrvHdlr hdlr, 
                                 T_osUint8    nInputNo,
                                 T_osUint8    nBucket)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d nBucket/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo, nBucket));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERT(nBucket<4);

    /* Input port to array index */
    internalPort--;

    IDTHAL_WriteBitfield(hdlr, 
                         IDTInput_inputFredRegArray[internalPort],
                         BF_BUCKET_SEL_MASK, 
                         BF_BUCKET_SEL_LSB, 
                         nBucket);
}

/**
 * @brief Retrieve selected activity monitoring configuration.
 * @param hdlr Device driver handler
 * @param	  nInputNo the specified port
 * @return	  nBucket the bucket number assigned to the specified port.<ul>
 *			<li>0= bucket 0, controlled by the registers from 31H to 34H</li>
 *			<li>1= bucket 1, controlled by the registers from 35H to 38H</li>
 *			<li>2= bucket 2, controlled by the registers from 39H to 3cH</li>
 *			<li>3= bucket 3, controlled by the registers from 3dH to 40H</li></ul>
 */
T_osUint8 IDTInput_GetActivityMonitor(T_idtDrvHdlr hdlr, T_osUint8 nInputNo)
{
    T_osUint8 internalPort;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);

    /* Input port to array index */
    internalPort--;

    return IDTHAL_ReadBitfield(hdlr, 
                               IDTInput_inputFredRegArray[internalPort], 
                               BF_BUCKET_SEL_MASK,
                               BF_BUCKET_SEL_LSB);
}

/**
 * @brief   Set the configuration to Leaky Bucket activity monitoring
 * @param hdlr Device driver handler
 * @param	  nBucket the bucket number to provision 0-3
 * @param	  nUpper Upper threshold of Leaky Bucket
 * @param	  nLower Lower threshold of Leaky Bucket
 * @param	  nSize  Bucket size of Leaky Bucket
 * @param	  nDecay Decay rate of Leaky bucket<ul>
 * 			<li>0x0 = Bucket decrease by 1 in every 128 ms</li>
 * 			<li>0x1 = Bucket decrease by 1 in every 256 ms</li>
 * 			<li>0x2 = Bucket decrease by 1 in every 512 ms</li>
 * 			<li>0x3 = Bucket decrease by 1 in every 1024 ms</li></ul>
 */
void IDTInput_SetBucketConfig(T_idtDrvHdlr hdlr, 
                              T_osUint8 nBucket, 
                              T_osUint8 nUpper,
                              T_osUint8 nLower, 
                              T_osUint8 nSize, 
                              T_osUint8 nDecay)
{
    const bucketReg_t *ptr;

    OS_LOGGER(5,
              ("%s dev/%s nBucket/%d nUpper/%d nLower/%d "
               "nSize/%d nDecay/%d\n", 
               __FUNCTION__, hdlr->name_m, nBucket, nUpper, nLower, nSize, nDecay));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nBucket<3);

    ptr = &IDTInput_bucketRegArray[nBucket];

    /* All leaky bucket registers (provisionable parameters) are byte wide. */
    IDTHal_Write(hdlr, ptr->upperThreshReg_m, nUpper);
    IDTHal_Write(hdlr, ptr->lowerThreshReg_m, nLower);
    IDTHal_Write(hdlr, ptr->bucketSizeReg_m, nSize);
    IDTHal_Write(hdlr, ptr->decayRateReg_m, nDecay);
}

/**
 * @brief   Get activity monitoring configuration (Leaky Bucket)
 * @param hdlr Device driver handler
 * @param	  nBucket the bucket number to provision 0-3
 * @param	  nUpper_p Upper threshold of Leaky Bucket
 * @param	  nLower_p Lower threshold of Leaky Bucket
 * @param	  nSize_p  Bucket size of Leaky Bucket
 * @param	  nDecay_p Decay rate of Leaky bucket<ul>
 * 			<li>0x0 = Bucket decrease by 1 in every 128 ms</li>
 * 			<li>0x1 = Bucket decrease by 1 in every 256 ms</li>
 * 			<li>0x2 = Bucket decrease by 1 in every 512 ms</li>
 * 			<li>0x3 = Bucket decrease by 1 in every 1024 ms</li></ul>
 */
void IDTInput_GetBucketConfig(T_idtDrvHdlr hdlr, 
                              T_osUint8 nBucket,
                              T_osUint8 *nUpper_p, 
                              T_osUint8 *nLower_p, 
                              T_osUint8 *nSize_p,
                              T_osUint8 *nDecay_p)
{
    const bucketReg_t *ptr;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nBucket<3);
    OS_ASSERT_PTR_NULL(nUpper_p);
    OS_ASSERT_PTR_NULL(nLower_p);
    OS_ASSERT_PTR_NULL(nSize_p);
    OS_ASSERT_PTR_NULL(nDecay_p);

    ptr = &IDTInput_bucketRegArray[nBucket];

    /* All leaky bucket registers (provisionable parameters) are byte
     * wide. */
    *nUpper_p = IDTHal_Read(hdlr, ptr->upperThreshReg_m);
    *nLower_p = IDTHal_Read(hdlr, ptr->lowerThreshReg_m);
    *nSize_p = IDTHal_Read(hdlr, ptr->bucketSizeReg_m);
    *nDecay_p = IDTHal_Read(hdlr, ptr->decayRateReg_m);
}

/**
 * @brief   Assign a pre-divider for the specified input
 * @param hdlr Device driver handler
 * @param nInputNo  the specified input port number
 * @param nDivider pre-divider assigned to the specified input<ul>
 * 				<li>0 = bypass both dividers</li>
 * 				<li>1 = Lock-8k divider</li>
 * 				<li>2 = DivN divider</li></ul>
 */
void IDTInput_SetPreDivider(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
                            T_idtInputDividerMux nDivider)
{
    T_osUint8 internalPort;
    T_osUint8 nIntInd;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d nDivider/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo, nDivider));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERT((internalPort<=14)&&(internalPort>=3));
    OS_ASSERT(nDivider<=2);

    /* Input number to index */
    internalPort--;
    nIntInd = IDTHal_Read(hdlr, IDTInput_inputFredRegArray[internalPort]);

    IDTHAL_ModifyBitfield(nIntInd, BF_DIRECT_DIV_MASK, BF_DIRECT_DIV_LSB,
                          ((T_osUint8) nDivider & 2) != 0);

    IDTHAL_ModifyBitfield(nIntInd, BF_LOCK_8K_MASK, BF_LOCK_8K_LSB,
                          ((T_osUint8) nDivider & 1) != 0);

    IDTHal_Write(hdlr, IDTInput_inputFredRegArray[internalPort], nIntInd);
}

/**
 * @brief   Get configured input port pre-divider
 * @param hdlr Device driver handler
 * @param nInputNo  the specified input port number
 * @return pre-divider assigned to the specified input<ul>
 * 				<li>0 = bypass both dividers</li>
 * 				<li>1 = Lock-8k divider</li>
 * 				<li>2 = DivN divider</li></ul>
 */
T_idtInputDividerMux IDTInput_GetPreDivider(T_idtDrvHdlr hdlr, 
                                            T_osUint8 nInputNo)
{
    T_osUint8 internalPort;
    T_osUint8 nIntInd;
    T_idtInputDividerMux retVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERT((internalPort<=14)&&(internalPort>=3));

    /* Input number to index */
    internalPort--;
    nIntInd = IDTHal_Read(hdlr, IDTInput_inputFredRegArray[internalPort]);

    retVal =
        (T_idtInputDividerMux) (IDTHAL_GetBitfield(
                                    nIntInd,
                                    (BF_LOCK_8K_MASK|BF_DIRECT_DIV_MASK), 
                                    BF_LOCK_8K_LSB));
    return retVal;
}

/**
 * @brief   Set the dividen factor to pre-divider assigned to a certain input
 * @param hdlr Device driver handler
 * @param nInputNo the input port number to which the pre-divider
 * @param nFactor division factor as a preset of the specified divider
 */
void IDTInput_SetDivisionFactor(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
                                T_osUint16 nFactor)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d nFactor/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo, nFactor));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERT((internalPort<=14)&&(internalPort>=3));
    OS_ASSERT(nFactor<(1<<16));

    /* Select input */
    IDTHAL_WriteBitfield(hdlr, 
                         REG_PRE_DIV_CH_CNFG, 
                         BF_PRE_DIV_CH_VALUE_MASK,
                         BF_PRE_DIV_CH_VALUE_LSB, 
                         internalPort);

    /* Set divisor factor */
    IDTHal_Write(hdlr, REG_PRE_DIVN_LOW_CNFG,
                 (nFactor & BF_PRE_DIVN_VALUE_A_MASK));
    IDTHAL_WriteBitfield(hdlr, 
                         REG_PRE_DIVN_HIGH_CNFG, 
                         BF_PRE_DIVN_VALUE_B_MASK,
                         BF_PRE_DIVN_VALUE_B_LSB, 
                         (nFactor >> BF_PRE_DIVN_VALUE_A_SIZE));
}

/**
 * @brief   Get the dividen factor to pre-divider assigned to a certain input
 * @param hdlr Device driver handler
 * @param nInputNo the input port number to which the pre-divider
 * @return division factor as a preset of the specified divider (24
 * bit 2's compliment)
 */
T_osUint16 IDTInput_GetDivisionFactor(T_idtDrvHdlr hdlr, T_osUint8 nInputNo)
{
    T_osUint8 internalPort;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERT((internalPort<=14)&&(internalPort>=3));

    /* Select input */
    IDTHAL_WriteBitfield(hdlr, 
                         REG_PRE_DIV_CH_CNFG, 
                         BF_PRE_DIV_CH_VALUE_MASK,
                         BF_PRE_DIV_CH_VALUE_LSB, 
                         internalPort);

    /* Get divisor factor */
    return (IDTHal_Read(hdlr, REG_PRE_DIVN_LOW_CNFG)
            | (IDTHAL_ReadBitfield(
                   hdlr, 
                   REG_PRE_DIVN_HIGH_CNFG, 
                   BF_PRE_DIVN_VALUE_B_MASK,
                   BF_PRE_DIVN_VALUE_B_LSB) << BF_PRE_DIVN_VALUE_A_SIZE));
}

/**
 * @brief Set the usage of high frequency (> 155.52MHz) divider.
 * @param hdlr Device driver handler
 * @param nInputNo Input port number
 * @param nUsage  <ul><li>HF_Bypass : The HF divider is bypassed</li>
 *          <li>HF_Divide_4 : The HF divider's division factor is 4</li>
 *          <li>HF_Divide_5 : The HF divider's division factor is 5</li></ul>
 */
void IDTInput_SetInputHFdivider(T_idtDrvHdlr hdlr, 
                                T_osUint8 nInputNo,
                                T_idtHighFreqencyDivisor nUsage)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d nUsage/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo, nUsage));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nUsage<HF_MAX);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERTS((internalPort==5)||(internalPort==6),
               ("Input port %d does not support feature\n",
                nInputNo));

    if (internalPort == 5)
        IDTHAL_WriteBitfield(hdlr, 
                             REG_IN5_IN6_HF_DIV_CNFG, 
                             BF_IN5_DIV_MASK,
                             BF_IN5_DIV_LSB, 
                             (T_osUint8) nUsage);

    if (internalPort == 6)
        IDTHAL_WriteBitfield(hdlr, 
                             REG_IN5_IN6_HF_DIV_CNFG, 
                             BF_IN6_DIV_MASK,
                             BF_IN6_DIV_LSB, 
                             (T_osUint8) nUsage);

}

/**
 * @brief Get high frequency configuration divider.
 * @param hdlr Device driver handler
 * @param nInputNo Input port number
 * @return  <ul><li>HF_Bypass : The HF divider is bypassed</li>
 *              <li>HF_Divide_4 : The HF divider's division factor is 4</li>
 *              <li>HF_Divide_5 : The HF divider's division factor is 5</li></ul>
 */
T_idtHighFreqencyDivisor IDTInput_GetInputHFdivider(T_idtDrvHdlr hdlr,
                                                    T_osUint8 nInputNo)
{
    T_osUint8 internalPort;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERTS((internalPort==5)||(internalPort==6),
               ("Input port %d does not support feature\n",
                nInputNo));

    if (internalPort == 5)
        return IDTHAL_ReadBitfield(hdlr, 
                                   REG_IN5_IN6_HF_DIV_CNFG, 
                                   BF_IN5_DIV_MASK,
                                   BF_IN5_DIV_LSB);

    if (internalPort == 6)
        return IDTHAL_ReadBitfield(hdlr, 
                                   REG_IN5_IN6_HF_DIV_CNFG, 
                                   BF_IN6_DIV_MASK,
                                   BF_IN6_DIV_LSB);

    /* Should not get here */
    return 0;
}

/**
 * @brief Get the frequency offset of the specified input clock. The
 * returned value is updated every 16 seconds.
 * @param hdlr Device driver handler
 * @param nInputNo number of the input clock whose frequency offset is
 * returned.
 * @return Frequency offset
 */
T_osUint8 IDTInput_GetInputFreqOffset(T_idtDrvHdlr hdlr, T_osUint8 nInputNo)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERT(internalPort<=IDT_DRV_MAX_INPUT);

    /* Bitfield takes non-indexed value of input port */
    IDTHAL_WriteBitfield(hdlr, 
                         REG_IN_FREQ_READ_CH_CNFG,
                         BF_IN_FREQ_READ_CH_MASK, 
                         BF_IN_FREQ_READ_CH_LSB, 
                         internalPort);

    /* Technically we should wait for 16 seconds (register is updated
       every 16 seconds. */
    return IDTHal_Read(hdlr, REG_IN_FREQ_READ_STS);
}

/**
 * @brief   Get the status of a specified input clock
 * @param hdlr Device driver handler
 * @param	  nInputNo number of the input clock
 \todo description
 * @return	 nStatus.Bit0: 	=1, Phase lock alarm.
 **				=0, No phase lock alarm.
 **		 nStatus.Bit1:	=1, Clock no-activity alarm.
 **				=0, No clock no-activity alarm.
 **		 nStatus.Bit2: 	=1, Clock frequency hard alarm.
 **				=0, No clock frequency hard alarm.
 */
T_osUint8 IDTInput_GetInputClockStatus(T_idtDrvHdlr hdlr, 
                                       T_osUint8 nInputNo)
{
    T_osUint8 internalPort;
    T_osUint8 regAddr;
    T_osUint8 bfLsb;
    T_osUint8 bfMask;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERT(internalPort<=IDT_DRV_MAX_INPUT);

    regAddr = IDTInput_inputStatusRegArray[(internalPort - 1) / 2];
    bfLsb = ((internalPort - 1) % 2) ? 4 : 0;
    bfMask = 0x0F << bfLsb;
    return IDTHAL_ReadBitfield(hdlr, regAddr, bfMask, bfLsb);
}

/**
 * @brief Get the clock validation status of an input port
 * @param hdlr Device driver handler
 * @param nInputNo Input port
 * @return 0-Invalid clock, 1-Valid clock
 *
 */
T_osUint8 IDTInput_GetInputClockValidation(T_idtDrvHdlr hdlr, 
                                           T_osUint8 nInputNo)
{
    T_osUint8 internalPort;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);

    /* Input number to index */
    internalPort--;

    if (internalPort < 7)
    {
        return IDTHAL_ReadBitfield(hdlr, REG_INPUT_VALID1_STS,
                                   (BF_IN1_MASK << internalPort),
                                   (BF_IN1_LSB + internalPort));
    }
    else
    {
        internalPort -= 8;
        return IDTHAL_ReadBitfield(hdlr, REG_INPUT_VALID2_STS,
                                   (BF_IN9_MASK << internalPort),
                                   (BF_IN9_LSB + internalPort));
    }

    /* Should never get here */
    return 0;
}

/**
 * @brief   Enable to select anyone of input clocks
 * @param hdlr Device driver handler
 */
void IDTInput_EnableAllInputs(T_idtDrvHdlr hdlr)
{
    int i;
    OS_ASSERT_PTR_NULL(hdlr);

    for (i = 0; i < IDT_DRV_MAX_INPUT; i++)
    {
        /* If there is a translated input port, then enable it */
        if (hdlr->deviceConfig_m->inputExt2IntXlate_ma[i + 1])
        {
            IDTInput_SetInputEnable(hdlr, i + 1, 1);
        }
    }
}

/**
 * @brief   Disable to select anyone of input clocks
 * @param hdlr Device driver handler
 */
void IDTInput_DisableAllInputs(T_idtDrvHdlr hdlr)
{
    int i;
    OS_ASSERT_PTR_NULL(hdlr);

    for (i = 0; i < IDT_DRV_MAX_INPUT; i++)
    {
        /* If there is a translated input port, then enable it */
        if (hdlr->deviceConfig_m->inputExt2IntXlate_ma[i + 1])
        {
            IDTInput_SetInputEnable(hdlr, i + 1, 0);
        }
    }
}

/**
 * @brief Enable/Disable selected input
 * @param hdlr Device driver handler
 * @param nInputNo number of the input clock
 * @param enable 1-Disable 0-Enable input port
 */
void IDTInput_SetInputEnable(T_idtDrvHdlr hdlr, 
                             T_osUint8 nInputNo, 
                             T_osUint8 enable)
{
    T_osUint8 internalPort;
    T_osUint8 regAddr;
    T_osUint8 bfLsb;
    T_osUint8 bfMask;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d enable/%d\n",
               __FUNCTION__, hdlr->name_m, nInputNo, enable));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);

    if (internalPort <= 8)
    {
        regAddr = REG_REMOTE_INPUT_VALID1_CNFG;
        bfLsb = internalPort - 1;
    }
    else
    {
        regAddr = REG_REMOTE_INPUT_VALID2_CNFG;
        bfLsb = internalPort - 9;
    }
    bfMask = 1 << bfLsb;

    /* The bit is actually a disable (1-disable, 0-enable) */
    IDTHAL_WriteBitfield(hdlr, regAddr, bfMask, bfLsb, enable?0:1);
}

/**
 * @brief Retrieve enable/disable status of selected input
 * @param hdlr Device driver handler
 * @param nInputNo number of the input clock
 * @return enable 1-Disable 0-Enable input port
 */
T_osUint8 IDTInput_GetInputEnable(T_idtDrvHdlr hdlr, T_osUint8 nInputNo)
{
    T_osUint8 internalPort;
    T_osUint8 regAddr;
    T_osUint8 bfLsb;
    T_osUint8 bfMask;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);

    if (internalPort <= 8)
    {
        regAddr = REG_REMOTE_INPUT_VALID1_CNFG;
        bfLsb = internalPort - 1;
    }
    else
    {
        regAddr = REG_REMOTE_INPUT_VALID2_CNFG;
        bfLsb = internalPort - 9;
    }
    bfMask = 1 << bfLsb;

    /* The bit is actually a disable (1-disable, 0-enable) */
    return IDTHAL_ReadBitfield(hdlr, regAddr, bfMask, bfLsb)?0:1;
}

/**
 * @brief Function to select input external sync frequency
 * @param hdlr Device driver handler
 * @param syncFreq Input external sync
 */
void IDTInput_SetSyncFrequency(T_idtDrvHdlr hdlr, 
                               T_idtInputSyncFreq syncFreq)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(syncFreq<SYNC_MAX);
    OS_LOGGER(5,
              ("%s dev/%s syncFreq/%d\n",
               __FUNCTION__, hdlr->name_m, syncFreq));

    IDTHAL_WriteBitfield(hdlr, 
                         REG_INPUT_MODE_CNFG,
                         BF_SYNC_FREQ_MASK,
                         BF_SYNC_FREQ_LSB,
                         syncFreq);
}

/**
 * @brief Function to retrieve select input external sync frequency
 * @param hdlr Device driver handler
 * @return Input external sync
 */
T_idtInputSyncFreq IDTInput_GetSyncFrequency(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return (T_idtInputSyncFreq)IDTHAL_ReadBitfield(hdlr,
                                                   REG_INPUT_MODE_CNFG,
                                                   BF_SYNC_FREQ_MASK,
                                                   BF_SYNC_FREQ_LSB);
}

/**
 * @brief Function to provision sampling configuration for external sync
 * @param hdlr Device driver handler
 * @param exSyncN External sync (1-EXT SYNC1, 2-EX SYNC2)
 * @param sampling Sampling configuration
 */
void IDTInput_SetSyncSampling(T_idtDrvHdlr hdlr,
                              T_osUint8 exSyncN,
                              T_externalSyncSampling sampling)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT((exSyncN>0) && ( exSyncN<=2));
    OS_ASSERT(sampling<externalSyncSampling_MAX);

    OS_LOGGER(5,
              ("%s dev/%s exSyncN/%d sampling/%d\n", 
               __FUNCTION__, hdlr->name_m, exSyncN, sampling));

    switch (exSyncN) {
    case 1:
        IDTHAL_WriteBitfield(hdlr,
                             REG_SYNC_PHASE_CNFG,
                             BF_SYNC_PH1_MASK,
                             BF_SYNC_PH1_LSB,
                             sampling);
        break;
    case 2:
        IDTHAL_WriteBitfield(hdlr,
                             REG_SYNC_PHASE_CNFG,
                             BF_SYNC_PH2_MASK,
                             BF_SYNC_PH2_LSB,
                             sampling);
        break;
    default:
        OS_ASSERTS(0,("Invalid sync number %d\n",exSyncN));
        break;
    }
}

/**
 * @brief Function to retrieve sampling configuration for external sync
 * @param hdlr Device driver handler
 * @param exSyncN External sync (1-EXT SYNC1, 2-EX SYNC2)
 * @return Sampling configuration
 */
T_externalSyncSampling IDTInput_GetSyncSampling(T_idtDrvHdlr hdlr,
                                                T_osUint8 exSyncN)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT((exSyncN>0) && ( exSyncN<=2));

    switch (exSyncN) {
    case 1:
        return (T_externalSyncSampling)IDTHAL_ReadBitfield(
            hdlr,
            REG_SYNC_PHASE_CNFG,
            BF_SYNC_PH1_MASK,
            BF_SYNC_PH1_LSB);
        break;
    case 2:
        return (T_externalSyncSampling)IDTHAL_ReadBitfield(
            hdlr,
            REG_SYNC_PHASE_CNFG,
            BF_SYNC_PH2_MASK,
            BF_SYNC_PH2_LSB);
        break;
    default:
        OS_ASSERTS(0,("Invalid sync number %d\n",exSyncN));
        break;
    }

    /* Should never get here */
    return externalSyncSampling_MAX;
}

/**
 * @brief Function to provision synchronization alignment
 * @param hdlr Device driver handler
 * @param edge Edge to syncrhonize against
 */
void IDTInput_SetSyncEdge(T_idtDrvHdlr hdlr,
                          T_externalSyncEdge edge)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(edge<externalSyncEdge_MAX);

    OS_LOGGER(5,
              ("%s dev/%s edge/%d\n", 
               __FUNCTION__, hdlr->name_m, edge));

    IDTHAL_WriteBitfield(hdlr,
                         REG_SYNC_PHASE_CNFG,
                         BF_SYNC_EDGE_MASK,
                         BF_SYNC_EDGE_LSB,
                         edge);
}

/**
 * @brief Function to retrieve synchronization alignment
 * @param hdlr Device driver handler
 * @return Edge to syncrhonize against
 */
T_externalSyncEdge IDTInput_GetSyncEdge(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return (T_externalSyncEdge)IDTHAL_ReadBitfield(hdlr,
                                                   REG_SYNC_PHASE_CNFG,
                                                   BF_SYNC_EDGE_MASK,
                                                   BF_SYNC_EDGE_LSB);
}

/**
 * @brief Function to provision the sync allowable range before
 * alarming.
 * @param hdlr Device driver handler
 * @param range Allowable range
 */
void IDTInput_SetSyncAlarmRange(T_idtDrvHdlr hdlr,
                                T_externalSyncAlarmRange range)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(range<externalSyncAlarmRange_MAX);

    OS_LOGGER(5,
              ("%s dev/%s range/%d\n", 
               __FUNCTION__, hdlr->name_m, range));

    IDTHAL_WriteBitfield(hdlr,
                         REG_SYNC_PHASE_CNFG,
                         BF_SYNC_MON_LIMT_MASK,
                         BF_SYNC_MON_LIMT_LSB,
                         range);
}

/**
 * @brief Function to retrieve the sync allowable range before
 * alarming.
 * @param hdlr Device driver handler 
 * @return Allowable range
 */
T_externalSyncAlarmRange IDTInput_GetSyncAlarmRange(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return (T_externalSyncAlarmRange)IDTHAL_ReadBitfield(
        hdlr,
        REG_SYNC_PHASE_CNFG,
        BF_SYNC_MON_LIMT_MASK,
        BF_SYNC_MON_LIMT_LSB);
}

/**
 * @brief Function to provision sync bypass mode
 * @param hdlr Device driver handler
 * @param bypass Bypass mode
 */
void IDTInput_SetSyncBypass(T_idtDrvHdlr hdlr,
                            T_externalSyncBypass bypass)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(bypass<externalSyncBypass_MAX);

    OS_LOGGER(5,
              ("%s dev/%s bypass/%d\n", 
               __FUNCTION__, hdlr->name_m, bypass));

    IDTHAL_WriteBitfield(hdlr,
                         REG_SYNC_MONITOR_CNFG,
                         BF_SYNC_BYPASS_MASK,
                         BF_SYNC_BYPASS_LSB,
                         bypass);
}

/**
 * @brief Function to retrieve sync bypass mode
 * @param hdlr Device driver handler
 * @return Bypass mode
 */
T_externalSyncBypass IDTInput_GetSyncBypass(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return (T_externalSyncBypass)IDTHAL_ReadBitfield(hdlr,
                                                     REG_SYNC_MONITOR_CNFG,
                                                     BF_SYNC_BYPASS_MASK,
                                                     BF_SYNC_BYPASS_LSB);
}

/**
 * @brief Function to provision enable mode for input external sync
 * @param hdlr Device driver handler
 * @param enable Enable mode
 */
void IDTInput_SetSyncEnable(T_idtDrvHdlr hdlr,
                            T_externalSyncEnable enable)
{
    T_osUint8 regVal;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(enable<externalSyncEnable_MAX);

    OS_LOGGER(5,
              ("%s dev/%s enable/%d\n", 
               __FUNCTION__, hdlr->name_m, enable));

    regVal = IDTHal_Read(hdlr,
                         REG_INPUT_MODE_CNFG);
    switch(enable) {
    case externalSyncEnable_Disable:
        IDTHAL_ModifyBitfield(regVal,
                              BF_AUTO_EXT_SYNC_EN_MASK,
                              BF_AUTO_EXT_SYNC_EN_LSB,
                              0);
        IDTHAL_ModifyBitfield(regVal,
                              BF_EXT_SYNC_EN_MASK,
                              BF_EXT_SYNC_EN_LSB,
                              0);
        break;
    case externalSyncEnable_Enable:
        IDTHAL_ModifyBitfield(regVal,
                              BF_AUTO_EXT_SYNC_EN_MASK,
                              BF_AUTO_EXT_SYNC_EN_LSB,
                              0);
        IDTHAL_ModifyBitfield(regVal,
                              BF_EXT_SYNC_EN_MASK,
                              BF_EXT_SYNC_EN_LSB,
                              1);
        break;
    case externalSyncEnable_Auto:
        IDTHAL_ModifyBitfield(regVal,
                              BF_AUTO_EXT_SYNC_EN_MASK,
                              BF_AUTO_EXT_SYNC_EN_LSB,
                              1);
        IDTHAL_ModifyBitfield(regVal,
                              BF_EXT_SYNC_EN_MASK,
                              BF_EXT_SYNC_EN_LSB,
                              1);
        break;
    default:
        OS_ASSERTS(0,("Invalid enable mode %d\n",enable));
        break;
    }  
    IDTHal_Write(hdlr,
                 REG_INPUT_MODE_CNFG,
                 regVal);
}

/**
 * @brief Function to retrieve external sync enable mode.
 * @param hdlr Device driver handler
 * @return Enable mode
 */
T_externalSyncEnable IDTInput_GetSyncEnable(T_idtDrvHdlr hdlr)
{
    T_osUint8 regVal;

    OS_ASSERT_PTR_NULL(hdlr);

    regVal = IDTHal_Read(hdlr,
                         REG_INPUT_MODE_CNFG);

    if (!IDTHAL_GetBitfield(regVal,
                            BF_EXT_SYNC_EN_MASK,
                            BF_EXT_SYNC_EN_LSB))
        return externalSyncEnable_Disable;
    else
        if (!IDTHAL_GetBitfield(regVal,
                                BF_AUTO_EXT_SYNC_EN_MASK,
                                BF_AUTO_EXT_SYNC_EN_LSB))
            return externalSyncEnable_Enable;
        else
            return externalSyncEnable_Auto;
}

/**
 * @brief Provision AMI signal for input port 
 * @param hdlr Device driver handler
 * @param nInputNo the specified input port number
 * @param amiSignal AMI signal
 *    <ul><li>AmiFreq_64kHzPlus8kHz= 64 kHz + 8 kHz signal</li>
 *        <li> AmiFreq_64kHzPlus400Hz= 64 kHz + 400 Hz signal</li></ul>
 */
void IDTInput_SetAmiInputFrequency(
    T_idtDrvHdlr                        hdlr,
    T_osUint8                           nInputNo,
    T_idtAmiInputFrequencyBitFieldValue amiSignal)
{
    T_osUint8 internalPort;

    OS_LOGGER(5,
              ("%s dev/%s nInputNo/%d amiSignal/%d\n", 
               __FUNCTION__, hdlr->name_m, nInputNo, amiSignal));

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    OS_ASSERT(amiSignal<AmiFreq_MAX);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERTS(internalPort<3,
               ("Input port %d is no AMI port\n",nInputNo));


    if (internalPort == 1)
    {
        IDTHAL_WriteBitfield(hdlr,
                             REG_IN1_CNFG,
                             BF_400HZ_SEL_MASK,
                             BF_400HZ_SEL_LSB,
                             amiSignal);
    }
    else if (internalPort == 2)
    {
        IDTHAL_WriteBitfield(hdlr,
                             REG_IN2_CNFG,
                             BF_400HZ_SEL_MASK,
                             BF_400HZ_SEL_LSB,
                             amiSignal);
    }
}

/**
 * @brief Function to retrieve AMI input frequency for input port
 * @param hdlr Device driver handler
 * @param nInputNo the specified input port number
 * @return AMI input frequency
 */
T_idtAmiInputFrequencyBitFieldValue IDTInput_GetAmiInputFrequency(
    T_idtDrvHdlr hdlr,
    T_osUint8    nInputNo)
{
    T_osUint8 internalPort;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nInputNo<=IDT_DRV_MAX_INPUT);
    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr, nInputNo, internalPort);
    OS_ASSERTS(internalPort<3,
               ("Input port %d is not AMI port\n",nInputNo));

    if (internalPort == 1)
    {
        return ((T_idtAmiInputFrequencyBitFieldValue)
                IDTHAL_ReadBitfield(hdlr,
                                    REG_IN1_CNFG,
                                    BF_400HZ_SEL_MASK,
                                    BF_400HZ_SEL_LSB));
    }
    else
    {
        return ((T_idtAmiInputFrequencyBitFieldValue)
                IDTHAL_ReadBitfield(hdlr,
                                    REG_IN2_CNFG,
                                    BF_400HZ_SEL_MASK,
                                    BF_400HZ_SEL_LSB));
    }
}
