/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "Interrupt.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * \brief Set the INT_REQ pin high-Z when the interrupt is inactive
 * \param hdlr Device driver handler
 * \param   nEnable <ul><li>0= INT_REQ output a inactive level</li>
 *                      <li>1= INT_REQ output High-Z</li></ul>
 */
void SetINTpinHz(T_idtDrvHdlr hdlr, T_idtUchar nEnable)
{
    OS_LOGGER(5,
            ("%s dev/%s nEnable/%u\n", __FUNCTION__, hdlr->name_m, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHAL_WriteBitfield(hdlr, REG_INTERRUPT_CNFG, BF_HZ_EN_MASK, BF_HZ_EN_LSB,
            nEnable);
}
/**
 * \brief   Set what level is used to indicated interrupt on INT_REQ pin
 * \param hdlr Device driver handler
 * \param   nLevel <ul><li>0= Low level to indicate a interrupt occurred</li>
 *                     <li>1= High level to indicate a interrupt occurred</li></ul>
 */
void SetINTActiveLevel(T_idtDrvHdlr hdlr, T_idtUchar nLevel)
{
    OS_LOGGER(5,
            ("%s dev/%s nLevel/%u\n", __FUNCTION__, hdlr->name_m, nLevel));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHAL_WriteBitfield(hdlr, REG_INTERRUPT_CNFG, BF_INT_POL_MASK, BF_INT_POL_LSB,
            nLevel);
}

/**
 * \brief   Get 14-bit interrupt status
 * \param hdlr Device driver handler
 * \return  14-bit status of interrupt (one per input)
 */
T_idtUint GetINTStatus(T_idtDrvHdlr hdlr)
{
    T_idtUint uStatus;

    OS_ASSERT_PTR_NULL(hdlr);

    uStatus = (T_idtUint) IDTHal_Read(hdlr, REG_INTERRUPTS1_STS);
    uStatus |= (T_idtUint) (IDTHAL_ReadBitfield(hdlr, REG_INTERRUPTS2_STS,
                            (BF_IN9_MASK |
                             BF_IN10_MASK |
                             BF_IN11_MASK |
                             BF_IN12_MASK |
                             BF_IN13_MASK |
                             BF_IN14_MASK),
                            BF_IN9_LSB) << 8);
    return (uStatus);
}

/**
 * \brief   Clear the corresponding interrupt status
 * \param hdlr Device driver handler
 * \param uINTtoClear IDTHal_Write '1' to corresponding bit to clear the interrupt status
 */
void SetINTStatusToClear(T_idtDrvHdlr hdlr, T_idtUint uINTtoClear)
{

    OS_LOGGER(5,
            ("%s dev/%s uINTtoClear/%u\n", __FUNCTION__, hdlr->name_m, uINTtoClear));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHal_Write(hdlr, REG_INTERRUPTS1_STS,
            (T_idtUchar) (uINTtoClear & 0xFF));
    IDTHAL_WriteBitfield(hdlr, REG_INTERRUPTS2_STS, BF_IN_MSB_MASK, BF_IN_MSB_LSB,
            (uINTtoClear >> 8));
}
/**
 * \brief   Set the corresponding interrupt enable to interrupt or not
 * \param hdlr Device driver handler
 * \param   uMask 14 bit value with each bit corresponding to a input port:
 *                    <ul><li>0= Disable corresponding interrupt to occur</li>
 *                        <li>1= Enable corresponding interrupt to occur</li></ul>
 */
void SetINTEnable(T_idtDrvHdlr hdlr, T_idtUint uMask)
{

    OS_LOGGER(5, ("%s dev/%s uMask/%u\n", __FUNCTION__, hdlr->name_m, uMask));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHal_Write(hdlr, REG_INTERRUPTS1_MASK_CNFG,
            (T_idtUchar) (uMask & BF_IN_LSB_MASK));
    IDTHAL_WriteBitfield(hdlr, REG_INTERRUPTS2_MASK_CNFG, BF_IN_MSB_MASK,
            BF_IN_MSB_LSB, (uMask >> BF_IN_LSB_SIZE));
}
