/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "osNumeric.h"

#include "Define.h"
#include "Hal.h"
#include "Output.h"
#include "DpllCnfg.h"
#include "ApllApi.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/**
 * @brief Enumerated type listing possible output path selections
 */
typedef enum
{
    OutputPathBfVal_T0SonetGigEth = 0, /**< From T0 Sonet/GigE path */
    OutputPathBfVal_T0Dpll_Eth = 3, /**< From T0 DPLL ETH path */
    OutputPathBfVal_T0Dpll_7776kHz = 4, /**< From T0 DPLL 77.76 MHz path */
    OutputPathBfVal_T0Dpll_12E1_GPS_E3_T3 = 5, /**< From T0 DPLL
                                                * 12E1/GPS/E3/T3 path */
    OutputPathBfVal_T0Dpll_16E1_16T1 = 6, /**< From T0 DPLL 16E1/16T1 path */
    OutputPathBfVal_T0Dpll_GSM_OBSAI_16E1_16T1 = 7, /**< From T0 DPLL
                                                * GSM/OBSAI/16E1/16T1
                                                * path */

    OutputPathBfVal_T4SonetGigEth = 8, /**< From T4 Sonet/GigE path */
    OutputPathBfVal_T4Dpll_Eth = 11, /**< From T4 DPLL ETH path */
    OutputPathBfVal_T4Dpll_7776kHz = 12, /**< From T4 DPLL 77.76 MHz path */
    OutputPathBfVal_T4Dpll_12E1_24T1_E3_T3 = 13, /**< From T4 DPLL
                                             * 12E1/E3/T3/24T1 path */
    OutputPathBfVal_T4Dpll_16E1_16T1 = 14, /**< From T4 DPLL 16E1/16T1 path */
    OutputPathBfVal_T4Dpll_GSM_GPS_16E1_16T1 = 15, /**< From T4 DPLL
                                               * GSM/GPS/16E1/16T1
                                               * path */
    OutputPathBfVal_NULL, /**< Not supported */
    OutputPathBfVal_MAX = OutputPathBfVal_NULL /**< \internal Not
                                         * supported */
} T_idtOutputPathBfVal;

/** 
 * @brief Enumerated type supported output port interfaces in bit mask
 * format
 */
enum {
   OUTPUTIF_AMI_MASK = 1<<OUTPUTIF_AMI, /**< AMI Interface mask */
   OUTPUTIF_CMOS_MASK = 1<<OUTPUTIF_CMOS, /**< CMOS Interface mask */
   OUTPUTIF_LVDS_MASK = 1<<OUTPUTIF_LVDS, /**< LVDS Interface mask */
   OUTPUTIF_PECL_MASK = 1<<OUTPUTIF_PECL /**< PECL Interface mask */
};

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
/**
 * @brief Table listing output port configuration registers
 */
static const T_osUint8 IDTOutput_outputRegArray[] =
{ REG_OUT1_FREQ_CNFG, REG_OUT2_FREQ_CNFG, REG_OUT3_FREQ_CNFG,
  REG_OUT4_FREQ_CNFG, REG_OUT5_FREQ_CNFG, REG_OUT6_FREQ_CNFG,
  REG_OUT7_FREQ_CNFG, REG_OUT8_FREQ_CNFG, REG_OUT9_FREQ_CNFG };

/**
 * @brief Table translating external output paths to internal bitfield
 * settings
 */
static const T_idtOutputPathBfVal IDTOutput_outputPathSelectionBfArray[Dpll_MAX][OutputPath_MAX] =
{
    { /* T0 DPLL */
        OutputPathBfVal_T0SonetGigEth, /**< OutputPath_SonetGigE */
        OutputPathBfVal_T0Dpll_7776kHz, /**< OutputPath_7776kHz */
        OutputPathBfVal_T0Dpll_Eth, /**< OutputPath_Eth  */
        OutputPathBfVal_T0Dpll_16E1_16T1, /**< OutputPath_16E1_16T1 */
        OutputPathBfVal_T0Dpll_12E1_GPS_E3_T3, /**< OutputPath_12E1_E3_T3 */
        OutputPathBfVal_T0Dpll_GSM_OBSAI_16E1_16T1, /**< OutputPath_GSM_16E1_16T1 */
        OutputPathBfVal_T0Dpll_12E1_GPS_E3_T3, /** OutputPath_GPS */
        OutputPathBfVal_T0Dpll_GSM_OBSAI_16E1_16T1, /**< OutputPath_OBSAI */
        OutputPathBfVal_NULL /**< OutputPath_24T1 */
    },
    { /* T4 DPLL */
        OutputPathBfVal_T4SonetGigEth, /**< OutputPath_SonetGigE */
        OutputPathBfVal_T4Dpll_7776kHz, /**< OutputPath_7776kHz */
        OutputPathBfVal_T4Dpll_Eth, /**< OutputPath_Eth  */
        OutputPathBfVal_T4Dpll_16E1_16T1, /**< OutputPath_16E1_16T1 */
        OutputPathBfVal_T4Dpll_12E1_24T1_E3_T3, /**< OutputPath_12E1_E3_T3 */
        OutputPathBfVal_T4Dpll_GSM_GPS_16E1_16T1, /**< OutputPath_GSM_16E1_16T1  */
        OutputPathBfVal_T4Dpll_GSM_GPS_16E1_16T1, /**< OutputPath_GPS */
        OutputPathBfVal_NULL,  /**< OutputPath_OBSAI */
        OutputPathBfVal_T4Dpll_12E1_24T1_E3_T3 /**< OutputPath_24T1 */
    } 
};

/**
 * @brief Table to translate between bit field and enumerated type.
 * Used for IDTOutput_GetOutputConfig function
 */
static const T_outputPathType bfVal2EnumPathType_ca[OutputPathBfVal_MAX] =  {
    OutputPath_SonetGigEth, /* OutputPathBfVal_T0SonetGigEth */
    OutputPath_SonetGigEth,
    OutputPath_SonetGigEth,
    OutputPath_Eth, /* OutputPathBfVal_T0Dpll_Eth */
    OutputPath_7776kHz, /* OutputPathBfVal_T0Dpll_7776kHz */
    OutputPath_MAX, /* Special case:
                        OutputPathBfVal_T0Dpll_12E1_GPS_E3_T3 */
    OutputPath_16E1_16T1, /* OutputPathBfVal_T0Dpll_16E1_16T1 */
    OutputPath_MAX, /* Special case:
                        OutputPathBfVal_T0Dpll_GSM_OBSAI_16E1_16T1 */
    OutputPath_SonetGigEth, /* OutputPath_T4Apll */
    OutputPath_SonetGigEth,
    OutputPath_SonetGigEth,
    OutputPath_Eth, /* OutputPathBfVal_T4SonetGigEth */
    OutputPath_7776kHz, /* OutputPathBfVal_T4Dpll_7776kHz */
    OutputPath_MAX, /* OutputPathBfVal_T4Dpll_12E1_24T1_E3_T3  */
    OutputPath_16E1_16T1, /* OutputPathBfVal_T4Dpll_16E1_16T1  */
    OutputPath_MAX /* OutputPathBfVal_T4Dpll_GSM_GPS_16E1_16T1 */
};

/**
 * @brief Table listing supported interfaces per internal output port
 */
static const T_osUint8 IDTOutput_OutputPortInterface[IDT_DRV_MAX_OUTPUT+1] = {
        (0), /**< \internal Not used, 0 */
        (OUTPUTIF_CMOS_MASK), /**< Output Port 1 */
        (OUTPUTIF_CMOS_MASK), /**< Output Port 2 */
        (OUTPUTIF_CMOS_MASK), /**< Output Port 3 */
        (OUTPUTIF_CMOS_MASK), /**< Output Port 4 */
        (OUTPUTIF_CMOS_MASK), /**< Output Port 5 */
        (OUTPUTIF_LVDS_MASK | OUTPUTIF_PECL_MASK), /**< Output Port 6 */
        (OUTPUTIF_LVDS_MASK | OUTPUTIF_PECL_MASK), /**< Output Port 7 */
        (OUTPUTIF_AMI_MASK), /**< Output Port 8 */
        (OUTPUTIF_CMOS_MASK), /**< Output Port 9 */
        (OUTPUTIF_LVDS_MASK | OUTPUTIF_PECL_MASK), /**< Output Port 10 */
        (OUTPUTIF_LVDS_MASK | OUTPUTIF_PECL_MASK) /**< Output Port 11 */
};

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * @brief Set the configuration of Output port
 * @param hdlr Device driver handler
 * @param nOutN Output port that want to be used
 * @param nDpll DPLL instance
 * @param nPath Select a specified path as input of output divider
 * @param nFactor Dividen factor of a certain output divider. Look up
 * the table in datasheet
 * @param apllConfig APLL configuration
 */
void IDTOutput_SetOutputConfig(T_idtDrvHdlr hdlr, T_osUint8 nOutN,
        T_idtDpllInstance nDpll, T_outputPathType nPath, T_osUint8 nFactor, T_idtApllConfigPerOutput *apllConfig)
{
    T_osUint8 internalPort;
    T_idtDpllType dpllType;
    T_osUint8 nIntInd;
    T_idtOutputPathBfVal pathSelection = OutputPathBfVal_NULL;
    T_idtOutputApllConfig outputApll;

    OS_LOGGER(5,
              ("%s dev/%s nOutN/%d nDpll/%d nPath/%d nFactor/%d\n",
               __FUNCTION__, hdlr->name_m, nOutN, nDpll, nPath, nFactor));

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT((nOutN<=IDT_DRV_MAX_OUTPUT)&&(nOutN>0));

    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr, nOutN, internalPort);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    if (APLL_CONFIGURED(outputApll))
    {
        OS_LOGGER(5,
            ("%s dev/%s Apll id/%d apll input/%d apll output/%d external output/%d\n",
             __FUNCTION__, hdlr->name_m, outputApll.apllInst, outputApll.apllInput, outputApll.apllOutput, outputApll.extOutput));

        OS_ASSERT_PTR_NULL(apllConfig);

        if (hdlr->deviceConfig_m->outputFreqValidFunc_m)
        {
            OS_ASSERTS((hdlr->deviceConfig_m->outputFreqValidFunc_m(
                            hdlr, nOutN, OutFreq_MAX, apllConfig)),
                        ("Invalid output frequency for "
                        "output port/%d dev/%s\n",
                        nOutN, hdlr->name_m));
        }
    }

    /* Some translation is required. We need to translate from
     * external path to output path selection bitfield values */
    pathSelection = IDTOutput_outputPathSelectionBfArray[dpllType][nPath];
    OS_ASSERTS(pathSelection != OutputPathBfVal_NULL,
            ("Unsupported path (%d) with selected DPLL/APLL (%d)\n", 
             nPath, nDpll));

    /* From port number to index */
    internalPort--;

    /* Read output configuration register from device */
    nIntInd = IDTHal_Read(hdlr, IDTOutput_outputRegArray[internalPort]);

    /* For the majority of outputs - they are identical except for
     * ports 8/9 */
    if (internalPort < 7)
    {
        OS_ASSERT(nFactor<(1<<BF_OUT1_DIVIDER_SIZE));

        /* Output ports 1-7
         *   Same bit fields for port 1-7 - we'll just use port 1
         *   mask/lsb defines */
        IDTHAL_ModifyBitfield(nIntInd, BF_OUT1_PATH_SEL_MASK,
                BF_OUT1_PATH_SEL_LSB, (T_osUint8) pathSelection);

        IDTHAL_ModifyBitfield(nIntInd, BF_OUT1_DIVIDER_MASK,
                BF_OUT1_DIVIDER_LSB, nFactor);
    }
    else
    {
        T_osUint8 pathSel;
        if (internalPort == 7) /* Output port 8 */
        {
            OS_ASSERT(nPath==OutputPath_7776kHz);

            if (dpllType == Dpll_T0)
                pathSel = 1;
            else if (dpllType == Dpll_T4)
                pathSel = 0;
            else
                OS_ASSERTS(0,("Output %d does not support "
                               "path %d using DPLL/APLL %d",
                               nOutN,nPath,dpllType));

            IDTHAL_ModifyBitfield(nIntInd, 
                                  BF_OUT8_PATH_SEL_MASK, 
                                  BF_OUT8_PATH_SEL_LSB,
                                  pathSel);
        }

        if (internalPort == 8) /* Output port 9 */
        {
            OS_ASSERT(nPath==OutputPath_16E1_16T1);

            if (dpllType == Dpll_T0)
                pathSel = 1;
            else if (dpllType == Dpll_T4)
                pathSel = 0;
            else
                OS_ASSERTS(0,("Output %d does not support "
                               "path %d using DPLL/APLL %d",
                               nOutN,nPath,dpllType));

            IDTHAL_ModifyBitfield(nIntInd, 
                                  BF_OUT9_PATH_SEL_MASK, 
                                  BF_OUT9_PATH_SEL_LSB,
                                  pathSel);
        }

        /* No output divider for these ports */
    }

    /* Write contents of modified register value back to device */
    IDTHal_Write(hdlr, IDTOutput_outputRegArray[internalPort], nIntInd);

    /* Check if need to configure the APLL */
    if (APLL_CONFIGURED(outputApll))
    {
        IDTApll_SetApllConfigPerOutput(hdlr, outputApll.apllInst, outputApll.apllOutput, apllConfig);
    }

    OS_LOGGER(9,
            ("%s dev/%s nOutN/%d nDpll/%d nPath/%d nFactor/%d - DONE!\n", __FUNCTION__, hdlr->name_m, nOutN, nDpll, nPath, nFactor));
}

static void IDTOutput_TranslateBfToEnumType(T_idtDrvHdlr      hdlr, 
                                            T_osUint8         bfVal,
                                            T_outputPathType *nPath_p)
{
    T_idtDpllInstance nDpll;

    OS_ASSERT(bfVal<OutputPathBfVal_MAX);

    /* Ok - we need to handle this special cases first
     *
     * Really - it's because of how we define the enumerated type. A
     * bit field can map to multiple enumerated types. We have to do
     * some additional look up (output mux) to determine the correct
     * enumerated type.
     */
    switch (bfVal)
    {
    case OutputPathBfVal_T0Dpll_12E1_GPS_E3_T3:
    {
        T_idtDpllOutputSelectorB selectorB;

        /* Need further information to determine whether it's
         * 12E1_E3_T3 or GPS */
        nDpll = hdlr->deviceConfig_m->pllInt2ExtXlate_ma[Dpll_T0];
        selectorB = IDTDpll_GetDpllOutputPathSelectorB(hdlr, nDpll);
        if (selectorB == DPLLOutputSelB_GPS)
            *nPath_p = OutputPath_GPS;
        else
            *nPath_p = OutputPath_12E1_E3_T3;
        break;
    }

    case OutputPathBfVal_T0Dpll_GSM_OBSAI_16E1_16T1:
    {
        T_idtDpllOutputSelectorA selectorA;

        /* Need further information to determine whether it's
         * GSM_16E1_16T1 or OBSAI */
        nDpll = hdlr->deviceConfig_m->pllInt2ExtXlate_ma[Dpll_T0];
        selectorA = IDTDpll_GetDpllOutputPathSelectorA(hdlr, nDpll);
        if (selectorA == DPLLOutputSelA_OBSAI)
            *nPath_p = OutputPath_OBSAI;
        else
            *nPath_p = OutputPath_GSM_16E1_16T1;
        break;
    }

    case OutputPathBfVal_T4Dpll_12E1_24T1_E3_T3:
    {
        T_idtDpllOutputSelectorB selectorB;

        /* Need further information to determine whether it's
         * 12E1_E3_T3 or 24T1 */
        nDpll = hdlr->deviceConfig_m->pllInt2ExtXlate_ma[Dpll_T4];
        selectorB = IDTDpll_GetDpllOutputPathSelectorB(hdlr, nDpll);
        if (selectorB == DPLLOutputSelB_24T1)
            *nPath_p = OutputPath_24T1;
        else
            *nPath_p = OutputPath_12E1_E3_T3;
        break;
    }

    case OutputPathBfVal_T4Dpll_GSM_GPS_16E1_16T1:
    {
        T_idtDpllOutputSelectorA selectorA;

        /* Need further information to determine whether it's
         * GSM_16E1_16T1 or GPS */
        nDpll = hdlr->deviceConfig_m->pllInt2ExtXlate_ma[Dpll_T4];
        selectorA = IDTDpll_GetDpllOutputPathSelectorA(hdlr, nDpll);
        if (selectorA == DPLLOutputSelA_GPS)
            *nPath_p = OutputPath_GPS;
        else
            *nPath_p = OutputPath_GSM_16E1_16T1;
        break;
    }

    default:
    {
        /* For simple types we just use look up table (see above) */
        *nPath_p = bfVal2EnumPathType_ca[bfVal];
        OS_ASSERTS((*nPath_p != OutputPath_MAX),
                   ("%s:%d Unhandled bitfield value %d",
                    __FILE__,__FUNCTION__,bfVal));
        break;
    }

    }
}

/**
 * @brief Get the configuration of Output port
 * @param hdlr Device driver handler
 * @param nOutN Output port that want to be used
 * @param nDpll_p DPLL instance
 * @param nPath_p Select a specified path as input of output divider
 * @param nFactor_p Dividen factor of a certain output divider. Look up
 * the table in datasheet
 * @param apllConfig APLL configuration. Set to NULL if the output port does not go through APLL.
 */
void IDTOutput_GetOutputConfig(T_idtDrvHdlr              hdlr, 
                               T_osUint8                 nOutN,
                               T_idtDpllInstance        *nDpll_p, 
                               T_outputPathType         *nPath_p, 
                               T_osUint8                *nFactor_p, 
                               T_idtApllConfigPerOutput *apllConfig)
{
    T_osUint8 internalPort;
    T_osUint8 nIntInd;
    T_osUint8 bfVal;
    T_idtDpllType dpllType;
    T_idtOutputApllConfig outputApll;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT_PTR_NULL(nDpll_p);
    OS_ASSERT_PTR_NULL(nPath_p);
    OS_ASSERT_PTR_NULL(nFactor_p);
    OS_ASSERT_PTR_NULL(apllConfig);
    OS_ASSERT((nOutN<=IDT_DRV_MAX_OUTPUT)&&(nOutN>0));
    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr, nOutN, internalPort);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    OS_LOGGER(5,
              ("%s dev/%s Apll chip Id/%d Apll input/%d "
               "Apll output/%d external output/%d\n",
               __FUNCTION__, hdlr->name_m, outputApll.apllInst,
               outputApll.apllInput, outputApll.apllOutput, 
               outputApll.extOutput));

    *nDpll_p = DPLL_INSTANCE_MAX;
    *nPath_p = OutputPath_MAX;
    *nFactor_p = 0;

    /* From port number to index */
    internalPort--;

    /* Read output configuration register from device */
    nIntInd = IDTHal_Read(hdlr, IDTOutput_outputRegArray[internalPort]);

    /* For the majority of outputs - they are identical except for
     * ports 8/9 */
    if (internalPort < 7)
    {
        /* Output ports 1-7
         *   Same bit fields for port 1-7 - we'll just use port 1
         *   mask/lsb defines */
        bfVal = IDTHAL_GetBitfield(nIntInd, 
                                   BF_OUT1_PATH_SEL_MASK,
                                   BF_OUT1_PATH_SEL_LSB);

        /* Translate bf value to enumerated type */
        IDTOutput_TranslateBfToEnumType(hdlr, bfVal, nPath_p);

        dpllType = (bfVal < 8) ? Dpll_T0 : Dpll_T4;

        *nFactor_p = IDTHAL_GetBitfield(nIntInd, 
                                        BF_OUT1_DIVIDER_MASK,
                                        BF_OUT1_DIVIDER_LSB);
    }
    else
    {
        if (internalPort == 7) /* Output port 8 */
        {
            *nPath_p = OutputPath_7776kHz;
            dpllType = IDTHAL_GetBitfield(nIntInd, 
                                          BF_OUT8_PATH_SEL_MASK,
                                          BF_OUT8_PATH_SEL_LSB)? 
                Dpll_T0 : Dpll_T4;
        }

        if (internalPort == 8) /* Output port 9 */
        {
            *nPath_p = OutputPath_16E1_16T1;
            dpllType = IDTHAL_GetBitfield(nIntInd, 
                                          BF_OUT9_PATH_SEL_MASK,
                                          BF_OUT9_PATH_SEL_LSB)?
                Dpll_T0 : Dpll_T4;
        }
       
        /* No divider, therefore 1 */
        *nFactor_p = 1;
    }

    /* Translate from DPLL type to PLL instance */
    *nDpll_p = hdlr->deviceConfig_m->pllInt2ExtXlate_ma[dpllType];

    if (APLL_CONFIGURED(outputApll))
    {
        IDTApll_GetApllConfigPerOutput(hdlr, 
                                       outputApll.apllInst, 
                                       outputApll.apllOutput, 
                                       apllConfig);
    }
}

/**
 * @brief Set the specified output port to generate a invert signal
 * @param hdlr Device driver handler
 * @param nOutN	Output port that want to be used
 * @param invert 0-do no invert, 1-invert output signal
 */
void IDTOutput_SetOutputInvert(T_idtDrvHdlr hdlr, T_osUint8 nOutN, T_osUint8 invert)
{
	T_osUint8 internalPort;
	T_osUint8 regOff;
	T_osUint8 bfLsb;
	T_osUint8 bfMask;
    T_idtOutputApllConfig outputApll;

	OS_LOGGER(5,
		   ("%s dev/%s nOutN/%d invert/%d\n", __FUNCTION__, hdlr->name_m, nOutN, invert));

	OS_ASSERT_PTR_NULL(hdlr);
	OS_ASSERT((nOutN<=IDT_DRV_MAX_OUTPUT)&&(nOutN>0));

	IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr,nOutN,internalPort);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    OS_LOGGER(5,
               ("%s dev/%s Apll chip Id/%d Apll input/%d Apll output/%d external output/%d\n",
                __FUNCTION__, hdlr->name_m, outputApll.apllInst, outputApll.apllInput, outputApll.apllOutput, outputApll.extOutput));

    /* Skip APLL output */
    if (APLL_CONFIGURED(outputApll))
    {
        OS_LOGGER(5, ("APLL output path, no inversion\n"));
        return;
    }

	if (internalPort <= 5)
	{
		regOff = REG_OUT9_FREQ_CNFG;
		bfLsb = internalPort - 1;
	}
	else
	{
		regOff = REG_OUT8_FREQ_CNFG;
		bfLsb = internalPort - 6;
	}
	bfMask = 1 << (bfLsb);

	IDTHAL_WriteBitfield(hdlr, regOff, bfMask, bfLsb, invert);
}

/**
 * @brief Get output signal inversion status
 * @param hdlr Device driver handler
 * @param nOutN Output port that want to be used
 * @return 0-do no invert, 1-invert output signal
 */
T_osUint8 IDTOutput_GetOutputInvert(T_idtDrvHdlr hdlr, T_osUint8 nOutN)
{
    T_osUint8 internalPort;
    T_osUint8 regOff;
    T_osUint8 bfLsb;
    T_osUint8 bfMask;
    T_idtOutputApllConfig outputApll;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT((nOutN<=IDT_DRV_MAX_OUTPUT)&&(nOutN>0));
    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr,nOutN,internalPort);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    if (APLL_CONFIGURED(outputApll))
    {
        OS_LOGGER(5, ("APLL output path, no inversion\n"));
        return 0;
    }

    if (internalPort <= 5)
    {
        regOff = REG_OUT9_FREQ_CNFG;
        bfLsb = internalPort - 1;
    }
    else
    {
        regOff = REG_OUT8_FREQ_CNFG;
        bfLsb = internalPort - 6;
    }
    bfMask = 1 << (bfLsb);

    return IDTHAL_ReadBitfield(hdlr, regOff, bfMask, bfLsb);
}

/**
 * @brief Enable/Disable output port. Consult data sheet for supported ports.
 * @param hdlr Device driver handler
 * @param nOutN Output port that want to be used
 * @param   nEnable  <ul><li>0=Disable port</li>
 *                       <li>1=Enable port</li></ul>
 */
void IDTOutput_SetOutputEnable(T_idtDrvHdlr hdlr, T_osUint8 nOutN, T_osUint8 nEnable)
{
    T_osUint8 internalPort;
    T_idtOutputApllConfig outputApll;

    OS_LOGGER(5,
           ("%s dev/%s nOutN/%d nEnable/%d\n", __FUNCTION__, hdlr->name_m, nOutN, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr,nOutN,internalPort);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    /* From port number to index */
    internalPort--;

    if (internalPort == 7) /* Output port 8 */
        IDTHAL_WriteBitfield(hdlr, REG_OUT8_FREQ_CNFG,
                             BF_OUT8_EN_MASK, BF_OUT8_EN_LSB,
                             nEnable);
    else if (internalPort == 8) /* Output port 9 */
        IDTHAL_WriteBitfield(hdlr, REG_OUT9_FREQ_CNFG,
                             BF_OUT9_EN_MASK, BF_OUT9_EN_LSB,
                             nEnable);
    else if (APLL_CONFIGURED(outputApll))
    {
        T_idtApllOutputEn enable = nEnable ? APLL_OUTPUT_ENABLE : APLL_OUTPUT_DISABLE;
        IDTApll_SetOutputEnState(hdlr, outputApll.apllInst, outputApll.apllOutput, enable);
    }
    else
    {
        /* When disable, set output divider to 0.
                   Skip enable, as the output divider needs to be
                   set via IDTOutput_SetOutputConfig function or
                   IDTHlApi_ConfigureOutput */
        if (!nEnable)
        {
            IDTHAL_WriteBitfield(hdlr,
                                 IDTOutput_outputRegArray[internalPort], 
                                 BF_OUT1_DIVIDER_MASK,
                                 BF_OUT1_DIVIDER_LSB,
                                 0);
        }
        else
        {
            OS_LOGGER(0, ("Enable operation not supported for output port %d," 
                        " use IDTHlApi_ConfigureOutput or IDTOutput_SetOutputConfig" 
                        " to configure the output port\n", nOutN));
        }
    }
}

/**
 * @brief Get enable/disable status of output port. Consult data sheet for supported ports.
 * @param hdlr Device driver handler
 * @param nOutN Output port that want to be used
 * @return   nEnable  <ul><li>0=Disable port</li>
 *                       <li>1=Enable port</li></ul>
 */
T_osUint8 IDTOutput_GetOutputEnable(T_idtDrvHdlr hdlr, T_osUint8 nOutN)
{
    T_osUint8 internalPort;
    T_idtOutputApllConfig outputApll;

    OS_ASSERT_PTR_NULL(hdlr);

    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr,nOutN,internalPort);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    OS_LOGGER(5,
            ("%s dev/%s Apll chip Id/%d Apll input/%d Apll output/%d external output/%d\n",
             __FUNCTION__, hdlr->name_m, outputApll.apllInst, outputApll.apllInput, outputApll.apllOutput, outputApll.extOutput));

    /* From port number to index */
    internalPort--;

    if (internalPort == 7) /* Output port 8 */ 
        return IDTHAL_ReadBitfield(hdlr, REG_OUT8_FREQ_CNFG, BF_OUT8_EN_MASK, BF_OUT8_EN_LSB);
    else if (internalPort == 8) /* Output port 9 */
        return IDTHAL_ReadBitfield(hdlr, REG_OUT9_FREQ_CNFG, BF_OUT9_EN_MASK, BF_OUT9_EN_LSB);
    else if (APLL_CONFIGURED(outputApll))
    {
        if (IDTApll_GetOutputEnState(hdlr, outputApll.apllInst, outputApll.apllOutput) == APLL_OUTPUT_ENABLE) 
            return 1;
        else
            return 0;
    }
	else
    {
        T_osUint8 bfVal;

        /* The rest of ports has the same divider mask and offset, so
         * just use output 1 values */
        bfVal = IDTHAL_ReadBitfield(hdlr, 
                                    IDTOutput_outputRegArray[internalPort], 
                                    BF_OUT1_DIVIDER_MASK, 
                                    BF_OUT1_DIVIDER_LSB);

        /* Other ports don't have an enable per se but have special
         * values for their output dividers which squelch the signal
         * high or low */
        return ((bfVal==0)||(bfVal==15))?0:1;
    }
}

/**
 * @brief Set output port interface type. Consult data sheet for
 * supported interfaces per port
 * @param hdlr Device driver handler
 * @param nOutN Output port number
 * @param outIf Interface type.
 */
void IDTOutput_SetOuputpuInterface(T_idtDrvHdlr hdlr, T_osUint8 nOutN,
        T_outputInterface outIf)
{
    T_osUint8 internalPort;
    T_osUint8 nIntInd;
    T_osUint8 interfaceMask;
    T_idtOutputApllConfig outputApll;

    OS_LOGGER(5, ("%s dev/%s nOutN/%d interface/%d\n",
                   __FUNCTION__, hdlr->name_m, nOutN, outIf));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(outIf<OUTPUTIF_MAX);

    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr, nOutN, internalPort);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    OS_LOGGER(5,
            ("%s dev/%s Apll chip Id/%d Apll input/%d Apll output/%d external output/%d\n",
             __FUNCTION__, hdlr->name_m, outputApll.apllInst, outputApll.apllInput, outputApll.apllOutput, outputApll.extOutput));

    /* Convert interface to mask format */
    interfaceMask = 1<<outIf;

    /* Check if we support interface */
    OS_ASSERTS(interfaceMask & IDTOutput_OutputPortInterface[internalPort],
                ("Output port %d does not support interface type %d\n",
                 nOutN,outIf));

    if (internalPort == 6)
    {
        nIntInd = IDTHal_Read(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG);
        switch (outIf)
        {
        case OUTPUTIF_LVDS:
            IDTHAL_ModifyBitfield(nIntInd, BF_OUT6_PECL_LVDS_MASK,
                    BF_OUT6_PECL_LVDS_LSB, 0);
            break;
        case OUTPUTIF_PECL:
            IDTHAL_ModifyBitfield(nIntInd, BF_OUT6_PECL_LVDS_MASK,
                    BF_OUT6_PECL_LVDS_LSB, 1);
            break;
        default:
            OS_ASSERTS(0,
                    ("Output port %d does not support interface type %d\n", nOutN, outIf));
            return;
            break;
        }
        IDTHal_Write(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG, nIntInd);
    }

    if (internalPort == 7)
    {
        nIntInd = IDTHal_Read(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG);
        switch (outIf)
        {
        case OUTPUTIF_LVDS:
            IDTHAL_ModifyBitfield(nIntInd, BF_OUT7_PECL_LVDS_MASK,
                    BF_OUT7_PECL_LVDS_LSB, 0);
            break;
        case OUTPUTIF_PECL:
            IDTHAL_ModifyBitfield(nIntInd, BF_OUT7_PECL_LVDS_MASK,
                    BF_OUT7_PECL_LVDS_LSB, 1);
            break;
        default:
            OS_ASSERTS(0,
                    ("Output port %d does not support interface type %d\n", nOutN, outIf));
            return;
            break;
        }
        IDTHal_Write(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG, nIntInd);
    }

    if (APLL_CONFIGURED(outputApll))
    {
        T_idtApllOutputSigType apllInterface;
        apllInterface = (outIf == OUTPUTIF_LVDS) ? APLL_OUT_SIG_LVDS : APLL_OUT_SIG_LVPECL;
        IDTApll_SetOutputSigType(hdlr, outputApll.apllInst, outputApll.apllOutput, apllInterface);
    }
}

/**
 * @brief Get output port interface type. Consult data sheet for
 * supported interfaces per port
 * @param hdlr Device driver handler
 * @param nOutN Output port number
 * @return Interface type.
 */
T_outputInterface IDTOutput_GetOuputpuInterface(T_idtDrvHdlr hdlr, T_osUint8 nOutN)
{
    T_osUint8 internalPort;
    T_osUint8 nIntInd;
    T_idtOutputApllConfig outputApll;

    OS_LOGGER(5, ("%s dev/%s nOutN/%u\n", __FUNCTION__, hdlr->name_m, nOutN));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr, nOutN, internalPort);
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    OS_LOGGER(5,
            ("%s dev/%s Apll chip Id/%d Apll input/%d Apll output/%d external output/%d\n",
             __FUNCTION__, hdlr->name_m, outputApll.apllInst, outputApll.apllInput, outputApll.apllOutput, outputApll.extOutput));

    if (APLL_CONFIGURED(outputApll))
    {
        T_idtApllOutputSigType apllInterface;
        apllInterface = IDTApll_GetOutputSigType(hdlr, outputApll.apllInst, outputApll.apllOutput);
        
        if (APLL_OUT_SIG_LVPECL == apllInterface)
            return OUTPUTIF_PECL;
        else if (APLL_OUT_SIG_LVDS == apllInterface)
            return OUTPUTIF_LVDS;
        else
            return OUTPUTIF_MAX;
    } 

    if ((internalPort < 6) || (internalPort == 9))
    {
        /* For internal ports, 1-5 & 9, they only support CMOS interface */
        return OUTPUTIF_CMOS;
    }

    if (internalPort == 8)
    {
        /* For internal port 8, it only supports AMI */
        return OUTPUTIF_AMI;
    }

    if (internalPort == 6)
    {
        nIntInd = IDTHal_Read(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG);
        return ((IDTHAL_GetBitfield(nIntInd, BF_OUT6_PECL_LVDS_MASK,
                BF_OUT6_PECL_LVDS_LSB))?OUTPUTIF_PECL:OUTPUTIF_LVDS);
    }

    if (internalPort == 7)
    {
        nIntInd = IDTHal_Read(hdlr, REG_DIFFERENTIAL_IN_OUT_CNFG);
        return ((IDTHAL_GetBitfield(nIntInd, BF_OUT7_PECL_LVDS_MASK,
                BF_OUT7_PECL_LVDS_LSB))?OUTPUTIF_PECL:OUTPUTIF_LVDS);
    }

    /* Should never get here */
    return OUTPUTIF_MAX;
}

/**
 * @brief Function to control output frame syncs. 
 * @param hdlr Device driver handler
 * @param frameSync Select between FR and MFR frame syncs
 * @param enable Enable frame sync
 * @param invert Invert output frame sync (0-noninverted, 1-invert)
 * @param pulse Select between clock (50:50 clock cycle) or pulse
 * (dictated by IN3) (0-clock, 1-pulse)
 */
void IDTOutput_SetFrameSync(T_idtDrvHdlr hdlr,
                            T_frameSync frameSync,
                            T_osUint8 enable,
                            T_osUint8 invert,
                            T_osUint8 pulse)
{
    T_osUint8 regVal;

    OS_LOGGER(5, ("%s dev/%s frameSync/%d enable/%d"
                   "invert/%d pulse/%d\n", __FUNCTION__, 
                   hdlr->name_m,
                   frameSync,
                   enable,
                   invert,
                   pulse));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(frameSync<frameSync_MAX);
    OS_ASSERT(enable<2);
    OS_ASSERT(invert<2);
    OS_ASSERT(pulse<2);

    regVal = IDTHal_Read(hdlr, REG_FR_MFR_SYNC_CNFG);
    if (frameSync == frameSync_FRSYNC)
        {
            IDTHAL_ModifyBitfield(regVal, 
                                  BF_8K_EN_MASK, BF_8K_EN_LSB, 
                                  enable);
            IDTHAL_ModifyBitfield(regVal, 
                                  BF_8K_INV_MASK, BF_8K_INV_LSB,
                                  invert);
            IDTHAL_ModifyBitfield(regVal, 
                                  BF_8K_PUL_MASK, BF_8K_PUL_LSB,
                                  pulse);
        }
    else
        {
            /* MFR Sync */
            IDTHAL_ModifyBitfield(regVal, 
                                  BF_2K_EN_MASK, BF_2K_EN_LSB, 
                                  enable);
            IDTHAL_ModifyBitfield(regVal, 
                                  BF_2K_INV_MASK, BF_2K_INV_LSB,
                                  invert);
            IDTHAL_ModifyBitfield(regVal, 
                                  BF_2K_PUL_MASK, BF_2K_PUL_LSB,
                                  pulse);
        }
    IDTHal_Write(hdlr, REG_FR_MFR_SYNC_CNFG, regVal);
}

/**
 * @brief Function to retrieve output frame sync configuration.
 * @param hdlr Device driver handler
 * @param frameSync Select between FR and MFR frame syncs
 * @param enable_p Enable frame sync
 * @param invert_p Invert output frame sync (0-noninverted, 1-invert)
 * @param pulse_p Select between clock (50:50 clock cycle) or pulse
 * (dictated by IN3) (0-clock, 1-pulse)
 */
void IDTOutput_GetFrameSync(T_idtDrvHdlr hdlr,
                            T_frameSync frameSync,
                            T_osUint8 *enable_p,
                            T_osUint8 *invert_p,
                            T_osUint8 *pulse_p)
{
    T_osUint8 regVal;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT_PTR_NULL(enable_p);
    OS_ASSERT_PTR_NULL(invert_p);
    OS_ASSERT_PTR_NULL(pulse_p);

    regVal = IDTHal_Read(hdlr, REG_FR_MFR_SYNC_CNFG);
    if (frameSync == frameSync_FRSYNC)
        {
            *enable_p = IDTHAL_GetBitfield(regVal, 
                                           BF_8K_EN_MASK, BF_8K_EN_LSB);
            *invert_p = IDTHAL_GetBitfield(regVal, 
                                           BF_8K_INV_MASK, BF_8K_INV_LSB);
            *pulse_p = IDTHAL_GetBitfield(regVal, 
                                          BF_8K_PUL_MASK, BF_8K_PUL_LSB);
        }
    else
        {
            /* MFR Sync */
            *enable_p = IDTHAL_GetBitfield(regVal, 
                                           BF_2K_EN_MASK, BF_2K_EN_LSB);
            *invert_p = IDTHAL_GetBitfield(regVal, 
                                           BF_2K_INV_MASK, BF_2K_INV_LSB);
            *pulse_p = IDTHAL_GetBitfield(regVal, 
                                          BF_2K_PUL_MASK, BF_2K_PUL_LSB);
        }
 }

/**
 * @brief Function provision trigger for frame sync pulses. 
 *
 * Only valid for frame syncs when provisioned for pulse.
 *
 * @param hdlr Device driver handler
 * @param isRisingEdge Frame sync pulse triggered by rising edge.
 */
void IDTOutput_SetFrameSyncPulseEdge(T_idtDrvHdlr hdlr,
                                     T_osUint8   isRisingEdge)
{
    OS_LOGGER(5, ("%s dev/%s isRisingEdge/%d\n", __FUNCTION__,
                   hdlr->name_m,isRisingEdge));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(isRisingEdge<2);

    IDTHAL_WriteBitfield(hdlr, 
                         REG_FR_MFR_SYNC_CNFG,
                         BF_2K_8K_PUL_POSITION_MASK,
                         BF_2K_8K_PUL_POSITION_LSB,
                         isRisingEdge);
}

/**
 * @brief Function retrieve trigger for frame sync pulses. 
 *
 * Only valid for frame syncs when provisioned for pulse.
 *
 * @param hdlr Device driver handler
 * @return Frame sync pulse triggered by rising edge (0-falling edge,
 * 1-rising edge)
 */
T_osUint8 IDTOutput_GetFrameSyncPulseEdge(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return IDTHAL_ReadBitfield(hdlr, 
                               REG_FR_MFR_SYNC_CNFG,
                               BF_2K_8K_PUL_POSITION_MASK,
                               BF_2K_8K_PUL_POSITION_LSB);
}

/**
 * @brief Function disables APLL input
 *
 * Only valid for output path has APLL configured
 *
 * @param hdlr Device driver handler
 * @param outputApllConfig APLL output path configuration
 */
void IDTOutput_DisableApllInput(T_idtDrvHdlr hdlr,
                                T_idtOutputApllConfig outputApllConfig)
{
    OS_LOGGER(5, ("%s dev/%s nApllInput/%u\n", __FUNCTION__, hdlr->name_m, outputApllConfig.apllInput));

    OS_ASSERT_PTR_NULL(hdlr);

    if (APLL_CONFIGURED(outputApllConfig))
    {
        /* Use out1 mask and lsb value, since they are the same for all outputs */
        IDTHAL_WriteBitfield(hdlr,
                            IDTOutput_outputRegArray[outputApllConfig.apllInput-1],
                            BF_OUT1_DIVIDER_MASK,
                            BF_OUT1_DIVIDER_LSB,
                            0);
    }
}

/**
 * @brief Function disables all internal outputs
 *
 * Should ONLY be called from device init function
 * to disable internal outputs for better jitter performance
 *
 * @param hdlr Device driver handler
 */
void IDTOutput_DisableAllIntOutput(T_idtDrvHdlr hdlr)
{
    T_osUint8 i;

    OS_LOGGER(5, ("%s dev/%s\n", __FUNCTION__, hdlr->name_m));

    OS_ASSERT_PTR_NULL(hdlr);

    for (i = 0; i < sizeof(IDTOutput_outputRegArray); i++)
    {
        if (REG_OUT8_FREQ_CNFG == IDTOutput_outputRegArray[i])
        {
            IDTHAL_WriteBitfield(hdlr, REG_OUT8_FREQ_CNFG,
                             BF_OUT8_EN_MASK, BF_OUT8_EN_LSB,
                             0);
        }
        else if (REG_OUT9_FREQ_CNFG == IDTOutput_outputRegArray[i])
        {
            IDTHAL_WriteBitfield(hdlr, REG_OUT9_FREQ_CNFG,
                             BF_OUT9_EN_MASK, BF_OUT9_EN_LSB,
                             0);
        }
        else
        {
            IDTHAL_WriteBitfield(hdlr,
                                 IDTOutput_outputRegArray[i],
                                 BF_OUT1_DIVIDER_MASK,
                                 BF_OUT1_DIVIDER_LSB,
                                 0);
        }
    }
}

