/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <check.h>

#include "Define.h"
#include "osNumeric.h"
#include "Input.h"
#include "General.h"
#include "inputFreq.h"

#include "check_main.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

START_TEST (test_ConfigureInput1PPS)
{
    IDTHlApi_ConfigureInput1PPS(hdlr,_i);
}
END_TEST

/**
 * @brief Quick and dirty function to calculate input frequency from device 
 * @param hdlr Device driver handle 
 * @param nInputNo input port number 
 * @return calculated frequency
 */
static T_osUint32 calculateFrequency (T_idtDrvHdlr hdlr,
                                      T_osUint8    nInputNo)
{
    T_osUint16 nFactor;
    T_idtInputDividerMux nDivider;
    T_idtInputFrequencyBitfieldValue nFrequency;

    T_osUint8 internalPort;

    T_osUint32 calcFrequency = 0;

    IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr,nInputNo,internalPort);

    nFactor = IDTInput_GetDivisionFactor(hdlr, nInputNo);
    nDivider = IDTInput_GetPreDivider(hdlr, nInputNo);

    switch (nDivider)
    {
    case Divider_Bypass:
        calcFrequency = 1;
        break;

    case Divider_DivN:
        calcFrequency = nFactor+1;
        break;

    default:
        OS_ERROR(("Unsupported input divider type %d\n",
                  nDivider));
        return 0;
        break;
    }

    nFrequency = IDTInput_GetInputFrequency(hdlr, nInputNo);
    switch (nFrequency)
    {	
    case Freq_8K:
        calcFrequency *= 8000;
        break;
	case Freq_1544M:
        {
            networkType_t networkType;

            networkType = IDTGeneral_GetNetworkType(hdlr);
            if (networkType == Network_SONET)
            {
                calcFrequency *= 1544000;
            }
            if (networkType == Network_SDH)
            {
                calcFrequency *= 2048000;
            }
            break;
        }
	case Freq_648M:
        calcFrequency *= 6480000;
        break;
	case Freq_1944M:
        calcFrequency *= 19440000;
        break;
	case Freq_2592M:
        calcFrequency *= 25920000;
        break;
	case Freq_3888M:
        calcFrequency *= 38880000;
        break;
	case Freq_2K:
        calcFrequency *= 2000;
        break;
	case Freq_4K:
        calcFrequency *= 4000;
        break;
	case Freq_1PPS:
        calcFrequency *= 1;
        break;
	case Freq_625M:
        calcFrequency *= 6250000;
        break;
	case Freq_10M:
        calcFrequency *= 10000000;
        break;
    default:
        OS_ERROR(("Unsupported input DPLL frequency %d\n",
                  nFrequency));
        return 0;
        break;
    }

    if ((internalPort == 5) || (internalPort == 6))
    {
        T_idtHighFreqencyDivisor nUsage;

        nUsage = IDTInput_GetInputHFdivider(hdlr, nInputNo);
        switch (nUsage)
        {
        case HF_Bypass:
            break;
        case HF_Divide_4:
            calcFrequency *= 4;
            break;
        case HF_Divide_5:
            calcFrequency *= 5;
            break;
        default:
            OS_ERROR(("Unsupported high frequency divider %d\n",
                      nUsage));
            return 0;
            break;
        }
    }

    return calcFrequency;
}

START_TEST (test_ConfigureInputFrequency)
{
    T_osUint16 i;
    static const T_osUint32 frequencies[] = {
        2000,
        4000,
        8000,
        1544000,
        2048000,
        6250000,
        6480000,
        19440000,
        25920000,
        38880000,
        51840000,
        77760000,
        155520000,
        25000000,
        125000000,
        156250000,
        311040000,
        312500000,
        625000000,
        622080000,
    };

    /* Arbitarily pick a port. */
    for (i=0; i<sizeof(frequencies)/sizeof(T_osUint32); i++) {
        T_osUint32 calcFrequency;

        /* Skip invalid input frequency */
        if (hdlr->deviceConfig_m->inputFreqValidFunc_m) {
            if (!hdlr->deviceConfig_m->inputFreqValidFunc_m(hdlr, _i, frequencies[i], NULL))
                continue;
        }

        /* Provision input port */
        IDTHlApi_ConfigureInputFrequency(hdlr, 
                                         _i,  
                                         (frequencies[i]/1000));

        /* Verify using core API */
        calcFrequency = calculateFrequency(hdlr, _i);

        fail_unless(calcFrequency == frequencies[i],
                    "IDTHlApi_ConfigureInputFrequency prov/%ld calc/%ld\n",
                    frequencies[i], calcFrequency);
    }
}
END_TEST

START_TEST (test_DisableAllInputPriorities)
{
    IDTHlApi_DisableAllInputPriorities(hdlr);
}
END_TEST


Suite *InputFreq_suite (void)
{
    T_osUint8 i;
    Suite *s = suite_create ("InputFreq");

    /* General test case */
    TCase *tc_input = tcase_create ("InputFreq");

    tcase_add_checked_fixture (tc_input,
                               initalizeDriver, 
                               deinitilizeDriver);

    for (i=1; i<IDT_DRV_MAX_INPUT; i++)
    {
        T_osUint8 internalPort;

        internalPort = hdlr->deviceConfig_m->inputExt2IntXlate_ma[i];
        if (internalPort != 0)
        {
            if (internalPort > 2)
            {
                if (hdlr->deviceConfig_m->inputFreqValidFunc_m) {
                    if (hdlr->deviceConfig_m->inputFreqValidFunc_m(hdlr, i, 1, NULL))
                        tcase_add_loop_test (tc_input, 
                                             test_ConfigureInput1PPS,
                                             i,i+1);
                }

                tcase_add_loop_test (tc_input, 
                                     test_ConfigureInputFrequency,
                                     i,i+1);
            }
        }
        else 
        {
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_ConfigureInput1PPS,
                                              6, i,i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_ConfigureInputFrequency,
                                              6, i,i+1);
        }
    }

    tcase_add_test (tc_input, test_DisableAllInputPriorities);

    suite_add_tcase (s, tc_input);

    return s;
}


