/*********************************************************************
 * Copyright (c) 2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 *********************************************************************/

#ifndef OSADAPT_H_
#define OSADAPT_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/**
 * \file osAdapt.h
 *
 * This file contains define macros that IDT software uses to
 * access various system features.
 *
 * Customers must provide their system specific equivalent functions.
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/
/**
 * \brief Macro to test a condition. If condition evaluates to false, error
 * handling is invoked. Normal operation continues on an evaluation to
 * true.
 *
 * This macro can be set to nothing if no parameter checking is
 * desired.
 *
 * \param cond Condition to evaluate
 */
#define OS_ASSERT(cond) \
  assert((cond))

/**
 * \brief Macro to test a condition. If condition evaluates to false, an
 * error message is output and error handling is invoked. Normal
 * operation continues on an evaluation to true.
 *
 * This macro can be set to nothing if no parameter checking is
 * required.
 *
 * \param cond Condition to evaluate
 * \param fmt Output messaged formated like bracketed "printf". For example
 * \code
 *  DRV_ASSERTS( port < 5, ("Invalid port %d. Valid range is 0..5",x));
 * \endcode
 */
#define OS_ASSERTS(cond,fmt) \
  if (!(cond)) {              \
     __osFilename = __FILE__;  \
     __osFileline = __LINE__;  \
     osError_cygwin fmt;      \
  }

/**
 * \brief Macro to test if pointer is NULL. If pointer is NULL, error
 * handling is invoked. Normal operation continues when pointer is not
 * NULL.
 *
 * This macro can be set to nothing if no pointer checking is desired.
 *
 * \param ptr Pointer to evaluate for NULL
 */
#define OS_ASSERT_PTR_NULL(ptr) \
  assert((ptr) != NULL)

/** \brief Macro utility function called by driver when an unexpected
 * critical error has occurred.
 *
 * This macro <b>CAN NOT</b> be set to nothing since a critical error
 * has occurred and operation post-error is undefined.
 *
 * \param Error message formatted similarly like printf
 */
#define OS_ERROR(fmt)          \
   do {                         \
     __osFilename = __FILE__;  \
     __osFileline = __LINE__;  \
     osError_cygwin fmt;       \
   } while(0)

/** \brief Macro utility function to set output log level */
#define OS_SETLOGLVL(lvl) \
  osLoggerSetLvl_cygwin(lvl)

/** \brief Macro utility function to get output log level */
#define OS_GETLOGLVL() \
  osLoggerGetLvl_cygwin()

/**
 * \brief Macro function to provide logging capability to device driver
 *
 * \param lvl Log output level (1-highest level ... 9-lowest level)
 * \param fmt Log message formatted similarly like printf
 */
#define OS_LOGGER(lvl,fmt) \
  if (lvl <= __osLogLevel_g) osLoggerPrint_cygwin fmt

/**
 *\brief Macro function to trace function calls.
 *
 *\param fmt Log message formatted similarly like printf
 *
 *\note If the compiler does not support __FUNCTION__ construct,
 *         please remove __FUNCTION__ assignment code.
 */
#define OS_TRACE(fmt)					\
	do {								\
		__osFilename = (strrchr(__FILE__, '\\')) ? \
				(char *)(strrchr(__FILE__, '\\')+1) : __FILE__; \
		__osFileline = __LINE__;		\
		__osFuncname = __FUNCTION__;	\
		if (5 <= __osLogLevel_g)		\
			osTrace_cygwin fmt;		\
	} while(0)

/**
 * \brief Macro function to provide dynamic allocation for device driver
 * \param numBytes Number of bytes to allocate
 * \return Pointer to allocated memory
 */
#define OS_MALLOC(numBytes) \
  malloc(numBytes)

/**
 * \brief Macro function to free dynamically allocated memory.
 * \param ptr Pointer to allocated memory
 */
#define OS_FREE(ptr) \
  free(ptr)

/**
 * \brief Macro function to zero memory
 * \param ptr Pointer to block of memory to zero
 * \param size Size of block of memory in bytes
 */
#define OS_ZERO(ptr,size) \
    memset((void *)ptr,0,size)

/**
 * @brief Macro to perform a milisecond delay
 *
 * Added special case when we are compiling for Windows environment.
 *
 * @param msec Number of miliseconds to delay
 */
#if defined (_MSC_VER)
#   include <windows.h>
#   define OS_MSEC_DELAY(msec) Sleep(msec)
#else
#   include <unistd.h>
#   define OS_MSEC_DELAY(msec) sleep(msec/1000)
#endif

/**
 * @brief Macro to compare memory
 * @param ptrA pointer to memory to compare
 * @param ptrB pointer to memory to compare
 * @param size number of bytes to compare
 * @return equal=0, less than=-ve number, greater than=+ve number
 */
#define OS_MEMCMP(ptrA,ptrB,size) \
    memcmp((const void*)ptrA,(const void *)ptrB,(size_t)size)

/**
 * @brief Macro to get time string
 */
#define OS_CTIME() \
  osCtime_cygwin()

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/

extern int __osLogLevel_g;
extern char *__osFilename;
extern int __osFileline;
extern const char *__osFuncname;

void
osLoggerSetLvl_cygwin(int lvl);
int
osLoggerGetLvl_cygwin(void);
int
osLoggerPrint_cygwin(const char *fmt, ...);
int
osTrace_cygwin(const char *fmt, ...);
int
osError_cygwin(const char *fmt, ...);
char *
osCtime_cygwin(void);

#endif /* OSADAPT_H_ */
