/*********************************************************************
 * Copyright (c) 2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

#include "osAdapt.h"
#include "osNumeric.h"

/*********************************************************************
 * C O N S T A N T S    &     G L O B A L    V A R I A B L E S
 *********************************************************************/

int __osLogLevel_g = 9; /**< Variable to set highest log message output */
char *__osFilename;  /**< Variable to log file name **/
int __osFileline;  /**< Variable to log file line number */
const char *__osFuncname = NULL; /**< Variable to log function name */

/*********************************************************************
 * F U N C T I O N    I M P L E M E N T A T I O N
 *********************************************************************/

/**
 * \brief Function to set highest level of log messages to output
 *
 * Log trace messages priority 0-Highest ... 9-Lowest
 *
 * \param lvl Highest level of trace messages to output.
 */
void osLoggerSetLvl_linux(int lvl)
{
    if ((lvl >= 0) && (lvl <= 9))
        __osLogLevel_g = lvl;
}

/**
 * \brief Function to get highest level of log messages to output
 *
 * Log trace messages priority 0-Highest ... 9-Lowest
 *
 * \return Highest level of trace messages to output.
 */
int osLoggerGetLvl_linux(void)
{
    return __osLogLevel_g;
}

/**
 * \brief Function to output log trace message.
 *
 * Currently this interface provides a wrapper around printf but it
 * can be adapted for more complex systems (i.e. already present
 * logging system).
 *
 * \param fmt message formatted similarly like printf
 */
int osLoggerPrint_linux (const char *fmt, ...)
{
    va_list args;
    int rc;

    /* Initialize the argument list. */
    va_start (args, fmt);

    /* Pass to printf */
    rc = vprintf(fmt,args);

    /* Clean up. */
    va_end (args);

    return rc;
}

/**
 * \brief Function to output function trace message.
 *
 * Currently this interface provides a wrapper around printf but it
 * can be adapted for more complex systems (i.e. already present
 * logging system).
 *
 * \param fmt message formatted similarly like printf
 */
int osTrace_linux (const char *fmt, ...)
{
    va_list args;
    int rc;

    if (__osFuncname)
    {
        /* Print out file name, line and function name */
        printf("%s:%d:%s: ",__osFilename,__osFileline,__osFuncname);
    }
    else
    {
        /* Print out file name, line */
        printf("%s:%d: ",__osFilename,__osFileline);
    }

    /* Initialize the argument list. */
    va_start (args, fmt);

    /* Pass to printf */
    rc = vprintf(fmt,args);

    /* Clean up. */
    va_end (args);

    return rc;
}

/**
 * \brief Function to output error message.
 *
 * Currently this interface provides a wrapper around printf but it
 * can be adapted for more complex systems (i.e. already present
 * logging system).
 *
 * \param fmt message formatted similarly like printf
 */
int osError_linux (const char *fmt, ...)
{
    va_list args;
    int rc;

    /* Print out filename and line */
    printf("%s:%d ",__osFilename,__osFileline);

    /* Initialize the argument list. */
    va_start (args, fmt);

    /* Pass to printf */
    rc = vprintf(fmt,args);

    /* Clean up. */
    va_end (args);

    /* Abort */
    abort();
    return rc;
}

/**
 * \brief Function to get formatted time string.
 *
 */
char *
osCtime_linux(void)
{
    time_t currentTime;

    /* Get the current time */
    time(&currentTime);

    /* Convert to formatted string */
    return ctime(&currentTime);
}
