#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <map>
#include <vector>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

using namespace std;

/************************************************************************
 * T Y P E D E F S   &    D E F I N E S
 ************************************************************************/ 
typedef struct bfData_s{
  string         bfName_m;
  int            bfSize_m;
  int            bfLsb_m;
  int            line_m;   /* First occurrence of bitfield in file */
  vector<string> regNameVect_m;
} *bfData_t;

typedef struct regData_s {
  string           name_m;
  string           descr_m;
  int              line_m;   /* First occurrence of register in file */
  vector<bfData_t> bfVect_m;
} *regData_t;

/************************************************************************
 * G L O B A L    V A R I A B L E S
 ************************************************************************/ 
map<string, bfData_t> bitfieldMap_g;
map<int, regData_t> registerMap_g;
map<int, regData_t> registerPage1Map_g;

/************************************************************************
 * F U N C T I O N S
 ************************************************************************/ 
char *strupper(char *input)
{
  char *temp = input;
  while (*temp) { *temp = toupper(*temp); temp++; }
  return input;
}

void parseRow(xmlDocPtr doc, xmlNodePtr cur) {

  int idx;

  xmlChar *regAddrStr = NULL;
  xmlChar *regNameStr = NULL;
  xmlChar *regDescStr = NULL;

  xmlChar *bfNameStr = NULL;
  xmlChar *bfSizeStr = NULL;

  string regNameStrUpperCase;
  string bfNameStrUpperCase;
  unsigned long int regAddr;
  int bfSize;
  int bfLsb;
  int bfMask;

  regData_t currReg;

  /* Contents are as follows:
   *     [0] Register Offset Address
   *     [1] Register Name
   *     [2] Register Description
   *     [3] Register Category
   *     [4] Register Type
   *     [5] Bitfield Position 7
   *     [6] Bitfield Position 6
   *     [7] Bitfield Position 5
   *     [8] Bitfield Position 4
   *     [9] Bitfield Position 3
   *    [10] Bitfield Position 2
   *    [11] Bitfield Position 1
   *    [12] Bitfield Position 0
   */
  for (cur = cur->xmlChildrenNode, idx = 0;
       cur != NULL;
       cur = cur->next)
    {
      if ((!xmlStrcmp(cur->name, (const xmlChar *)"Cell"))) {

        xmlNodePtr dataPtr = cur->xmlChildrenNode;

        bfSizeStr = xmlGetProp(cur, (const xmlChar *)"MergeAcross");
        if (bfSizeStr) {	  
          bfSize = atoi((const char *)bfSizeStr)+1;
        }
        else {
          bfSize = 1;
        }

        if ((dataPtr) && ((!xmlStrcmp(dataPtr->name, 
                                      (const xmlChar *)"Data")))) {

          /* Address (hex) */
          if (idx==0) {
            string temp = "Address";

            regAddrStr = xmlNodeListGetString(doc, 
                                              dataPtr->xmlChildrenNode, 1);

            /* Special case of header row, we'll skip it */
            if (!temp.compare((const char *)regAddrStr))
              {
                xmlFree(regAddrStr);
                return;
              }
            regAddr = strtol((const char *)regAddrStr,NULL,16);
            xmlFree(regAddrStr);
          }

          /* Register name (string) */
          if (idx==1) {
            regNameStr = xmlNodeListGetString(doc, 
                                              dataPtr->xmlChildrenNode, 1);
            regNameStrUpperCase = strupper((char *)regNameStr);
            xmlFree(regNameStr);
          }

          /* Register description (string) */
          if (idx==2) {
            regDescStr = xmlNodeListGetString(doc, 
                                              dataPtr->xmlChildrenNode, 1);

            /* Add register to map */
            currReg = (regData_t)new struct regData_s[1];
            currReg->name_m = regNameStrUpperCase;
            currReg->descr_m = (char*)regDescStr;
            currReg->line_m = dataPtr->line;
            if (registerMap_g.find((int)regAddr) == registerMap_g.end())
              registerMap_g[(int)regAddr] = currReg;
            else {
              /* Duplicate, page 1 register - it's kinda of a hack at
                 the momment. We'll revisit this as devices come
                 online. */
              registerPage1Map_g[(int)regAddr] = currReg;
            }

            xmlFree(regDescStr);
          }

          /* Bitfield names (string) */
          if ((idx>=5)&&(idx<=12)) {
            bfNameStr = xmlNodeListGetString(doc, dataPtr->xmlChildrenNode, 1);
            bfNameStrUpperCase = strupper((char *)bfNameStr);
            xmlFree(bfNameStr);

            /* Calculate LSB and bit mask */
            bfLsb = 8-(idx-5)-bfSize;
            bfMask = (0xFF>>(8-bfSize))<<bfLsb;

            /* Check for duplicate bitfield */
            map<string, bfData_t>::iterator bitfieldMapIter;
            bfData_t temp;

            bitfieldMapIter = bitfieldMap_g.find(bfNameStrUpperCase);

            if (bitfieldMapIter != bitfieldMap_g.end()) {
              temp = bitfieldMapIter->second;

              if ((temp->bfSize_m != bfSize) || 
                  (temp->bfLsb_m != bfLsb)) {		
                /* Same name but different size/lsb */
                /*    Report an error */
                cerr << "Duplicate bitfield found " 
                     << bfNameStrUpperCase << " line numbers: "
                     << temp->line_m << " & "
                     << dataPtr->line << endl;
              }
              else {

                /* Same name and bitfield data. Don't output data (we're
                   already done it). */

                /* Add register to list of owners for this bitfield */
                bitfieldMap_g[bfNameStrUpperCase]->regNameVect_m.
                  push_back(regNameStrUpperCase);

                /* Save bitfield to register map */
                currReg->bfVect_m.push_back(temp);
              }
            }
            else {

              /* Bitfield not found. Save bitfield data */
              temp = (bfData_t)new struct bfData_s[1];
              temp->bfName_m = bfNameStrUpperCase;
              temp->bfSize_m = bfSize;
              temp->bfLsb_m = bfLsb;
              temp->line_m = dataPtr->line;
              temp->regNameVect_m.push_back(regNameStrUpperCase);

              /* Save bitfield in unique bitfield map */
              bitfieldMap_g[bfNameStrUpperCase] = temp;

              /* Save bitfield to register map */
              currReg->bfVect_m.push_back(temp);
            }	
          }
        }
        idx += bfSize;
      }
    }
}

void parseWorksheet (xmlDocPtr doc, xmlNodePtr cur) 
{

  int flag = 0;

  /* Find table element */
  cur = cur->xmlChildrenNode;
  while (cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"Table"))) {
      flag = 1;
      break;
    }
    cur = cur->next;
  }
  if (!flag)
    {
      cerr << "Table element not found" << endl;
      return ;
    }

  /* Find row elements */
  cur = cur->xmlChildrenNode;
  while (cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"Row"))){
      parseRow (doc, cur);
    }		 
    cur = cur->next;
  }

  return;
}

static void parseDoc(char *docname) {

  xmlDocPtr doc;
  xmlNodePtr cur;
  int flag =0;

  doc = xmlParseFile(docname);
	
  if (doc == NULL ) {
    cerr << "XML document not parsed successfully." << endl;
    return;
  }
	
  cur = xmlDocGetRootElement(doc);
	
  if (cur == NULL) {
    cerr << "Empty XML document" << endl;
    xmlFreeDoc(doc);
    return;
  }
	
  if (xmlStrcmp(cur->name, (const xmlChar *) "Workbook")) {
    cerr << "XML document of the wrong type, root node "
         << "!= Workbook" << endl;
    xmlFreeDoc(doc);
    return;
  }
	
  cur = cur->xmlChildrenNode;
  while (cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"Worksheet"))){
      flag = 1;
      parseWorksheet (doc, cur);
    }
		 
    cur = cur->next;
  }
	
  if (!flag)
    cerr << "Worksheet element not found\n";

  xmlFreeDoc(doc);
}

void outputDrvHeader(ostringstream &outStream)
{
  outStream <<
    "/*********************************************************************\n"
    " * Copyright (c) 2012-2013 Integrated Device Technology, Inc.\n"
    " * All Rights Reserved\n"
    " *\n"
    " * This file is provided for use with IDT products only, and does not\n"
    " * convey any license, whether express or implied, to any intellectual\n"
    " * property rights of Integrated Device Technology, Inc.\n"
    " * Distribution, copying or modification of the contents of this file\n"
    " * and/or any derivatives thereto is not permitted, except as\n"
    " * specifically authorized in writing. This notice shall be included\n"
    " * in its entirety on all copies made from this file.\n"
    " *\n"
    " * The contents herein are the proprietary and confidential\n"
    " * information of Integrated Device Technology Inc. and shall not be\n"
    " * distributed or furnished to third parties or made public without\n"
    " * prior written permission by Integrated Device Technology Inc.\n"
    " * Disclosures of this file in any form to the United States\n"
    " * Government may not be made without a written predetermination that\n"
    " * it will be protected by limited or restricted rights as set forth\n"
    " * in the applicable Federal Acquisition Regulations or other\n"
    " * equivalent governmental rule or regulation.\n"
    " *\n"
    " * The contents herein are provided \"as is\" and without any express or\n"
    " * implied warranties, including, without limitation, the implied\n"
    " * warranties of merchantability and fitness for a particular purpose.\n"
    " *\n"
    " *********************************************************************/\n"
    "\n"
    "#ifndef __PUMA_CMN_REGISTER_DEF_H__\n"
    "#define __PUMA_CMN_REGISTER_DEF_H__\n"
    "\n"
    "#ifdef __cplusplus\n"
    "extern \"C\"\n"
    "{\n"
    "#endif\n"
    "/*! \\file RegisterDef.h\n"
    " *  @brief This header file contains register and bitfield information for\n"
    " *  accessing device.\n"
    " *  \\see Datasheet for detailed register descriptions.\n"
    " */\n"
    "\n"
    "/** \\defgroup RegistersDef Register definitions\n"
    " *\n"
    " * @{ */\n"
    "\n\n"
    "/**\n"
    " * @brief Enumerated type listing register offsers and bit field\n"
    " * constants.\n"
    " */\n"
    "enum {\n\n";
}

void outputDrvFooter(ostringstream &outStream)
{
  outStream <<
    "};\n\n"
    "\n/** @}*/\n"
    "\n#ifdef __cplusplus\n"
    "}\n"
    "#endif /* __cplusplus */\n"
    "#endif /* __PUMA_CMN_REGISTER_DEF_H__ */\n";
}

void outputDrvCode(ostringstream &outStream)
{
  map<int, regData_t>::iterator regMapIter;
  regData_t regTemp;

  outputDrvHeader(outStream);

  outStream << "/*======================Register Definitions"
            << "==========================*/" << endl;
  for (regMapIter = registerMap_g.begin();
       regMapIter != registerMap_g.end();
       regMapIter++) {

    int regAddr = ((int)(regMapIter->first));
    regTemp = ((regData_t)(regMapIter->second));

    outStream << "    REG_" << regTemp->name_m
              << " = 0x"
              << std::setw(2) << setfill('0') << uppercase << hex << regAddr
              << ","
              << " /**< " << regTemp->descr_m << " */" << endl;
  }
  outStream << endl 
            << "/* Page 1 Registers */" << endl;
  for (regMapIter = registerPage1Map_g.begin();
       regMapIter != registerPage1Map_g.end();
       regMapIter++) {

    int regAddr = ((int)(regMapIter->first));
    regTemp = ((regData_t)(regMapIter->second));

    outStream << "    REG_" << regTemp->name_m
              << " = 0x"
              << std::setw(2) << setfill('0') << uppercase << hex << regAddr
              << ","
              << " /**< " << regTemp->descr_m << " */" << endl;
  }
  outStream << "/*==================End of Register Definitions"
            << "=======================*/" << endl << endl;

  outStream << "/*======================Bitfield Definitions"
    "==========================*/" << endl;
  for (regMapIter = registerMap_g.begin();
       regMapIter != registerMap_g.end();
       regMapIter++) {
    regTemp = ((regData_t)(regMapIter->second));
    vector<bfData_t>::iterator bfIter;
    for (bfIter = regTemp->bfVect_m.begin();
         bfIter < regTemp->bfVect_m.end();
         bfIter++) {
      bfData_t temp = ((bfData_t)(*bfIter));

      /* If we're already displayed this bitfield, skip to next */
      if (temp->regNameVect_m[0] != regTemp->name_m) 
        continue;

      /* Print out list of register using this bitfield */
      outStream << "    /**" << endl;
      vector<string>::iterator parentIter;
      for (parentIter = temp->regNameVect_m.begin();
           parentIter < temp->regNameVect_m.end();
           parentIter++) {
        outStream << "     * " << *parentIter << endl;
      }
      outStream << "     */" << endl;

      /* Print out bitfield information */
      outStream << "    BF_" << temp->bfName_m
                << "_SIZE = " << temp->bfSize_m << "," << endl;

      outStream << "    BF_" << temp->bfName_m
                << "_MASK = 0x" 
                << setw(2) << setfill('0') << uppercase << hex 
                << ((0xFF>>(8-temp->bfSize_m))<<temp->bfLsb_m)
                << "," << endl;

      outStream << "    BF_" << temp->bfName_m
                << "_LSB = " << temp->bfLsb_m
                << ","
                << endl << endl;
    }
  
  }
  outStream << endl 
            << "/* Page 1 releated bitfields */" << endl;
  for (regMapIter = registerPage1Map_g.begin();
       regMapIter != registerPage1Map_g.end();
       regMapIter++) {
    regTemp = ((regData_t)(regMapIter->second));
    vector<bfData_t>::iterator bfIter;
    for (bfIter = regTemp->bfVect_m.begin();
         bfIter < regTemp->bfVect_m.end();
         bfIter++) {
      bfData_t temp = ((bfData_t)(*bfIter));

      /* If we're already displayed this bitfield, skip to next */
      if (temp->regNameVect_m[0] != regTemp->name_m) 
        continue;

      /* Print out list of register using this bitfield */
      outStream << "    /**" << endl;
      vector<string>::iterator parentIter;
      for (parentIter = temp->regNameVect_m.begin();
           parentIter < temp->regNameVect_m.end();
           parentIter++) {
        outStream << "     * " << *parentIter << endl;
      }
      outStream << "     */" << endl;

      /* Print out bitfield information */
      outStream << "    BF_" << temp->bfName_m
                << "_SIZE = " << temp->bfSize_m << "," << endl;

      outStream << "    BF_" << temp->bfName_m
                << "_MASK = 0x" 
                << setw(2) << setfill('0') << uppercase << hex 
                << ((0xFF>>(8-temp->bfSize_m))<<temp->bfLsb_m)
                << "," << endl;

      outStream << "    BF_" << temp->bfName_m
                << "_LSB = " << temp->bfLsb_m
                << ","
                << endl << endl;
     }
  }
  outStream << "/*==================End of Bitfield Definitions"
            << "=======================*/" << endl << endl;
  outStream << "    REGMAP_MAX" << endl;
  outputDrvFooter(outStream);
}

static void outputBitfieldData(std::map<int, regData_t> &registerMap)
{
  map<int, regData_t>::iterator regMapIter;
  regData_t                     regTemp;
  int                           i;

  cout << endl << endl << "/* Register bitfield data */" << endl << endl;
  /* Output bitfield data arrays (one per register) */  
  for (regMapIter = registerMap.begin();
       regMapIter != registerMap.end();
       regMapIter++) {
    regTemp = ((regData_t)(regMapIter->second));

    cout << "const BitfieldName_t " 
         << regTemp->name_m << "_Bitfields[] = {" << endl;

    vector<bfData_t>::iterator bfIter;
    for (bfIter = regTemp->bfVect_m.begin(), i=0;
         bfIter < regTemp->bfVect_m.end();
         bfIter++, i++) {

      bfData_t temp = ((bfData_t)(*bfIter));

      cout << "   { \"" << temp->bfName_m 
           << "\", "
           << temp->bfLsb_m
           << ", 0x"
           << setw(2) << setfill('0') << uppercase << hex 
           << ((0xFF>>(8-temp->bfSize_m))<<temp->bfLsb_m)
           << ", "
           << temp->bfSize_m
           << " }";

      /* Add comma after entry (except for last entry) */
      if ((regTemp->bfVect_m.size() > 1) && 
          (i<(int)(regTemp->bfVect_m.size()-1)))
        cout << ",";

      cout << endl;
    }
    cout << "};" << endl;
  }
  cout << endl;
}

static void outputRegisterData(std::map<int, regData_t> &registerMap)
{
  map<int, regData_t>::iterator regMapIter;
  regData_t					  regTemp;
  int 						  i;

  cout << endl << endl << "/* Register data */" << endl << endl;
  cout << "const RegisterName_t RegisterNameArray[] = {" << endl;
  for (regMapIter = registerMap.begin(), i=0;
       regMapIter != registerMap.end();
       regMapIter++, i++) {
	regTemp = ((regData_t)(regMapIter->second));
  
	cout << "   REGNAME_ENTRY(" 
		 << regTemp->name_m << ", "
		 << regTemp->bfVect_m.size() << ")";
  
	/* Add comma after entry (except for last entry) */
	if ((registerMap.size()> 1) && 
		(i<(int)(registerMap.size()-1)))
      cout << ",";
  
	  cout << endl;
	}
	cout << "};" << endl << endl;
  
	cout << "const int RegisterNameArraySize = (sizeof(RegisterNameArray)/" 
		 << endl
		 << "									sizeof(RegisterName_t));"
		 << endl;
}

void outputAnnotateCode(void)
{
  /* It is not to most space efficient data. Since annotator is a
     program this is off system (i.e. not meant to be part of an
     embedded system), it doesn't have to be space efficient. */

  /* Output page 0 & page 1 bitfield data arrays */
  outputBitfieldData(registerMap_g);
  outputBitfieldData(registerPage1Map_g);
 
  /* Output register array */
  outputRegisterData(registerMap_g);
  outputRegisterData(registerPage1Map_g);
}

int main(int argc, char **argv) {

  char *docname;
  ostringstream outStream;

  if (argc <= 1) {
    cout << "Usage: "<< argv[0] <<" [EXCEL XML DOC]" << endl;
    return(0);
  }

  docname = argv[1];

  /* Initialize maps */
  bitfieldMap_g.clear();
  registerMap_g.clear();

  parseDoc (docname);

  outputDrvCode(outStream);
  cout << outStream.str();
  /*outputAnnotateCode();*/

  return (1);
}
