#include <stdio.h>
#include <getopt.h>

#include "idt82V3911.h"

#include "ReadWrite_sim.h"

#include "save.h"
#include "load.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/**
 * @brief Structure to define program command line arguments
 */
typedef struct {
    T_osUint8 isSim_m;             /**< Switch to similated mode */
    T_osUint8 i2cAddr_m;           /**< I2C address */
    T_osUint8 isVersion1_m;        /**< Version 1 or 2 rgf file format */
    T_osUint8 isOperSave_m;        /**< Save / Load Operation */
    char      *filename_m;         /**< Filename to write to /read from */
} globalArgs_t;

#define no_argument       0
#define required_argument 1
#define optional_argument 2

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
/** Detailed command line options paramters */
static const struct option longOpts[] = {
    { "help",   no_argument,       NULL, 'h' },
    { "i2c",    required_argument, NULL, 'e' },
    { "sim",    no_argument,       NULL, 'm' },
    { "ver2",   no_argument,       NULL, '2' },
    { "ver1",   no_argument,       NULL, '1' },
    { "save",   no_argument,       NULL, 's' },
    { "load",   no_argument,       NULL, 'l' },
    { "file",   required_argument, NULL, 'f' },
    { NULL,     no_argument,       NULL,  0 }
};

/** Command line option string */
static const char *optString = "em?h";

const T_idtDrvAccess accessMethods[] = {
    { Init_Sim,
      DeInit_Sim,
      RDByte_Sim,
      WRByte_Sim,
      NULL }, /* I2C - user implementation required*/
    { Init_Sim,
      DeInit_Sim,
      RDByte_Sim,
      WRByte_Sim,
      NULL } /* Simulated */
};

static FILE *filePtr;  /* Global file pointer */

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
void printHelp(void)
{
    printf(
           "\nUsage:  rgf [OPTION]\n\n"
           "82V3911 Device register load/save application\n"
           "\n"
           "-?                      show this page\n"
           "--load\n"
           "--save\n"
           "--ver1\n"
           "--ver2\n"
           "--file [FILE NAME]\n"
           "--i2c [I2C ADDRESS]     Device is connected through I2C interface\n"
           "--sim                   Device is in simulation mode\n"
           "\n");
}

int processCmdLineArguments(int argc, char *argv[], globalArgs_t *gblArgs)
{
    int opt;
    int longIndex;

    /* Initialize global arguments variable */
    gblArgs->isSim_m = 1;
    gblArgs->isVersion1_m = 0; /* default to version 2 format */
    gblArgs->isOperSave_m = 1;
    gblArgs->filename_m = NULL;

    if (argc <= 1)
    {
        printHelp();
        exit(0);
    }
 
    /* parse command line arguments */
    opt = getopt_long( argc, argv, optString, longOpts, &longIndex );
    while( opt != -1 ) {
        switch (opt) {

            /* Help */
            case 'h':
            case '?':
                printHelp();
                exit(0);
            break;

            /* I2C Access */
            case 'e':
                gblArgs->isSim_m = 0;
                gblArgs->i2cAddr_m = strtol(optarg, NULL, 0);
            break;

            /* Simulated Access */
            case 'm':
                gblArgs->isSim_m = 1;
            break;

            /* Version 1 */
            case '1':
                gblArgs->isVersion1_m = 1;
            break;

            /* Version 2 */
            case '2':
                gblArgs->isVersion1_m = 0;
            break;

            /* Save Operation */
            case 's':
                gblArgs->isOperSave_m = 1;
            break;

            /* Load Operation */
            case 'l':
                gblArgs->isOperSave_m = 0;
            break;

            /* File name */
            case 'f':
                gblArgs->filename_m = optarg;
            break;

            default:
            /* Shouldn't be here. */
            break;
        }
        
        opt = getopt_long( argc, argv, optString, longOpts, &longIndex );
    }

    return 0;
}

/**
 * @brief Wrapper read function to read from a file.
 * @param s String variable save to 
 * @param size Maximum size of string to save to.
 * @return pointer to string, null on error
 */
char *ReadFromFile(char *s, int size)
{
    char * rc;

    rc = fgets(s,size,filePtr);
    if (feof(filePtr) || (strlen(s)==0))
        return NULL;
    return rc;
}

/**
 * @brief Wrapper read function to read from a file.
 * @param s String variable save to 
 * @param size Maximum size of string to save to.
 * @return pointer to string, null on error
 */
char *ReadFromStdio(char *s, int size)
{
    scanf("%s",s);
    return s;
}

/**
 * @brief Wrapper write function to output to a file
 * @param format Printf format
 * @param ... Variable arguments 
 * @return number of character written
 */
int WriteToFile(const char *format, ...)
{
    va_list fmtargs;
    int rc;

    va_start(fmtargs,format);
    rc = vfprintf(filePtr,format,fmtargs);
    va_end(fmtargs);

    return rc;
}

int main (int argc, char *argv[])
{
    T_idtDrvHdlr hdlr;
    globalArgs_t globalArgs;

    /* Process command line options */
    processCmdLineArguments(argc,argv,&globalArgs);

    OS_SETLOGLVL(0);
    /* We just need to init driver to load/save
     * register values, and don't really care about   
     * configuring APLL xtals, so just leave them 
     * as NULL.
     */
    if (globalArgs.isSim_m)
        hdlr = IDTGeneral_InitDriver("dev3910", 
                                     accessMethods[1],
                                     IDT_82V3911,
                                     NULL, /* APLL xtal list */
                                     0, /* number of APLL xtals */
                                     0, /* I2C address - don't care */
                                     1);
    else
        hdlr = IDTGeneral_InitDriver("dev3910", 
                                     accessMethods[0], 
                                     IDT_82V3911,
                                     NULL, /* APLL xtal list */
                                     0, /* number of APLL xtals */
                                     globalArgs.i2cAddr_m,
                                     1);

    /* Depends on whether we've specified a file name or not */
    if (globalArgs.filename_m != NULL)
    {
        /* Open file for reading / writing */
        if (globalArgs.isOperSave_m)
            filePtr = fopen(globalArgs.filename_m,"w+t");
        else
            filePtr = fopen(globalArgs.filename_m,"r+t");

        /* Unable to open file ... */
        if (!filePtr) 
        {
            printf("Error unable ot open file: %s\n",
                   globalArgs.filename_m);
            /* Deinitialize driver */
            IDTGeneral_DeInitDriver(hdlr);
            return -1;
        }
    }

    if (globalArgs.isOperSave_m)
    {
        if (globalArgs.filename_m == NULL)
        {
            if (globalArgs.isVersion1_m)
                saveVersion1(printf, hdlr);
            else
                saveVersion2(printf, hdlr);
        }
        else
        {
            if (globalArgs.isVersion1_m)
                saveVersion1(WriteToFile, hdlr);
            else
                saveVersion2(WriteToFile, hdlr);
        }
    }
    else
    {
        if (globalArgs.filename_m == NULL)
        {
            if (globalArgs.isVersion1_m)
                loadVersion1(ReadFromStdio, hdlr);
            else
                loadVersion2(ReadFromStdio, hdlr);
        }
        else
        {
            if (globalArgs.isVersion1_m)
                loadVersion1(ReadFromFile, hdlr);
            else
                loadVersion2(ReadFromFile, hdlr);
        }
    }

    if (globalArgs.filename_m != NULL)
    {
        fclose(filePtr);
    }

    /* Deinitialize driver */
    IDTGeneral_DeInitDriver(hdlr);

    return 0;
}
